/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include <math.h>
#include "rsdsa.h"

#pragma warning( disable : 4152 )

 /// HIFN  Checks the range of a ViInt32 value against a specified range table.
 /// HIRET Returns the status code of this operation.
 /// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
 /// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
 /// HIPAR instrSession/instrument session.
 /// HIPAR rangeTable/RsCoreRangeTableRangedPtr to the range table.
 /// HIPAR value/The value to range check.
ViStatus rsdsa_CheckRangeTableRangeViInt32(ViSession                 instrSession,
                                           RsCoreRangeTableRangedPtr rangeTable,
                                           ViInt32                   value)
{
    return RsCore_InvalidViInt32Range(instrSession, value,
        (ViInt32)(rangeTable->min), (ViInt32)(rangeTable->max));
}

/// HIFN  Checks the range of a ViReal64 value against a specified range table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rangeTable/RsCoreRangeTableRangedPtr to the range table.
/// HIPAR value/The value to range check.
ViStatus rsdsa_CheckRangeTableRangeViReal64(ViSession                 instrSession,
                                            RsCoreRangeTableRangedPtr rangeTable,
                                            ViReal64                  value)
{
    return RsCore_InvalidViReal64Range(instrSession, value,
        rangeTable->min, rangeTable->max);
}


/******************************************************************************************************
    rsdsa_InitAttributes
    This function inits attributes to the desired values if needed.
 *****************************************************************************************************/
ViStatus rsdsa_InitAttributes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSDSA_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSDSA_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSDSA_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSDSA_SPECIFIC_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSDSA_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSDSA_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSDSA_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSDSA_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSDSA_FUNCTION_CAPABILITIES));

    // - Version Info -
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSDSA_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSDSA_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSDSA_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSDSA_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSDSA_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/******************************************************************************************************
    rsdsa_DefaultInstrSetup
    This function sends a default setup to the instrument. The
    rsdsa_reset function calls this function. The
    rsdsa_RsInit function calls this function when the
    user passes VI_FALSE for the reset parameter. This function is
    useful for configuring settings that other instrument driver
    functions require.

    Note:  Call this function only when the session is locked.
    *****************************************************************************************************/
ViStatus rsdsa_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rsdsa_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));

Error:
    return error;
}
