/****************************************************************************
 *
 * Rohde&Schwarz DOCSIS Signal Analyzer instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSDSA_HEADER
#define __RSDSA_HEADER

#include "rscore.h"
#include "rsdsa_attributes.h"
#include "rsdsa_internal.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSDSA_SIMULATION_ID_QUERY      "Rohde&Schwarz,DOCSIS Signal Analyzer,101210,v3.3.0-4d612ff"  /* Default definition of ID Query Response for simulation */
#define RSDSA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSDSA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */
#define RSDSA_OPC_TIMEOUT              30000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSDSA_CLASS_DRIVER_DESCRIPTION           ""
#define RSDSA_CLASS_DRIVER_PREFIX                ""
#define RSDSA_CLASS_DRIVER_VENDOR                ""
#define RSDSA_CLASS_DRIVER_REVISION              ""
#define RSDSA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSDSA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSDSA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSDSA_SPECIFIC_DRIVER_PREFIX             "RSDSA"
#define RSDSA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSDSA_SPECIFIC_DRIVER_VENDOR             ""
#define RSDSA_MAJOR_VERSION                      3L    /* Instrument driver major version          */
#define RSDSA_MINOR_VERSION                      4L   /* Instrument driver minor version          */
#define RSDSA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSDSA_SUPPORTED_INSTRUMENT_MODELS        "DOCSIS Signal Analyzer"    /* Instrument driver supported model(s)     */
#define RSDSA_GROUP_CAPABILITIES                 ""
#define RSDSA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSDSA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSDSA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSDSA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSDSA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSDSA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSDSA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSDSA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSDSA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSDSA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSDSA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSDSA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSDSA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSDSA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSDSA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSDSA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSDSA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSDSA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSDSA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSDSA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSDSA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSDSA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSDSA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSDSA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSDSA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSDSA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSDSA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

/* BEGIN GENERATE */
#define RSDSA_ATTR_ENABLE_EURODOCSIS30_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1553
#define RSDSA_ATTR_ED30_QUERY_DEMODULATOR_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1554
#define RSDSA_ATTR_ED30_QUERY_DECODER_LOCK_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1555
#define RSDSA_ATTR_ED30_QUERY_MPEG_LOCK_STATUS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1556
#define RSDSA_ATTR_ED30_QUERY_SIGNAL_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1557
#define RSDSA_ATTR_ED30_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1558
#define RSDSA_ATTR_ED30_QUERY_SYMBOL_CLOCK_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1559
#define RSDSA_ATTR_ED30_QUERY_CHANNEL_BIT_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1560
#define RSDSA_ATTR_ED30_QUERY_INTERLEAVER                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1561
#define RSDSA_ATTR_ED30_CONFIGURE_MER_COMPENSATION_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1562
#define RSDSA_ATTR_ED30_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1563
#define RSDSA_ATTR_ED30_QUERY_MER_RMS_COMPENSATED_OVERALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1564
#define RSDSA_ATTR_ED30_QUERY_MER_RMS_EQUALIZED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1565
#define RSDSA_ATTR_ED30_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1566
#define RSDSA_ATTR_ED30_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1567
#define RSDSA_ATTR_ED30_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1568
#define RSDSA_ATTR_ED30_QUERY_BER_BLOCKS_PROCESSED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1569
#define RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1570
#define RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1571
#define RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1572
#define RSDSA_ATTR_ED30_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1573
#define RSDSA_ATTR_ED30_QUERY_BER_PRE_REED_SOLOMON                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1574
#define RSDSA_ATTR_ED30_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1575
#define RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1576
#define RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_TOTAL_BITS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1577
#define RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1578
#define RSDSA_ATTR_ED30_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1579
#define RSDSA_ATTR_ED30_QUERY_CER_CORRECTED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1580
#define RSDSA_ATTR_ED30_QUERY_CER_UNCORRECTED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1581
#define RSDSA_ATTR_ED30_QUERY_CER_TOTAL_CODEWORDS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1582
#define RSDSA_ATTR_ED30_QUERY_CER_CORRECTED_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1583
#define RSDSA_ATTR_ED30_QUERY_CER_UNCORRECTED_CODEWORDS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1584
#define RSDSA_ATTR_ED30_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1585
#define RSDSA_ATTR_ED30_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1586
#define RSDSA_ATTR_ED30_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1587
#define RSDSA_ATTR_ED30_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1588
#define RSDSA_ATTR_ED30_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1591
#define RSDSA_ATTR_ED30_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1592
#define RSDSA_ATTR_ED30_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1593
#define RSDSA_ATTR_ED30_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1595
#define RSDSA_ATTR_ED30_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1596
#define RSDSA_ATTR_ED30_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1597
#define RSDSA_ATTR_ED30_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1598
#define RSDSA_ATTR_ED30_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1599
#define RSDSA_ATTR_ED30_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1600
#define RSDSA_ATTR_ED30_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1601
#define RSDSA_ATTR_ED30_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1602
#define RSDSA_ATTR_ED30_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1603
#define RSDSA_ATTR_ED30_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1604
#define RSDSA_ATTR_ED30_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1605
#define RSDSA_ATTR_ED30_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1606
#define RSDSA_ATTR_ED30_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1607
#define RSDSA_ATTR_ED30_QUERY_TRACE_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1608
#define RSDSA_ATTR_SYSTEM_ACQUISITION_LOG_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1609
#define RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1610
#define RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1611
#define RSDSA_ATTR_ED30_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1612
#define RSDSA_ATTR_ED30_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1613
#define RSDSA_ATTR_ED30_CONFIGURE_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1614
#define RSDSA_ATTR_ED30_CONFIGURE_SPECTRAL_INVERSION_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1615
#define RSDSA_ATTR_ED30_CONFIGURE_EQUALIZER_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1616
#define RSDSA_ATTR_ED30_QUERY_EQUALIZER_COEFFICIENTS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1617
#define RSDSA_ATTR_ED30_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1618
#define RSDSA_ATTR_ED30_CONFIGURE_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1619
#define RSDSA_ATTR_ED30_CONFIGURE_PREAMPLIFIER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1620
#define RSDSA_ATTR_ED30_CONFIGURE_ASI_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1621
#define RSDSA_ATTR_ED30_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1622
#define RSDSA_ATTR_ED30_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1623
#define RSDSA_ATTR_ED30_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1624
#define RSDSA_ATTR_ED30_ALARM_THRESHOLD_BER_POST_REED_SOLOMON               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1625
#define RSDSA_ATTR_ENABLE_DOCSIS30_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1626
#define RSDSA_ATTR_D30D_QUERY_DEMODULATOR_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1627
#define RSDSA_ATTR_D30D_QUERY_DECODER_LOCK_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1628
#define RSDSA_ATTR_D30D_QUERY_MPEG_LOCK_STATUS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1629
#define RSDSA_ATTR_D30D_QUERY_SIGNAL_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1630
#define RSDSA_ATTR_D30D_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1631
#define RSDSA_ATTR_D30D_QUERY_SYMBOL_CLOCK_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1632
#define RSDSA_ATTR_D30D_QUERY_CHANNEL_BIT_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1633
#define RSDSA_ATTR_D30D_QUERY_INTERLEAVER                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1634
#define RSDSA_ATTR_D30D_CONFIGURE_MER_COMPENSATION_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1635
#define RSDSA_ATTR_D30D_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1636
#define RSDSA_ATTR_D30D_QUERY_MER_RMS_COMPENSATED_OVERALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1637
#define RSDSA_ATTR_D30D_QUERY_MER_RMS_EQUALIZED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1638
#define RSDSA_ATTR_D30D_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1639
#define RSDSA_ATTR_D30D_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1640
#define RSDSA_ATTR_D30D_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1641
#define RSDSA_ATTR_D30D_QUERY_BER_BLOCKS_PROCESSED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1642
#define RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1643
#define RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1644
#define RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1645
#define RSDSA_ATTR_D30D_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1646
#define RSDSA_ATTR_D30D_QUERY_BER_PRE_VITERBI                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1647
#define RSDSA_ATTR_D30D_QUERY_BER_PRE_VITERBI_TOTAL_BITS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1648
#define RSDSA_ATTR_D30D_QUERY_BER_POST_VITERBI                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1649
#define RSDSA_ATTR_D30D_QUERY_BER_POST_VITERBI_TOTAL_BITS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1650
#define RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1651
#define RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1652
#define RSDSA_ATTR_D30D_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1653
#define RSDSA_ATTR_D30D_QUERY_CER_CORRECTED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1654
#define RSDSA_ATTR_D30D_QUERY_CER_UNCORRECTED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1655
#define RSDSA_ATTR_D30D_QUERY_CER_TOTAL_CODEWORDS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1656
#define RSDSA_ATTR_D30D_QUERY_CER_CORRECTED_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1657
#define RSDSA_ATTR_D30D_QUERY_CER_UNCORRECTED_CODEWORDS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1658
#define RSDSA_ATTR_D30D_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1659
#define RSDSA_ATTR_D30D_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1660
#define RSDSA_ATTR_D30D_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1661
#define RSDSA_ATTR_D30D_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1662
#define RSDSA_ATTR_D30D_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1665
#define RSDSA_ATTR_D30D_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1666
#define RSDSA_ATTR_D30D_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1667
#define RSDSA_ATTR_D30D_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1669
#define RSDSA_ATTR_D30D_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1670
#define RSDSA_ATTR_D30D_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1671
#define RSDSA_ATTR_D30D_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1672
#define RSDSA_ATTR_D30D_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1673
#define RSDSA_ATTR_D30D_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1674
#define RSDSA_ATTR_D30D_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1675
#define RSDSA_ATTR_D30D_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1676
#define RSDSA_ATTR_D30D_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1677
#define RSDSA_ATTR_D30D_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1678
#define RSDSA_ATTR_D30D_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1679
#define RSDSA_ATTR_D30D_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1680
#define RSDSA_ATTR_D30D_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1681
#define RSDSA_ATTR_D30D_QUERY_TRACE_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1682
#define RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1683
#define RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1684
#define RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1685
#define RSDSA_ATTR_D30D_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1686
#define RSDSA_ATTR_D30D_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1687
#define RSDSA_ATTR_D30D_CONFIGURE_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1688
#define RSDSA_ATTR_D30D_CONFIGURE_SPECTRAL_INVERSION_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1689
#define RSDSA_ATTR_D30D_CONFIGURE_EQUALIZER_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1690
#define RSDSA_ATTR_D30D_QUERY_EQUALIZER_COEFFICIENTS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1691
#define RSDSA_ATTR_D30D_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1692
#define RSDSA_ATTR_D30D_CONFIGURE_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1693
#define RSDSA_ATTR_D30D_CONFIGURE_PREAMPLIFIER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1694
#define RSDSA_ATTR_D30D_CONFIGURE_ASI_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1695
#define RSDSA_ATTR_D30D_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1696
#define RSDSA_ATTR_D30D_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1697
#define RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_PRE_VITERBI                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1698
#define RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_POST_VITERBI                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1699
#define RSDSA_ATTR_D30D_ALARM_THRESHOLD_BER_POST_REED_SOLOMON               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1700
#define RSDSA_ATTR_ENABLE_SC_QAM_J83C_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1701
#define RSDSA_ATTR_D30J_QUERY_DEMODULATOR_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1702
#define RSDSA_ATTR_D30J_QUERY_DECODER_LOCK_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1703
#define RSDSA_ATTR_D30J_QUERY_MPEG_LOCK_STATUS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1704
#define RSDSA_ATTR_D30J_QUERY_SIGNAL_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1705
#define RSDSA_ATTR_D30J_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1706
#define RSDSA_ATTR_D30J_QUERY_SYMBOL_CLOCK_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1707
#define RSDSA_ATTR_D30J_QUERY_CHANNEL_BIT_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1708
#define RSDSA_ATTR_D30J_QUERY_INTERLEAVER                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1709
#define RSDSA_ATTR_D30J_CONFIGURE_MER_COMPENSATION_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1710
#define RSDSA_ATTR_D30J_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1711
#define RSDSA_ATTR_D30J_QUERY_MER_RMS_COMPENSATED_OVERALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1712
#define RSDSA_ATTR_D30J_QUERY_MER_RMS_EQUALIZED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1713
#define RSDSA_ATTR_D30J_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1714
#define RSDSA_ATTR_D30J_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1715
#define RSDSA_ATTR_D30J_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1716
#define RSDSA_ATTR_D30J_QUERY_BER_BLOCKS_PROCESSED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1717
#define RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1718
#define RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1719
#define RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1720
#define RSDSA_ATTR_D30J_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1721
#define RSDSA_ATTR_D30J_QUERY_BER_PRE_REED_SOLOMON                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1722
#define RSDSA_ATTR_D30J_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1723
#define RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1724
#define RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_TOTAL_BITS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1725
#define RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1726
#define RSDSA_ATTR_D30J_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1727
#define RSDSA_ATTR_D30J_QUERY_CER_CORRECTED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1728
#define RSDSA_ATTR_D30J_QUERY_CER_UNCORRECTED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1729
#define RSDSA_ATTR_D30J_QUERY_CER_TOTAL_CODEWORDS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1730
#define RSDSA_ATTR_D30J_QUERY_CER_CORRECTED_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1731
#define RSDSA_ATTR_D30J_QUERY_CER_UNCORRECTED_CODEWORDS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1732
#define RSDSA_ATTR_D30J_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1733
#define RSDSA_ATTR_D30J_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1734
#define RSDSA_ATTR_D30J_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1735
#define RSDSA_ATTR_D30J_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1736
#define RSDSA_ATTR_D30J_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1739
#define RSDSA_ATTR_D30J_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1740
#define RSDSA_ATTR_D30J_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1741
#define RSDSA_ATTR_D30J_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1743
#define RSDSA_ATTR_D30J_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1744
#define RSDSA_ATTR_D30J_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1745
#define RSDSA_ATTR_D30J_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1746
#define RSDSA_ATTR_D30J_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1747
#define RSDSA_ATTR_D30J_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1748
#define RSDSA_ATTR_D30J_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1749
#define RSDSA_ATTR_D30J_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1750
#define RSDSA_ATTR_D30J_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1751
#define RSDSA_ATTR_D30J_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1752
#define RSDSA_ATTR_D30J_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1753
#define RSDSA_ATTR_D30J_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1754
#define RSDSA_ATTR_D30J_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1755
#define RSDSA_ATTR_D30J_QUERY_TRACE_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1756
#define RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1757
#define RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1758
#define RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1759
#define RSDSA_ATTR_D30J_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1760
#define RSDSA_ATTR_D30J_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1761
#define RSDSA_ATTR_D30J_CONFIGURE_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1762
#define RSDSA_ATTR_D30J_CONFIGURE_SPECTRAL_INVERSION_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1763
#define RSDSA_ATTR_D30J_CONFIGURE_EQUALIZER_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1764
#define RSDSA_ATTR_D30J_QUERY_EQUALIZER_COEFFICIENTS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1765
#define RSDSA_ATTR_D30J_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1766
#define RSDSA_ATTR_D30J_CONFIGURE_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1767
#define RSDSA_ATTR_D30J_CONFIGURE_PREAMPLIFIER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1768
#define RSDSA_ATTR_D30J_CONFIGURE_ASI_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1769
#define RSDSA_ATTR_D30J_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1770
#define RSDSA_ATTR_D30J_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1771
#define RSDSA_ATTR_D30J_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1772
#define RSDSA_ATTR_D30J_ALARM_THRESHOLD_BER_POST_REED_SOLOMON               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1773
#define RSDSA_ATTR_ENABLE_J83QAM_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1774
#define RSDSA_ATTR_JQD_QUERY_DEMODULATOR_LOCK_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1775
#define RSDSA_ATTR_JQD_QUERY_DECODER_LOCK_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1776
#define RSDSA_ATTR_JQD_QUERY_MPEG_LOCK_STATUS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1777
#define RSDSA_ATTR_JQD_QUERY_SIGNAL_POWER                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1778
#define RSDSA_ATTR_JQD_QUERY_FREQUENCY_OFFSET                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1779
#define RSDSA_ATTR_JQD_QUERY_SYMBOL_CLOCK_OFFSET                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1780
#define RSDSA_ATTR_JQD_QUERY_CHANNEL_BIT_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1781
#define RSDSA_ATTR_JQD_QUERY_INTERLEAVER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1782
#define RSDSA_ATTR_JQD_CONFIGURE_MER_COMPENSATION_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1783
#define RSDSA_ATTR_JQD_CONFIGURE_MER_SIGN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1784
#define RSDSA_ATTR_JQD_QUERY_MER_RMS_COMPENSATED_OVERALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1785
#define RSDSA_ATTR_JQD_QUERY_MER_RMS_EQUALIZED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1786
#define RSDSA_ATTR_JQD_QUERY_MER_PEAK                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1787
#define RSDSA_ATTR_JQD_RESET_ERROR_ANALYSIS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1788
#define RSDSA_ATTR_JQD_QUERY_BER_ELAPSED_TIME                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1789
#define RSDSA_ATTR_JQD_QUERY_BER_BLOCKS_PROCESSED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1790
#define RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1791
#define RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_MODE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1792
#define RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1793
#define RSDSA_ATTR_JQD_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1794
#define RSDSA_ATTR_JQD_QUERY_BER_PRE_REED_SOLOMON                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1795
#define RSDSA_ATTR_JQD_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1796
#define RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1797
#define RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1798
#define RSDSA_ATTR_JQD_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1799
#define RSDSA_ATTR_JQD_QUERY_CER_CORRECTED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1800
#define RSDSA_ATTR_JQD_QUERY_CER_UNCORRECTED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1801
#define RSDSA_ATTR_JQD_QUERY_CER_TOTAL_CODEWORDS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1802
#define RSDSA_ATTR_JQD_QUERY_CER_CORRECTED_CODEWORDS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1803
#define RSDSA_ATTR_JQD_QUERY_CER_UNCORRECTED_CODEWORDS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1804
#define RSDSA_ATTR_JQD_DISPLAY_CONSTELLATION_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1805
#define RSDSA_ATTR_JQD_DISPLAY_I_Q_DENSITY_PLOT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1806
#define RSDSA_ATTR_JQD_CONFIGURE_I_Q_MEASUREMENT_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1807
#define RSDSA_ATTR_JQD_CONFIGURE_I_Q_WINDOW_WIDTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1808
#define RSDSA_ATTR_JQD_CAPTURE_IQ_DATA_TO_FILENAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1811
#define RSDSA_ATTR_JQD_IQ_DATA_EXPORT_MAX_SAMPLES                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1812
#define RSDSA_ATTR_JQD_IQ_DATA_EXPORT_TIMEOUT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1813
#define RSDSA_ATTR_JQD_DISPLAY_MER_VS_TIME_PLOT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1815
#define RSDSA_ATTR_JQD_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1816
#define RSDSA_ATTR_JQD_DISPLAY_BER_VS_TIME_PLOT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1817
#define RSDSA_ATTR_JQD_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1818
#define RSDSA_ATTR_JQD_DISPLAY_POWER_VS_TIME_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1819
#define RSDSA_ATTR_JQD_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1820
#define RSDSA_ATTR_JQD_DISPLAY_CCDF_HISTOGRAM                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1821
#define RSDSA_ATTR_JQD_QUERY_CCDF_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1822
#define RSDSA_ATTR_JQD_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1823
#define RSDSA_ATTR_JQD_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1824
#define RSDSA_ATTR_JQD_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1825
#define RSDSA_ATTR_JQD_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1826
#define RSDSA_ATTR_JQD_DISPLAY_ECHO_PATTERN_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1827
#define RSDSA_ATTR_JQD_QUERY_TRACE_DATA                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1828
#define RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1829
#define RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_NEXT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1830
#define RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1831
#define RSDSA_ATTR_JQD_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1832
#define RSDSA_ATTR_JQD_CONFIGURE_FEC                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1833
#define RSDSA_ATTR_JQD_CONFIGURE_FREQUENCY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1834
#define RSDSA_ATTR_JQD_CONFIGURE_CONSTELLATION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1835
#define RSDSA_ATTR_JQD_CONFIGURE_SYMBOL_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1836
#define RSDSA_ATTR_JQD_CONFIGURE_ROLL_OFF                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1837
#define RSDSA_ATTR_JQD_CONFIGURE_SPECTRAL_INVERSION_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1838
#define RSDSA_ATTR_JQD_CONFIGURE_EQUALIZER_STATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1839
#define RSDSA_ATTR_JQD_QUERY_EQUALIZER_COEFFICIENTS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1840
#define RSDSA_ATTR_JQD_AUTO_LEVEL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1841
#define RSDSA_ATTR_JQD_CONFIGURE_ATTENUATION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1842
#define RSDSA_ATTR_JQD_CONFIGURE_PREAMPLIFIER_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1843
#define RSDSA_ATTR_JQD_CONFIGURE_ASI_OUTPUT_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1844
#define RSDSA_ATTR_JQD_CONFIGURE_PHASE_TRACKING_LOOP_BANDWIDTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1845
#define RSDSA_ATTR_JQD_ALARM_THRESHOLD_RMS_MER_OVERALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1846
#define RSDSA_ATTR_JQD_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1847
#define RSDSA_ATTR_JQD_ALARM_THRESHOLD_BER_POST_REED_SOLOMON                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1848
#define RSDSA_ATTR_ENABLE_DOCSIS_31_OFDM_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1849
#define RSDSA_ATTR_D31D_CONFIGURE_PROFILE_FILTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1850
#define RSDSA_ATTR_D31D_QUERY_PLC_LOCK_STATUS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1851
#define RSDSA_ATTR_D31D_QUERY_DEMODULATOR_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1852
#define RSDSA_ATTR_D31D_QUERY_SIGNAL_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1853
#define RSDSA_ATTR_D31D_QUERY_OCCUPIED_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1854
#define RSDSA_ATTR_D31D_QUERY_SIGNAL_POWER_PER_6MHZ                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1855
#define RSDSA_ATTR_D31D_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1856
#define RSDSA_ATTR_D31D_QUERY_SYMBOL_CLOCK_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1857
#define RSDSA_ATTR_D31D_QUERY_NCP_CRC_ERRORS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1858
#define RSDSA_ATTR_D31D_QUERY_PAYLOAD_DATA_RATE_PROFILE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1859
#define RSDSA_ATTR_D31D_CONFIGURE_MER_COMPENSATION_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1860
#define RSDSA_ATTR_D31D_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1861
#define RSDSA_ATTR_D31D_QUERY_MER_RMS_COMPENSATED_OVERALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1862
#define RSDSA_ATTR_D31D_QUERY_MER_RMS_PARTIALY_EQUALIZED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1863
#define RSDSA_ATTR_D31D_QUERY_MER_RMS_EQUALIZED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1864
#define RSDSA_ATTR_D31D_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1865
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_RMS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1866
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_RMS_COMPENSATED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1867
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_A_PEAK                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1868
#define RSDSA_ATTR_D31D_QUERY_PROFILE_A_CONSTELLATIONS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1869
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_RMS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1870
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_RMS_COMPENSATED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1871
#define RSDSA_ATTR_D31D_QUERY_MER_PROFILE_FILTER_PEAK                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1872
#define RSDSA_ATTR_D31D_QUERY_PROFILE_FILTER_CONSTELLATIONS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1873
#define RSDSA_ATTR_D31D_QUERY_MER_PILOT_RMS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1874
#define RSDSA_ATTR_D31D_QUERY_MER_PILOT_RMS_COMPENSATED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1875
#define RSDSA_ATTR_D31D_QUERY_MER_PILOT_PEAK                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1876
#define RSDSA_ATTR_D31D_QUERY_PILOT_CONSTELLATIONS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1877
#define RSDSA_ATTR_D31D_QUERY_MER_PLC_RMS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1878
#define RSDSA_ATTR_D31D_QUERY_MER_PLC_RMS_COMPENSATED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1879
#define RSDSA_ATTR_D31D_QUERY_MER_PLC_PEAK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1880
#define RSDSA_ATTR_D31D_QUERY_PLC_CONSTELLATIONS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1881
#define RSDSA_ATTR_D31D_QUERY_MER_NCP_RMS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1882
#define RSDSA_ATTR_D31D_QUERY_MER_NCP_RMS_COMPENSATED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1883
#define RSDSA_ATTR_D31D_QUERY_MER_NCP_PEAK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1884
#define RSDSA_ATTR_D31D_QUERY_NCP_CONSTELLATIONS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1885
#define RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_RMS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1886
#define RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_RMS_COMPENSATED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1887
#define RSDSA_ATTR_D31D_QUERY_MER_ZERO_BIT_LOADED_PEAK                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1888
#define RSDSA_ATTR_D31D_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1889
#define RSDSA_ATTR_D31D_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1890
#define RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_BER_SOURCE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1891
#define RSDSA_ATTR_D31D_QUERY_BER_PLC                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1892
#define RSDSA_ATTR_D31D_QUERY_BER_PLC_TOTAL_BITS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1893
#define RSDSA_ATTR_D31D_QUERY_BER_NCP                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1894
#define RSDSA_ATTR_D31D_QUERY_BER_NCP_TOTAL_BITS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1895
#define RSDSA_ATTR_D31D_QUERY_CER_PLC                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1896
#define RSDSA_ATTR_D31D_QUERY_CER_PLC_TOTAL_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1897
#define RSDSA_ATTR_D31D_QUERY_CER_NCP                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1898
#define RSDSA_ATTR_D31D_QUERY_CER_NCP_TOTAL_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1899
#define RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_BER_MODE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1900
#define RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_TEST_DEPTH         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1901
#define RSDSA_ATTR_D31D_CONFIGURE_ERROR_ANALYSIS_PROFILE_TEST_DEPTH_SOURCE  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1902
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_BLOCKS_PROCESSED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1903
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_PRE_LDPC                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1904
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_PRE_LDPC_TOTAL_BITS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1905
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_LDPC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1906
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_LDPC_TOTAL_BITS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1907
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_BCH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1908
#define RSDSA_ATTR_D31D_QUERY_PROFILE_BER_POST_BCH_TOTAL_BITS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1909
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1910
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC_TOTAL_CODEWORDS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1911
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_LDPC_CODEWORD_ERRORS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1912
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1913
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH_TOTAL_CODEWORDS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1914
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_POST_BCH_CODEWORD_ERRORS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1915
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_AVG_PAYLOAD_PER_CODEWORD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1916
#define RSDSA_ATTR_D31D_QUERY_PROFILE_CER_AVG_LDPC_ITERATIONS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1917
#define RSDSA_ATTR_D31D_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1918
#define RSDSA_ATTR_D31D_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1919
#define RSDSA_ATTR_D31D_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1920
#define RSDSA_ATTR_D31D_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1921
#define RSDSA_ATTR_D31D_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1924
#define RSDSA_ATTR_D31D_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1925
#define RSDSA_ATTR_D31D_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1926
#define RSDSA_ATTR_D31D_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1928
#define RSDSA_ATTR_D31D_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1929
#define RSDSA_ATTR_D31D_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1930
#define RSDSA_ATTR_D31D_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1931
#define RSDSA_ATTR_D31D_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1932
#define RSDSA_ATTR_D31D_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1933
#define RSDSA_ATTR_D31D_DISPLAY_MER_VS_SUBCARRIER_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1934
#define RSDSA_ATTR_D31D_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1935
#define RSDSA_ATTR_D31D_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1936
#define RSDSA_ATTR_D31D_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1937
#define RSDSA_ATTR_D31D_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1938
#define RSDSA_ATTR_D31D_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1939
#define RSDSA_ATTR_D31D_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1940
#define RSDSA_ATTR_D31D_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1941
#define RSDSA_ATTR_D31D_DISPLAY_SPECTRUM_ANALYSIS_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1942
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1943
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1944
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1945
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1946
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_AMPLITUDE_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 1947
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_AMPLITUDE_SCALE_PER_DIVISION RS_SPECIFIC_PUBLIC_ATTR_BASE + 1948
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_AUTO_SCALE_Y_AXIS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1949
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1951
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_CAPTURE_MODE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1952
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_RUN_SINGLE_SWEEP                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1953
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_TRACE_HOLD              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1954
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_STATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1955
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_X_POSITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1956
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_Y_POSITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1957
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1958
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1959
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1960
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1961
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1962
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1963
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1964
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1965
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 1966
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1967
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1968
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1969
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1970
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1971
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1972
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1973
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1974
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 1976
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 1977
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1978
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CONFIGURE_DISPLAY_MASK_VISIBLE    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1979
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_LOWER_CONFIGURATION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1980
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_DISPLAY_MASK_UPPER_CONFIGURATION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1981
#define RSDSA_ATTR_D31D_SPECTRUM_ANALYSIS_CLEAR_MASK                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1982
#define RSDSA_ATTR_D31D_QUERY_TRACE_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1983
#define RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1984
#define RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1985
#define RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1986
#define RSDSA_ATTR_D31D_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1987
#define RSDSA_ATTR_D31D_CAPTURE_PAYLOAD_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1988
#define RSDSA_ATTR_D31D_PAYLOAD_DATA_EXPORT_FILTER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1989
#define RSDSA_ATTR_D31D_PAYLOAD_DATA_EXPORT_MAX_FILE_SIZE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1990
#define RSDSA_ATTR_D31D_CONFIGURE_PLOT_DATA_FILTER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1992
#define RSDSA_ATTR_D31D_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1994
#define RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_0_FREQUENCY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1995
#define RSDSA_ATTR_D31D_CONFIGURE_EQUALIZER_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1996
#define RSDSA_ATTR_D31D_QUERY_EQUALIZER_COEFFICIENTS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1997
#define RSDSA_ATTR_D31D_CONFIGURE_PREFILTER_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1998
#define RSDSA_ATTR_D31D_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1999
#define RSDSA_ATTR_D31D_CONFIGURE_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2000
#define RSDSA_ATTR_D31D_CONFIGURE_PREAMPLIFIER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2001
#define RSDSA_ATTR_D31D_CONFIGURE_ACQUISITION_MODE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2003
#define RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_SPACING                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2004
#define RSDSA_ATTR_D31D_CONFIGURE_CYCLIC_PREFIX_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2005
#define RSDSA_ATTR_D31D_CONFIGURE_TIME_INTERLEAVER_DEPTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2006
#define RSDSA_ATTR_D31D_CONFIGURE_SUBCARRIER_CONFIG_FILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2007
#define RSDSA_ATTR_D31D_CONFIGURE_SFP_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2008
#define RSDSA_ATTR_D31D_CONFIGURE_SFP_PLC_UDP_PORT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2009
#define RSDSA_ATTR_D31D_CONFIGURE_SFP_PLC_UDP_PORT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2010
#define RSDSA_ATTR_D31D_CONFIGURE_SFP_PROFILE_UDP_PORT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2011
#define RSDSA_ATTR_D31D_CONFIGURE_SFP_PROFILE_UDP_PORT_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2012
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2013
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_PRE_LDPC                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2014
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_POST_LDPC                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2015
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_POST_BCH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2016
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_PLC                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2017
#define RSDSA_ATTR_D31D_ALARM_THRESHOLD_BER_NCP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2018
#define RSDSA_ATTR_ENABLE_DOWNSTREAM_SPECTRUM_ANALYSIS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2019
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2020
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2021
#define RSDSA_ATTR_DS_SPECAN_CLEAR_ALL_DISPLAY_STATES                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2022
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2023
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2024
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_CENTER_FREQUENCY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2025
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_START_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2026
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_STOP_FREQUENCY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2027
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_FREQUENCY_SPAN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2028
#define RSDSA_ATTR_DS_SPECAN_SET_FREQUENCY_SPAN_FULL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2029
#define RSDSA_ATTR_DS_SPECAN_SET_FREQUENCY_SPAN_MINIMUM                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2030
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2031
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_SWEEP_TIME                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2032
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ZERO_SPAN_SAMPLE_RATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2033
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_REFERENCE_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2034
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_SCALE_PER_DIVISION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2035
#define RSDSA_ATTR_DS_SPECAN_AUTO_SCALE_Y_AXIS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2036
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_RBW_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2037
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_RBW                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2038
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_VBW_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2039
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_VBW                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2040
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_AUTO_AVERAGE_COUNT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2041
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_MANUAL_AVERAGE_COUNT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2042
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_CAPTURE_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2043
#define RSDSA_ATTR_DS_SPECAN_RUN_SINGLE_SWEEP                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2044
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_DETECTOR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2045
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_TRACE_HOLD                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2046
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_MODE_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2047
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_MODE_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2048
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_GRID_SPACING                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2050
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_ACP_REFERENCE_CHANNEL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2051
#define RSDSA_ATTR_DS_SPECAN_QUERY_ACP_POWER_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2052
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_START_OFFSET             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2055
#define RSDSA_ATTR_DS_SPECAN_CONFIGURE_PHASE_NOISE_STOP_OFFSET              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2056
#define RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_SPOT_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2057
#define RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_INTEGRATED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2058
#define RSDSA_ATTR_DS_SPECAN_QUERY_PHASE_NOISE_BANDWIDTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2059
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2060
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2061
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2062
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2063
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2064
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2065
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2066
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2067
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2068
#define RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2069
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2070
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2071
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2988
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2073
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2074
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2075
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK RS_SPECIFIC_PUBLIC_ATTR_BASE + 2076
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2077
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2078
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2079
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2080
#define RSDSA_ATTR_DS_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2081
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 2082
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2083
#define RSDSA_ATTR_DS_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2084
#define RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_STORE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2085
#define RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_CLEAR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2086
#define RSDSA_ATTR_DS_SPECAN_DISPLAY_STATE_RECALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2087
#define RSDSA_ATTR_DS_SPECAN_QUERY_TRACE_DATA                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2088
#define RSDSA_ATTR_DS_SPECAN_MASK_VISIBLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2089
#define RSDSA_ATTR_DS_SPECAN_CLEAR_MASK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2090
#define RSDSA_ATTR_ENABLE_DOCSIS_30_ATDMA_MODE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2095
#define RSDSA_ATTR_D30A_CONFIGURE_MAP_GEN_IUC                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2096
#define RSDSA_ATTR_D30A_QUERY_BURST_DEMODULATOR_LOCK_STATUS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2097
#define RSDSA_ATTR_D30A_QUERY_FEC_LOCK_STATUS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2098
#define RSDSA_ATTR_D30A_QUERY_BURST_RECEIVE_POWER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2099
#define RSDSA_ATTR_D30A_QUERY_SIGNAL_POWER_PER_1600KHZ                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2100
#define RSDSA_ATTR_D30A_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2101
#define RSDSA_ATTR_D30A_QUERY_BURST_TIMING_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2102
#define RSDSA_ATTR_D30A_QUERY_PAYLOAD_DATA_RATE_IUC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2103
#define RSDSA_ATTR_D30A_QUERY_MINISLOTS_RECEIVED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2104
#define RSDSA_ATTR_D30A_QUERY_BURST_COVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2105
#define RSDSA_ATTR_D30A_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2106
#define RSDSA_ATTR_D30A_QUERY_MER_RMS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2107
#define RSDSA_ATTR_D30A_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2108
#define RSDSA_ATTR_D30A_QUERY_MER_IUC_RMS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2109
#define RSDSA_ATTR_D30A_QUERY_MER_IUC_PEAK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2110
#define RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_AMPLITUDE_DB                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2111
#define RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2112
#define RSDSA_ATTR_D30A_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2113
#define RSDSA_ATTR_D30A_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2114
#define RSDSA_ATTR_D30A_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2115
#define RSDSA_ATTR_D30A_QUERY_BER_BLOCKS_PROCESSED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2116
#define RSDSA_ATTR_D30A_QUERY_BER_BURSTS_EXPECTED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2117
#define RSDSA_ATTR_D30A_QUERY_BER_BURSTS_RECEIVED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2118
#define RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2119
#define RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2120
#define RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2121
#define RSDSA_ATTR_D30A_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 2122
#define RSDSA_ATTR_D30A_QUERY_BER_PRE_REED_SOLOMON                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2123
#define RSDSA_ATTR_D30A_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2124
#define RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2125
#define RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2126
#define RSDSA_ATTR_D30A_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2127
#define RSDSA_ATTR_D30A_QUERY_CER_CORRECTED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2128
#define RSDSA_ATTR_D30A_QUERY_CER_UNCORRECTED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2129
#define RSDSA_ATTR_D30A_QUERY_CER_TOTAL_CODEWORDS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2130
#define RSDSA_ATTR_D30A_QUERY_CER_CORRECTED_CODEWORDS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2131
#define RSDSA_ATTR_D30A_QUERY_CER_UNCORRECTED_CODEWORDS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2132
#define RSDSA_ATTR_D30A_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2133
#define RSDSA_ATTR_D30A_QUERY_CER_CODEWORD_COVERAGE_IUC                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2134
#define RSDSA_ATTR_D30A_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2135
#define RSDSA_ATTR_D30A_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2136
#define RSDSA_ATTR_D30A_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2137
#define RSDSA_ATTR_D30A_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2138
#define RSDSA_ATTR_D30A_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2141
#define RSDSA_ATTR_D30A_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2142
#define RSDSA_ATTR_D30A_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2143
#define RSDSA_ATTR_D30A_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2145
#define RSDSA_ATTR_D30A_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2146
#define RSDSA_ATTR_D30A_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2147
#define RSDSA_ATTR_D30A_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2148
#define RSDSA_ATTR_D30A_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2149
#define RSDSA_ATTR_D30A_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2150
#define RSDSA_ATTR_D30A_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2151
#define RSDSA_ATTR_D30A_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2152
#define RSDSA_ATTR_D30A_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2153
#define RSDSA_ATTR_D30A_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2154
#define RSDSA_ATTR_D30A_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2155
#define RSDSA_ATTR_D30A_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2156
#define RSDSA_ATTR_D30A_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2157
#define RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2158
#define RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2159
#define RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2160
#define RSDSA_ATTR_D30A_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2161
#define RSDSA_ATTR_D30A_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2162
#define RSDSA_ATTR_D30A_CONFIGURE_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2163
#define RSDSA_ATTR_D30A_CONFIGURE_CHANNEL_SYMBOL_RATE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2164
#define RSDSA_ATTR_D30A_CONFIGURE_FEC_PARAMETER_T                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2165
#define RSDSA_ATTR_D30A_CONFIGURE_FEC_PARAMETER_K                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2166
#define RSDSA_ATTR_D30A_CONFIGURE_LAST_CODEWORD_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2167
#define RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_PATTERN                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2168
#define RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2169
#define RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2170
#define RSDSA_ATTR_D30A_CONFIGURE_PREAMBLE_TYPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2171
#define RSDSA_ATTR_D30A_CONFIGURE_SCRAMBLER_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2172
#define RSDSA_ATTR_D30A_CONFIGURE_SCRAMBLER_SEED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2173
#define RSDSA_ATTR_D30A_CONFIGURE_GUARD_TIME_SIZE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2174
#define RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2175
#define RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_DEPTH            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2176
#define RSDSA_ATTR_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_BLOCK_SIZE       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2177
#define RSDSA_ATTR_D30A_CONFIGURE_MINISLOT_SIZE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2178
#define RSDSA_ATTR_D30A_CONFIGURE_EQUALIZER_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2179
#define RSDSA_ATTR_D30A_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2180
#define RSDSA_ATTR_D30A_CONFIGURE_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2181
#define RSDSA_ATTR_D30A_CONFIGURE_PREAMPLIFIER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2182
#define RSDSA_ATTR_D30A_CONFIGURE_GRANT_SIZE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2183
#define RSDSA_ATTR_D30A_CONFIGURE_STARTING_MINISLOT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2184
#define RSDSA_ATTR_D30A_CONFIGURE_SCHEDULING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2185
#define RSDSA_ATTR_D30A_CONFIGURE_REPEAT_INTERVAL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2186
#define RSDSA_ATTR_D30A_CONFIGURE_NUMBER_OF_BURSTS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2187
#define RSDSA_ATTR_D30A_CONFIGURE_RANGING_MODE_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2188
#define RSDSA_ATTR_D30A_CONFIGURE_UNTIMED_BURST_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2189
#define RSDSA_ATTR_D30A_CONFIGURE_INPUT_TRIGGER_OFFSET                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2190
#define RSDSA_ATTR_D30A_CONFIGURE_SFP_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2191
#define RSDSA_ATTR_D30A_CONFIGURE_SFP_IUC_UDP_PORT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2192
#define RSDSA_ATTR_D30A_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2193
#define RSDSA_ATTR_D30A_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2194
#define RSDSA_ATTR_D30A_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2195
#define RSDSA_ATTR_D30A_ALARM_THRESHOLD_BER_POST_REED_SOLOMON               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2196
#define RSDSA_ATTR_D30A_CONFIGURE_MER_BURSTS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2197
#define RSDSA_ATTR_ENABLE_DOCSIS_31_OFDMA_MODE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2198
#define RSDSA_ATTR_D31U_CONFIGURE_MAP_GEN_IUC                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2199
#define RSDSA_ATTR_D31U_QUERY_BURST_DEMODULATOR_LOCK_STATUS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2200
#define RSDSA_ATTR_D31U_QUERY_FEC_LOCK_STATUS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2201
#define RSDSA_ATTR_D31U_READ_BURST_RECEIVE_POWER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2202
#define RSDSA_ATTR_D31U_QUERY_SIGNAL_POWER_PER_1600KHZ                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2203
#define RSDSA_ATTR_D31U_QUERY_FREQUENCY_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2204
#define RSDSA_ATTR_D31U_QUERY_BURST_TIMING_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2205
#define RSDSA_ATTR_D31U_QUERY_PAYLOAD_DATA_RATE_IUC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2206
#define RSDSA_ATTR_D31U_QUERY_MINISLOTS_RECEIVED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2207
#define RSDSA_ATTR_D31U_QUERY_BURST_COVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2208
#define RSDSA_ATTR_D31U_CONFIGURE_MER_SIGN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2209
#define RSDSA_ATTR_D31U_QUERY_MER_RMS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2210
#define RSDSA_ATTR_D31U_QUERY_MER_RMS_PARTIALY_EQUALIZED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2211
#define RSDSA_ATTR_D31U_QUERY_MER_PEAK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2212
#define RSDSA_ATTR_D31U_QUERY_MER_PILOT_RMS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2213
#define RSDSA_ATTR_D31U_QUERY_MER_PILOT_PEAK                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2214
#define RSDSA_ATTR_D31U_QUERY_PILOT_CONSTELLATIONS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2215
#define RSDSA_ATTR_D31U_QUERY_MER_COMPLEMENTARY_PILOT_RMS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2216
#define RSDSA_ATTR_D31U_QUERY_MER_COMPLEMENTARY_PILOT_PEAK                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2217
#define RSDSA_ATTR_D31U_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2218
#define RSDSA_ATTR_D31U_QUERY_MER_MAP_GEN_IUC_RMS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2219
#define RSDSA_ATTR_D31U_QUERY_MER_MAP_GEN_IUC_PEAK                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2220
#define RSDSA_ATTR_D31U_QUERY_IUC_CONSTELLATIONS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2221
#define RSDSA_ATTR_D31U_RESET_ERROR_ANALYSIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2222
#define RSDSA_ATTR_D31U_QUERY_BER_ELAPSED_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2223
#define RSDSA_ATTR_D31U_QUERY_BER_BLOCKS_PROCESSED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2224
#define RSDSA_ATTR_D31U_QUERY_BER_BURSTS_EXPECTED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2225
#define RSDSA_ATTR_D31U_QUERY_BER_BURSTS_RECEIVED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2226
#define RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2227
#define RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2228
#define RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2229
#define RSDSA_ATTR_D31U_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 2230
#define RSDSA_ATTR_D31U_QUERY_BER_IUC_PRE_LDPC                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2231
#define RSDSA_ATTR_D31U_QUERY_BER_IUC_PRE_LDPC_TOTAL_BITS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2232
#define RSDSA_ATTR_D31U_QUERY_BER_IUC_POST_LDPC                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2233
#define RSDSA_ATTR_D31U_QUERY_BER_IUC_POST_LDPC_TOTAL_BITS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2234
#define RSDSA_ATTR_D31U_CONFIGURE_CODEWORD_LENGTH_FILTER                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2235
#define RSDSA_ATTR_D31U_QUERY_CER_POST_LDPC                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2236
#define RSDSA_ATTR_D31U_QUERY_CER_POST_LDPC_CODEWORD_ERRORS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2237
#define RSDSA_ATTR_D31U_QUERY_CER_AVERAGE_LDPC_ITERATIONS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2238
#define RSDSA_ATTR_D31U_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2239
#define RSDSA_ATTR_D31U_QUERY_CER_CODEWORD_COVERAGE_IUC                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2240
#define RSDSA_ATTR_D31U_DISPLAY_CONSTELLATION_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2241
#define RSDSA_ATTR_D31U_DISPLAY_I_Q_DENSITY_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2242
#define RSDSA_ATTR_D31U_CONFIGURE_I_Q_MEASUREMENT_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2243
#define RSDSA_ATTR_D31U_CONFIGURE_I_Q_WINDOW_WIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2244
#define RSDSA_ATTR_D31U_CAPTURE_IQ_DATA_TO_FILENAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2247
#define RSDSA_ATTR_D31U_IQ_DATA_EXPORT_MAX_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2248
#define RSDSA_ATTR_D31U_IQ_DATA_EXPORT_TIMEOUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2249
#define RSDSA_ATTR_D31U_DISPLAY_MER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2251
#define RSDSA_ATTR_D31U_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2252
#define RSDSA_ATTR_D31U_DISPLAY_BER_VS_TIME_PLOT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2253
#define RSDSA_ATTR_D31U_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2254
#define RSDSA_ATTR_D31U_DISPLAY_POWER_VS_TIME_PLOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2255
#define RSDSA_ATTR_D31U_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2256
#define RSDSA_ATTR_D31U_DISPLAY_MER_VS_SUBCARRIER_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2257
#define RSDSA_ATTR_D31U_DISPLAY_MER_VS_MINISLOT_PLOT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2258
#define RSDSA_ATTR_D31U_DISPLAY_CCDF_HISTOGRAM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2259
#define RSDSA_ATTR_D31U_QUERY_CCDF_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2260
#define RSDSA_ATTR_D31U_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2261
#define RSDSA_ATTR_D31U_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2262
#define RSDSA_ATTR_D31U_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2263
#define RSDSA_ATTR_D31U_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2264
#define RSDSA_ATTR_D31U_DISPLAY_ECHO_PATTERN_PLOT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2265
#define RSDSA_ATTR_D31U_DISPLAY_SPECTRUM_ANALYSIS_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2266
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2267
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2268
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2269
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2270
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AMPLITUDE_CONFIGURE_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2271
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AMPLITUDE_CONFIGURE_SCALE_PER_DIVISION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2272
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_AUTO_SCALE_Y_AXIS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2273
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2275
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_CAPTURE_MODE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2276
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_RUN_SINGLE_SWEEP                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2277
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CONFIGURE_TRACE_HOLD              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2278
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_STATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2279
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_X_POSITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2280
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_Y_POSITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2281
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2282
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2283
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2284
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2285
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2286
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2287
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2288
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2289
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2290
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2291
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2292
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2293
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2294
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2295
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2296
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2297
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2298
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 2299
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2300
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2301
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_MASK_VISIBLE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2302
#define RSDSA_ATTR_D31U_SPECTRUM_ANALYSIS_CLEAR_MASK                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2303
#define RSDSA_ATTR_D31U_QUERY_TRACE_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2308
#define RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2309
#define RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_NEXT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2310
#define RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_COUNT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2311
#define RSDSA_ATTR_D31U_SYSTEM_ACQUISITION_LOG_CLEAR_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2312
#define RSDSA_ATTR_D31U_CONFIGURE_PLOT_DATA_FILTER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2313
#define RSDSA_ATTR_D31U_CONFIGURE_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2314
#define RSDSA_ATTR_D31U_CONFIGURE_ROLL_OFF                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2315
#define RSDSA_ATTR_D31U_CONFIGURE_CYCLIC_PREFIX_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2316
#define RSDSA_ATTR_D31U_CONFIGURE_SYMBOLS_PER_FRAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2317
#define RSDSA_ATTR_D31U_CONFIGURE_SUBCARRIER_SPACING                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2318
#define RSDSA_ATTR_D31U_CONFIGURE_RANDOMIZER_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2319
#define RSDSA_ATTR_D31U_CONFIGURE_RANDOMIZER_SEED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2320
#define RSDSA_ATTR_D31U_CONFIGURE_EXCLUSION_BAND_START                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2321
#define RSDSA_ATTR_D31U_CONFIGURE_EXCLUSION_BAND_WIDTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2322
#define RSDSA_ATTR_D31U_CONFIGURE_MINISLOT_CONFIGURATION_MODE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2323
#define RSDSA_ATTR_D31U_CONFIGURE_MINISLOT_CONFIGURATION_FILE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2324
#define RSDSA_ATTR_D31U_CONFIGURE_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2325
#define RSDSA_ATTR_D31U_CONFIGURE_PILOT_PATTERN                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2326
#define RSDSA_ATTR_D31U_CONFIGURE_STARTING_MINISLOT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2327
#define RSDSA_ATTR_D31U_CONFIGURE_NUMBER_OF_MINISLOTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2328
#define RSDSA_ATTR_D31U_CONFIGURE_FIRST_ACTIVE_SUBCARRIER                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2329
#define RSDSA_ATTR_D31U_CONFIGURE_LAST_ACTIVE_SUBCARRIER                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2330
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_STARTING_MINISLOT_IN_FRAME RS_SPECIFIC_PUBLIC_ATTR_BASE + 2331
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_MINISLOTS       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2332
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_SUBCARRIERS     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2333
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_PATTERN          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2334
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_LENGTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2335
#define RSDSA_ATTR_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_OFFSET           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2336
#define RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_STARTING_MINISLOT_IN_FRAME   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2337
#define RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_MINISLOTS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2338
#define RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_SUBCARRIERS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2339
#define RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_PATTERN             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2340
#define RSDSA_ATTR_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_OFFSET              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2341
#define RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_START_SUBCARRIER           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2342
#define RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_SUBCARRIER_SKIP            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2343
#define RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_STAGGER_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2344
#define RSDSA_ATTR_D31U_CONFIGURE_WIDEBAND_PROBE_SYMBOL_IN_FRAME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2345
#define RSDSA_ATTR_D31U_CONFIGURE_BW_REQUEST_NUMBER_OF_MINISLOTS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2346
#define RSDSA_ATTR_D31U_CONFIGURE_BW_REQUEST_SUBSLOT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2347
#define RSDSA_ATTR_D31U_AUTO_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2348
#define RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_ATTENUATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2349
#define RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREAMPLIFIER_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2350
#define RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREFILTER_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2351
#define RSDSA_ATTR_D31U_CONFIGURE_SIGNAL_PATH_PREFILTER_LENGTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2352
#define RSDSA_ATTR_D31U_CONFIGURE_SCHEDULING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2353
#define RSDSA_ATTR_D31U_CONFIGURE_SCHEDULING_REPEAT_INTERVAL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2354
#define RSDSA_ATTR_D31U_CONFIGURE_INPUT_TRIGGER_OFFSET                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2355
#define RSDSA_ATTR_D31U_CONFIGURE_SFP_OUTPUT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2356
#define RSDSA_ATTR_D31U_CONFIGURE_SFP_IUC_UDP_PORT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2357
#define RSDSA_ATTR_D31U_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2358
#define RSDSA_ATTR_D31U_ALARM_THRESHOLD_RMS_MER_OVERALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2359
#define RSDSA_ATTR_D31U_ALARM_THRESHOLD_BER_PRE_LDPC                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2360
#define RSDSA_ATTR_D31U_ALARM_THRESHOLD_BER_POST_LDPC                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2361
#define RSDSA_ATTR_D31U_CONFIGURE_MER_BURSTS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2362
#define RSDSA_ATTR_ENABLE_UPSTREAM_SPECTRUM_ANALYSIS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2363
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2364
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2365
#define RSDSA_ATTR_US_SPECAN_CLEAR_ALL_DISPLAY_STATES                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2366
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2367
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2368
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_CENTER_FREQUENCY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2369
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_START_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2370
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_STOP_FREQUENCY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2371
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_FREQUENCY_SPAN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2372
#define RSDSA_ATTR_US_SPECAN_SET_FREQUENCY_SPAN_FULL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2373
#define RSDSA_ATTR_US_SPECAN_SET_FREQUENCY_SPAN_MINIMUM                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2374
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2375
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_SWEEP_TIME                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2376
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_ZERO_SPAN_SAMPLE_RATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2377
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_START_OFFSET             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2378
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_STOP_OFFSET              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2379
#define RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_SPOT_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2380
#define RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_INTEGRATED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2381
#define RSDSA_ATTR_US_SPECAN_QUERY_PHASE_NOISE_BANDWIDTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2382
#define RSDSA_ATTR_US_SPECAN_DISPLAY_CCDF_HISTOGRAM                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2383
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_CCDF_MODE_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2384
#define RSDSA_ATTR_US_SPECAN_QUERY_CCDF_PEAK                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2385
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_REFERENCE_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2386
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_SCALE_PER_DIVISION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2387
#define RSDSA_ATTR_US_SPECAN_AUTO_SCALE_Y_AXIS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2388
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_RBW_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2389
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_RBW                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2390
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_VBW_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2391
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_VBW                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2392
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_AUTO_AVERAGE_COUNT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2393
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_MANUAL_AVERAGE_COUNT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2394
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_CAPTURE_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2395
#define RSDSA_ATTR_US_SPECAN_RUN_SINGLE_SWEEP                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2396
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_DETECTOR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2397
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_TRACE_HOLD                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2398
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_ACP_MODE_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2399
#define RSDSA_ATTR_US_SPECAN_CONFIGURE_PHASE_NOISE_MODE_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2400
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_STATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2401
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_X_POSITION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2402
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_Y_POSITION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2403
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2404
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2405
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2406
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2407
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2408
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_CENTER_FREQUENCY  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2409
#define RSDSA_ATTR_US_SPECAN_ZERO_SPAN_MARKER_TO_TRIGGER_POINT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2410
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2411
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2412
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2413
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2414
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2415
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2416
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK RS_SPECIFIC_PUBLIC_ATTR_BASE + 2417
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2418
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2419
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2420
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2421
#define RSDSA_ATTR_US_SPECAN_ZERO_SPAN_DELTA_MARKER_TO_TRIGGER_POINT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2422
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_CENTER_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 2423
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2424
#define RSDSA_ATTR_US_SPECAN_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2425
#define RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_STORE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2426
#define RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_CLEAR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2427
#define RSDSA_ATTR_US_SPECAN_DISPLAY_STATE_RECALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2428
#define RSDSA_ATTR_US_SPECAN_QUERY_TRACE_DATA                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2429
#define RSDSA_ATTR_US_SPECAN_MASK_VISIBLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2430
#define RSDSA_ATTR_US_SPECAN_CLEAR_MASK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2431
#define RSDSA_ATTR_ENABLE_DYNAMIC_UPSTREAM_ANALYSIS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2436
#define RSDSA_ATTR_DU_ANALYSIS_RESTART_DISTANCE_ESTIMATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2437
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_FILTER_IUC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2438
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_FILTER_SID                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2439
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_DEMODULATOR_LOCK_STATUS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2440
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_FEC_LOCK_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2441
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MPEG_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2442
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_PLC_LOCK_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2443
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_RECEIVE_POWER                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2444
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_FREQUENCY_OFFSET                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2445
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_TIMING_OFFSET                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2446
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_PAYLOAD_DATA_RATE_IUC                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2447
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MINISLOTS_RECEIVED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2448
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BURST_COVERAGE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2449
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_DETECTED_UPSTREAM_MODE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2450
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MER_SIGN                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2451
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_RMS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2452
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_RMS_PARTIALY_EQUALIZED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2453
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PEAK                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2454
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PILOT_RMS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2455
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_PILOT_PEAK                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2456
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_PILOT_CONSTELLATIONS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2457
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_RMS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2458
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_PEAK           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2459
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2460
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_FILTERED_RMS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2461
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_MER_FILTERED_PEAK                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2462
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_DB              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2463
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2464
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2465
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_DOWNSTREAM_CHANNEL_ID                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2466
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_UPSTREAM_CHANNEL_ID                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2467
#define RSDSA_ATTR_DU_ANALYSIS_RESET_ERROR_ANALYSIS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2468
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_ELAPSED_TIME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2469
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BLOCKS_PROCESSED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2470
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BURSTS_EXPECTED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2471
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_BURSTS_RECEIVED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2472
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2473
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_MODE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2474
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2475
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 2476
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2477
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2478
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2479
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2480
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2481
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_PRE_LDPC                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2482
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_PRE_LDPC_TOTAL_BITS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2483
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_POST_LDPC                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2484
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_BER_IUC_POST_LDPC_TOTAL_BITS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2485
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_CODEWORD_LENGTH_FILTER             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2486
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CORRECTED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2487
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_UNCORRECTED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2488
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_TOTAL_CODEWORDS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2489
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CORRECTED_CODEWORDS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2490
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_UNCORRECTED_CODEWORDS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2491
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_POST_LDPC                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2492
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_POST_LDPC_CODEWORD_ERRORS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2493
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_AVERAGE_LDPC_ITERATIONS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2494
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_AVG_PAYLOAD_PER_CODEWORD_PER_IUC   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2495
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CER_CODEWORD_COVERAGE_IUC              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2496
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_CONSTELLATION_PLOT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2497
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_I_Q_DENSITY_PLOT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2498
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_I_Q_MEASUREMENT_MODE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2499
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_I_Q_WINDOW_WIDTH                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2500
#define RSDSA_ATTR_DU_ANALYSIS_CAPTURE_IQ_DATA_TO_FILENAME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2503
#define RSDSA_ATTR_DU_ANALYSIS_IQ_DATA_EXPORT_MAX_SAMPLES                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2504
#define RSDSA_ATTR_DU_ANALYSIS_IQ_DATA_EXPORT_TIMEOUT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2505
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_TIME_PLOT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2507
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2508
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_BER_VS_TIME_PLOT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2509
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2510
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_POWER_VS_TIME_PLOT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2511
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2512
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_SUBCARRIER_PLOT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2513
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_MER_VS_MINISLOT_PLOT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2514
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_CCDF_HISTOGRAM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2515
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_CCDF_PEAK                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2516
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2517
#define RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2518
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2519
#define RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2520
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_ECHO_PATTERN_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2521
#define RSDSA_ATTR_DU_ANALYSIS_DISPLAY_SPECTRUM_ANALYSIS_PLOT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2522
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2523
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2524
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2525
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2526
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_REFERENCE_LEVEL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2527
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2528
#define RSDSA_ATTR_DU_ANALYSIS_AUTO_SCALE_Y_AXIS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2529
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_AUTO_AVERAGE_COUNT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2530
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2531
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_TRACE_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2532
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_CAPTURE_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2533
#define RSDSA_ATTR_DU_ANALYSIS_RUN_SINGLE_SWEEP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2534
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2535
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2536
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_Y_POSITION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2537
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2538
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2539
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2540
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2541
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2542
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2543
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2544
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2545
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2546
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2547
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2548
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK RS_SPECIFIC_PUBLIC_ATTR_BASE + 2549
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2550
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2551
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2552
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2553
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2554
#define RSDSA_ATTR_DU_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2555
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_TRACE_DATA                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2556
#define RSDSA_ATTR_DU_ANALYSIS_MASK_VISIBLE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2557
#define RSDSA_ATTR_DU_ANALYSIS_CLEAR_MASK                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2558
#define RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_ALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2563
#define RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_NEXT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2564
#define RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_COUNT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2565
#define RSDSA_ATTR_DU_ANALYSIS_SYSTEM_ACQUISITION_LOG_CLEAR_ALL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2566
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_STANDARD                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2567
#define RSDSA_ATTR_DU_ANALYSIS_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2569
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_CONSTELLATION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2571
#define RSDSA_ATTR_DU_ANALYSIS_DOWNSTREAM_AUTO_LEVEL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2572
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_ATTENUATION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2573
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2574
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_CHANNEL_ID                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2575
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_ATTENUATION               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2576
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2577
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_UPSTREAM_TRAFFIC_DETECTED_THRESHOLD RS_SPECIFIC_PUBLIC_ATTR_BASE + 2580
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_OUTPUT_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2581
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2582
#define RSDSA_ATTR_DU_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2583
#define RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_RMS_MER_OVERALL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2584
#define RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2585
#define RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_POST_REED_SOLOMON        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2586
#define RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_PRE_LDPC                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2587
#define RSDSA_ATTR_DU_ANALYSIS_ALARM_THRESHOLD_BER_POST_LDPC                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2588
#define RSDSA_ATTR_ENABLE_DYNAMIC_SFD_ANALYSIS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2590
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_DEMODULATOR_LOCK_STATUS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2591
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_FEC_LOCK_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2592
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_RECEIVE_POWER                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2593
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_FREQUENCY_OFFSET                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2594
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_TIMING_OFFSET                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2595
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MINISLOTS_RECEIVED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2596
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BURST_COVERAGE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2597
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_SFD_PILOT_ACQUIRED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2598
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_DETECTED_UPSTREAM_MODE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2734
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MER_SIGN                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2599
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_RMS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2600
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_RMS_PARTIALY_EQUALIZED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2601
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PEAK                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2602
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PILOT_RMS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2603
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_PILOT_PEAK                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2604
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_PILOT_CONSTELLATIONS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2605
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_RMS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2606
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_MER_COMPLEMENTARY_PILOT_PEAK          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2607
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2608
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_DB             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2609
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2610
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2611
#define RSDSA_ATTR_SFD_ANALYSIS_RESET_ERROR_ANALYSIS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2612
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_ELAPSED_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2613
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BLOCKS_PROCESSED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2614
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BURSTS_EXPECTED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2615
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_BURSTS_RECEIVED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2616
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2617
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_MODE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2618
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_SOURCE  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2619
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_ERROR_ANALYSIS_TEST_DEPTH_PROGRESSION_SOURCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 2620
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2621
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_PRE_REED_SOLOMON_TOTAL_BITS       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2622
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2623
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_CORRECTED_BITS  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2624
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_BER_POST_REED_SOLOMON_UNCORRECTED_BITS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2625
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_CODEWORD_LENGTH_FILTER            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2626
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_CORRECTED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2627
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_UNCORRECTED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2628
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_TOTAL_CODEWORDS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2629
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_CORRECTED_CODEWORDS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2630
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_UNCORRECTED_CODEWORDS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2631
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_POST_LDPC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2632
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_POST_LDPC_CODEWORD_ERRORS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2633
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CER_AVERAGE_LDPC_ITERATIONS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2634
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_CONSTELLATION_PLOT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2635
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_I_Q_DENSITY_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2636
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_I_Q_MEASUREMENT_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2637
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_I_Q_WINDOW_WIDTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2638
#define RSDSA_ATTR_SFD_ANALYSIS_CAPTURE_IQ_DATA_TO_FILENAME                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2641
#define RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_MAX_SAMPLES                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2642
#define RSDSA_ATTR_SFD_ANALYSIS_IQ_DATA_EXPORT_TIMEOUT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2643
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_TIME_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2645
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2646
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_BER_VS_TIME_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2647
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2648
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_POWER_VS_TIME_PLOT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2649
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2650
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_SUBCARRIER_PLOT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2651
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_MER_VS_MINISLOT_PLOT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2652
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_CCDF_HISTOGRAM                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2653
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_CCDF_PEAK                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2654
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2655
#define RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2656
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2657
#define RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2658
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_ECHO_PATTERN_PLOT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2659
#define RSDSA_ATTR_SFD_ANALYSIS_DISPLAY_SPECTRUM_ANALYSIS_PLOT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2660
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_ALL_MARKERS_OFF           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2661
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_ALL_DELTA_MARKERS_OFF     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2662
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_CLEAR_ALL_MASKS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2663
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DISPLAY_ALL_MASKS_ENABLED RS_SPECIFIC_PUBLIC_ATTR_BASE + 2664
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_REFERENCE_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2665
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SCALE_PER_DIVISION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2666
#define RSDSA_ATTR_SFD_ANALYSIS_AUTO_SCALE_Y_AXIS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2667
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_AUTO_AVERAGE_COUNT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2668
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_MANUAL_AVERAGE_COUNT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2669
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_TRACE_MODE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2670
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_CAPTURE_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2671
#define RSDSA_ATTR_SFD_ANALYSIS_RUN_SINGLE_SWEEP                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2672
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_STATE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2673
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_X_POSITION         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2674
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_Y_POSITION         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2675
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_MAXIMUM_PEAK    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2676
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_LEFT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2677
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_PEAK_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2678
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2679
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2680
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2681
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_STATE        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2682
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_LINKED_REFERENCE_MARKER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2683
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_ABSOLUTE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2684
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_RELATIVE_X_POSITION RS_SPECIFIC_PUBLIC_ATTR_BASE + 2685
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_Y_POSITION   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2686
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_MAXIMUM_PEAK RS_SPECIFIC_PUBLIC_ATTR_BASE + 2687
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2688
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_PEAK_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2689
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_LEFT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2690
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_TO_NEXT_POINT_RIGHT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2691
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_SET_POWER_REFERENCE_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 2692
#define RSDSA_ATTR_SFD_ANALYSIS_SPECTRUM_ANALYSIS_DELTA_MARKER_BANDWIDTH_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2693
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_TRACE_DATA                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2694
#define RSDSA_ATTR_SFD_ANALYSIS_MASK_VISIBLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2695
#define RSDSA_ATTR_SFD_ANALYSIS_CLEAR_MASK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2696
#define RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2701
#define RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_NEXT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2702
#define RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_COUNT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2703
#define RSDSA_ATTR_SFD_ANALYSIS_SYSTEM_ACQUISITION_LOG_CLEAR_ALL            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2704
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFD_PILOT_FREQUENCY               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2705
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_PLOT_DATA_FILTER                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2706
#define RSDSA_ATTR_SFD_ANALYSIS_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2708
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_DOWNSTREAM_ATTENUATION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2711
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2712
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_UPSTREAM_ATTENUATION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2713
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2714
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_OUTPUT_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2717
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2718
#define RSDSA_ATTR_SFD_ANALYSIS_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2719
#define RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_RMS_MER_OVERALL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2720
#define RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_PRE_REED_SOLOMON        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2721
#define RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_POST_REED_SOLOMON       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2722
#define RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_PRE_LDPC                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2723
#define RSDSA_ATTR_SFD_ANALYSIS_ALARM_THRESHOLD_BER_POST_LDPC               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2724
#define RSDSA_ATTR_ENABLE_CM_RANGING_ANALYSIS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2726
#define RSDSA_ATTR_CM_RANGING_QUERY_BURST_DEMODULATOR_LOCK_STATUS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2727
#define RSDSA_ATTR_CM_RANGING_QUERY_FEC_LOCK_STATUS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2728
#define RSDSA_ATTR_CM_RANGING_QUERY_BURST_RECEIVE_POWER                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2729
#define RSDSA_ATTR_CM_RANGING_QUERY_FREQUENCY_OFFSET                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2730
#define RSDSA_ATTR_CM_RANGING_QUERY_BURST_TIMING_OFFSET                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2731
#define RSDSA_ATTR_CM_RANGING_QUERY_MINISLOTS_RECEIVED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2732
#define RSDSA_ATTR_CM_RANGING_QUERY_BURST_COVERAGE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2733
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_MER_SIGN                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2735
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_RMS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2736
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_RMS_PARTIALY_EQUALIZED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2737
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_PEAK                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2738
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_PILOT_RMS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2739
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_PILOT_PEAK                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2740
#define RSDSA_ATTR_CM_RANGING_QUERY_PILOT_CONSTELLATIONS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2741
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_COMPLEMENTARY_PILOT_RMS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2742
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_COMPLEMENTARY_PILOT_PEAK            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2743
#define RSDSA_ATTR_CM_RANGING_QUERY_COMPLEMENTARY_PILOT_CONSTELLATIONS      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2744
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_IUC_RMS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2745
#define RSDSA_ATTR_CM_RANGING_QUERY_MER_IUC_PEAK                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2746
#define RSDSA_ATTR_CM_RANGING_QUERY_IUC_CONSTELLATIONS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2747
#define RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_AMPLITUDE_DB               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2748
#define RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_AMPLITUDE_PERCENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2749
#define RSDSA_ATTR_CM_RANGING_QUERY_IQ_IMBALANCE_QUADRATURE_ERROR           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2750
#define RSDSA_ATTR_CM_RANGING_DISPLAY_CONSTELLATION_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2751
#define RSDSA_ATTR_CM_RANGING_DISPLAY_I_Q_DENSITY_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2752
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_I_Q_MEASUREMENT_MODE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2753
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_I_Q_WINDOW_WIDTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2754
#define RSDSA_ATTR_CM_RANGING_CAPTURE_IQ_DATA_TO_FILENAME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2757
#define RSDSA_ATTR_CM_RANGING_IQ_DATA_EXPORT_MAX_SAMPLES                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2758
#define RSDSA_ATTR_CM_RANGING_IQ_DATA_EXPORT_TIMEOUT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2759
#define RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_TIME_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2761
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_MER_VS_TIME_PLOT_FILTER_PERIOD      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2762
#define RSDSA_ATTR_CM_RANGING_DISPLAY_BER_VS_TIME_PLOT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2763
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_BER_VS_TIME_PLOT_FILTER_PERIOD      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2764
#define RSDSA_ATTR_CM_RANGING_DISPLAY_POWER_VS_TIME_PLOT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2765
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_POWER_VS_TIME_PLOT_FILTER_PERIOD    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2766
#define RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_SUBCARRIER_PLOT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2767
#define RSDSA_ATTR_CM_RANGING_DISPLAY_MER_VS_MINISLOT_PLOT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2768
#define RSDSA_ATTR_CM_RANGING_DISPLAY_CCDF_HISTOGRAM                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2769
#define RSDSA_ATTR_CM_RANGING_QUERY_CCDF_PEAK                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2770
#define RSDSA_ATTR_CM_RANGING_DISPLAY_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2771
#define RSDSA_ATTR_CM_RANGING_AUTO_SCALE_AMPLITUDE_PHASE_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2772
#define RSDSA_ATTR_CM_RANGING_DISPLAY_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2773
#define RSDSA_ATTR_CM_RANGING_AUTO_SCALE_AMPLITUDE_GROUP_DELAY_VS_SUBCARRIER_PLOT RS_SPECIFIC_PUBLIC_ATTR_BASE + 2774
#define RSDSA_ATTR_CM_RANGING_DISPLAY_ECHO_PATTERN_PLOT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2775
#define RSDSA_ATTR_CM_RANGING_DISPLAY_SPECTRUM_ANALYSIS_PLOT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2776
#define RSDSA_ATTR_CM_RANGING_QUERY_TRACE_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2777
#define RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2778
#define RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_NEXT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2779
#define RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_COUNT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2780
#define RSDSA_ATTR_CM_RANGING_SYSTEM_ACQUISITION_LOG_CLEAR_ALL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2781
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_PLOT_DATA_FILTER                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2782
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_FEC                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1993
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_MODE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2783
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_SFD_DOWNSTREAM_TRANSMIT_POWER       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2784
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_ASI_OUTPUT_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2785
#define RSDSA_ATTR_CM_RANGING_ED30_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2786
#define RSDSA_ATTR_CM_RANGING_ED30_CONFIGURE_CONSTELLATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2787
#define RSDSA_ATTR_CM_RANGING_D30D_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2788
#define RSDSA_ATTR_CM_RANGING_D30D_CONFIGURE_CONSTELLATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2789
#define RSDSA_ATTR_CONFIGURE_CHANNEL_INTERLEAVER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2790
#define RSDSA_ATTR_CM_RANGING_D30J_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2791
#define RSDSA_ATTR_CM_RANGING_D30J_CONFIGURE_CONSTELLATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2792
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2797
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_0_FREQUENCY         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2798
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_SPACING             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2799
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_CYCLIC_PREFIX_LENGTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2800
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_TIME_INTERLEAVER_DEPTH         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2801
#define RSDSA_ATTR_CM_RANGING_D31D_CONFIGURE_SUBCARRIER_CONFIG_FILE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2802
#define RSDSA_ATTR_CM_RANGING_QUERY_DOWNSTREAM_EQUALIZER_COEFFICIENTS       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2804
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_ATTENUATION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2807
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_DOWNSTREAM_PREAMPLIFIER_ENABLED     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2808
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2809
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_PLOT_DATA_FILTER_IUC       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2810
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2811
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_CONSTELLATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2812
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_CHANNEL_SYMBOL_RATE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2813
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FEC_PARAMETER_T                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2814
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_FEC_PARAMETER_K                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2815
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_LAST_CODEWORD_LENGTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2816
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_PATTERN               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2817
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_LENGTH                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2818
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_OFFSET                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2819
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_PREAMBLE_TYPE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2820
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_SCRAMBLER_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2821
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_SCRAMBLER_SEED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2822
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_GUARD_TIME_SIZE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2823
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2824
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_DEPTH RS_SPECIFIC_PUBLIC_ATTR_BASE + 2825
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_REED_SOLOMON_INTERLEAVER_BLOCK_SIZE RS_SPECIFIC_PUBLIC_ATTR_BASE + 2826
#define RSDSA_ATTR_CM_RANGING_D30A_CONFIGURE_MINISLOT_SIZE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2827
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2828
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_ROLL_OFF                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2829
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_CYCLIC_PREFIX_LENGTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2830
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_SYMBOLS_PER_FRAME              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2831
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_SUBCARRIER_SPACING             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2832
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_RANDOMIZER_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2833
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_RANDOMIZER_SEED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2834
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_EXCLUSION_BAND_START           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2835
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_EXCLUSION_BAND_WIDTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2836
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_MINISLOT_CONFIGURATION_MODE    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2837
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_MINISLOT_CONFIGURATION_FILE    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2838
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_CONSTELLATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2839
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_PILOT_PATTERN                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2840
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_STARTING_MINISLOT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2841
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_NUMBER_OF_MINISLOTS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2842
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FIRST_ACTIVE_SUBCARRIER        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2843
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_LAST_ACTIVE_SUBCARRIER         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2844
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_STARTING_MINISLOT_IN_FRAME RS_SPECIFIC_PUBLIC_ATTR_BASE + 2845
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_MINISLOTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2846
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_NUMBER_OF_SUBCARRIERS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2847
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_PATTERN RS_SPECIFIC_PUBLIC_ATTR_BASE + 2848
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_LENGTH RS_SPECIFIC_PUBLIC_ATTR_BASE + 2849
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_INITIAL_RANGING_PREAMBLE_OFFSET RS_SPECIFIC_PUBLIC_ATTR_BASE + 2850
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_STARTING_MINISLOT_IN_FRAME RS_SPECIFIC_PUBLIC_ATTR_BASE + 2851
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_MINISLOTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2852
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_NUMBER_OF_SUBCARRIERS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2853
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_PATTERN  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2854
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_FINE_RANGING_PREAMBLE_OFFSET   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2855
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_START_SUBCARRIER RS_SPECIFIC_PUBLIC_ATTR_BASE + 2856
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_SUBCARRIER_SKIP RS_SPECIFIC_PUBLIC_ATTR_BASE + 2857
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_STAGGER_ENABLED RS_SPECIFIC_PUBLIC_ATTR_BASE + 2858
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_WIDEBAND_PROBE_SYMBOL_IN_FRAME RS_SPECIFIC_PUBLIC_ATTR_BASE + 2859
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_BW_REQUEST_NUMBER_OF_MINISLOTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 2860
#define RSDSA_ATTR_CM_RANGING_D31U_CONFIGURE_BW_REQUEST_SUBSLOT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2861
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_OUTPUT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2864
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_IUC_UDP_PORT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2865
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_SFP_IUC_UDP_PORT_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2866
#define RSDSA_ATTR_CM_RANGING_UPSTREAM_AUTO_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2867
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_ATTENUATION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2868
#define RSDSA_ATTR_CM_RANGING_CONFIGURE_UPSTREAM_PREAMPLIFIER_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2869
#define RSDSA_ATTR_RF_INPUT_DOWNSTREAM_DETECT_ATTENUATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2872
#define RSDSA_ATTR_RF_INPUT_UPSTREAM_DETECT_ATTENUATION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2873
#define RSDSA_ATTR_RF_INPUT_DOWNSTREAM_ATTENUATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2874
#define RSDSA_ATTR_RF_INPUT_UPSTREAM_ATTENUATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2875
#define RSDSA_ATTR_RF_INPUT_DOWNSTREAM_OFFSET_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2876
#define RSDSA_ATTR_RF_INPUT_UPSTREAM_OFFSET_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2877
#define RSDSA_ATTR_RF_INPUT_DOWNSTREAM_PREAMPLIFIER_STATE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2878
#define RSDSA_ATTR_RF_INPUT_UPSTREAM_PREAMPLIFIER_STATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2879
#define RSDSA_ATTR_RF_INPUT_OPTIMIZE_MER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2880
#define RSDSA_ATTR_TRIGGER_INPUT_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2881
#define RSDSA_ATTR_TRIGGER_INPUT_OFFSET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2882
#define RSDSA_ATTR_TRIGGER_INPUT_SLOPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2883
#define RSDSA_ATTR_TRIGGER_INPUT_POLARITY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2884
#define RSDSA_ATTR_TRIGGER_INPUT_RESTART                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2885
#define RSDSA_ATTR_TRIGGER_INPUT_GENERATE_PULSE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2886
#define RSDSA_ATTR_TRIGGER_OUTPUT_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2887
#define RSDSA_ATTR_TRIGGER_OUTPUT_OFFSET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2888
#define RSDSA_ATTR_TRIGGER_OUTPUT_POLARITY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2889
#define RSDSA_ATTR_TRIGGER_OUTPUT_PULSE_LENGTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2890
#define RSDSA_ATTR_TRIGGER_OUTPUT_CYCLE_PERIOD                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2891
#define RSDSA_ATTR_INSTRUMENT_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2892
#define RSDSA_ATTR_INSTRUMENT_MODE_STATUS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2893
#define RSDSA_ATTR_SYSTEM_FIRMWARE_UPDATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2895
#define RSDSA_ATTR_SYSTEM_REBOOT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2900
#define RSDSA_ATTR_SYSTEM_SHUTDOWN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2901
#define RSDSA_ATTR_SYSTEM_PRESET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2902
#define RSDSA_ATTR_SYSTEM_PRESET_STORE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2903
#define RSDSA_ATTR_SYSTEM_REMOTE_DESKTOP                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2904
#define RSDSA_ATTR_POWER_UNITS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2905
#define RSDSA_ATTR_MEMORY_FILE_CATALOG_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2906
#define RSDSA_ATTR_MEMORY_CURRENT_DIRECTORY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2909
#define RSDSA_ATTR_MEMORY_DIRECTORY_CATALOG_LENGTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2918
#define RSDSA_ATTR_MEMORY_LOAD_FACTORY_CONFIGURATION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2929
#define RSDSA_ATTR_STATUS_PRESET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2938
#define RSDSA_ATTR_QUERY_EVENT_STATUS_REGISTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2939
#define RSDSA_ATTR_OPERATION_ENABLE_REGISTER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2940
#define RSDSA_ATTR_QUERY_OPERATIONAL_STATUS_REGISTER                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2941
#define RSDSA_ATTR_QUERY_OPERATION_EVENT_REGISTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2942
#define RSDSA_ATTR_QUESTIONABLE_ENABLE_REGISTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2943
#define RSDSA_ATTR_QUERY_QUESTIONABLE_STATUS_REGISTER                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2944
#define RSDSA_ATTR_QUERY_QUESTIONABLE_EVENT_REGISTER                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2945
#define RSDSA_ATTR_SYSTEM_NETWORK_RESET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2946
#define RSDSA_ATTR_SYSTEM_CONFIGURE_NETWORK_TYPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2948
#define RSDSA_ATTR_SYSTEM_QUERY_DSA_MAC_ADDRESS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2949
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_IP_ADDRESS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2950
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_SUBNET_MASK                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2951
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_GATEWAY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2952
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_DESTINATION_IP_ADDRESS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2953
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFP_PLUS_DATA_RATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2954
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_NETWORK_OPTION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2955
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_HOST_NAME                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2956
#define RSDSA_ATTR_SYSTEM_CONFIGURE_SFD_IP_ADDRESS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2957
#define RSDSA_ATTR_PRESET_DEVICE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2958
#define RSDSA_ATTR_RESET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2989
#define RSDSA_ATTR_ID_QUERY_RESPONSE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2974
#define RSDSA_ATTR_LOGGING                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2975
#define RSDSA_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2980
#define RSDSA_ATTR_ERROR_COUNT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2990
#define RSDSA_ATTR_ERROR_QUERY_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2991
#define RSDSA_ATTR_ERROR_QUERY_LIST                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2992
#define RSDSA_ATTR_QUEUE_NEXT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2985
#define RSDSA_ATTR_ERROR_QUERY_CODE_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2993
#define RSDSA_ATTR_ERROR_QUERY_CODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2994
#define RSDSA_ATTR_ERROR_CLEAR_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2995
#define RSDSA_ATTR_SYSTEM_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2996

// RsDSA_rngErrorAnalysisMode
#define RSDSA_VAL_ERROR_ANALYSIS_AUTO                                       0
#define RSDSA_VAL_ERROR_ANALYSIS_MANUAL                                     1

// RsDSA_rngErrorAnalysisTestDepth
#define RSDSA_VAL_TEST_DEPTH_1_BLOCK                                        0
#define RSDSA_VAL_TEST_DEPTH_10_BLOCKS                                      1
#define RSDSA_VAL_TEST_DEPTH_100_BLOCKS                                     2
#define RSDSA_VAL_TEST_DEPTH_1K_BLOCKS                                      3
#define RSDSA_VAL_TEST_DEPTH_10K_BLOCKS                                     4
#define RSDSA_VAL_TEST_DEPTH_100K_BLOCKS                                    5
#define RSDSA_VAL_TEST_DEPTH_1M_BLOCKS                                      6
#define RSDSA_VAL_TEST_DEPTH_10M_BLOCKS                                     7
#define RSDSA_VAL_TEST_DEPTH_100M_BLOCKS                                    8
#define RSDSA_VAL_TEST_DEPTH_1G_BLOCKS                                      9

// RsDSA_rngErrorAnalysisBERSource
#define RSDSA_VAL_BER_SOURCE_ESTIMATE_FROM_FEC                              0
#define RSDSA_VAL_BER_SOURCE_PSEUDONOISE23                                  1
#define RSDSA_VAL_BER_SOURCE_MAC_LFSR                                       2

// RsDSA_rngErrorAnalysisTestDepthSource
#define RSDSA_VAL_TEST_DEPTH_SOURCE_PRE_REED_SOLOMON                        0
#define RSDSA_VAL_TEST_DEPTH_SOURCE_POST_REED_SOLOMON                       1
#define RSDSA_VAL_TEST_DEPTH_SOURCE_PRE_VITERBI                             2
#define RSDSA_VAL_TEST_DEPTH_SOURCE_POST_VITERBI                            3
#define RSDSA_VAL_TEST_DEPTH_SOURCE_PRE_LDPC                                4
#define RSDSA_VAL_TEST_DEPTH_SOURCE_POST_LDPC                               5
#define RSDSA_VAL_TEST_DEPTH_SOURCE_POST_BCH                                6

// RsDSA_rngReferenceMarker
#define RSDSA_VAL_REFERENCE_MARKER_NORMAL                                   0
#define RSDSA_VAL_REFERENCE_MARKER_1                                        1
#define RSDSA_VAL_REFERENCE_MARKER_2                                        2
#define RSDSA_VAL_REFERENCE_MARKER_3                                        3
#define RSDSA_VAL_REFERENCE_MARKER_4                                        4
#define RSDSA_VAL_REFERENCE_MARKER_5                                        5

// RsDSA_rngInstrumentMode
#define RSDSA_VAL_INSTR_MODE_UNKNOWN                                        0
#define RSDSA_VAL_INSTR_MODE_ED30                                           1
#define RSDSA_VAL_INSTR_MODE_D30D                                           2
#define RSDSA_VAL_INSTR_MODE_D30J                                           3
#define RSDSA_VAL_INSTR_MODE_D31D                                           4
#define RSDSA_VAL_INSTR_MODE_JQD                                            5
#define RSDSA_VAL_INSTR_MODE_D30A                                           6
#define RSDSA_VAL_INSTR_MODE_D31U                                           7
#define RSDSA_VAL_INSTR_MODE_DUAN                                           8
#define RSDSA_VAL_INSTR_MODE_SUAN                                           9
#define RSDSA_VAL_INSTR_MODE_CMRA                                           10
#define RSDSA_VAL_INSTR_MODE_SAD                                            11
#define RSDSA_VAL_INSTR_MODE_SAUP                                           12

// RsDSA_rngCaptureMode
#define RSDSA_VAL_MEASUREMENT_CAPTURE_CONTINUOUS                            0
#define RSDSA_VAL_MEASUREMENT_CAPTURE_SINGLE                                1

// RsDSA_rngDetector
#define RSDSA_VAL_DETECTION_METHOD_SAMPLE                                   0
#define RSDSA_VAL_DETECTION_METHOD_RMS                                      1
#define RSDSA_VAL_DETECTION_METHOD_PEAK                                     2

// RsDSA_rngTraceHold
#define RSDSA_VAL_TRACE_MODE_WRITE                                          0
#define RSDSA_VAL_TRACE_MODE_MIN_HOLD                                       1
#define RSDSA_VAL_TRACE_MODE_MAX_HOLD                                       2
#define RSDSA_VAL_TRACE_MODE_MIN_MAX_HOLD                                   3

// RsDSA_rngACPFrequencyGridSpacing
#define RSDSA_VAL_ACP_FREQUENCY_GRID_6MHZ                                   0
#define RSDSA_VAL_ACP_FREQUENCY_GRID_7MHZ                                   1
#define RSDSA_VAL_ACP_FREQUENCY_GRID_8MHZ                                   2

// RsDSA_rngIQDensityMode
#define RSDSA_VAL_IQ_MEASUREMENT_MODE_HOLD                                  0
#define RSDSA_VAL_IQ_MEASUREMENT_MODE_SLIDING_WINDOW                        1

// RsDSA_rngTimePlotFilterPeriod
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_MINUTE                       0
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_10_MINUTES                   1
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_HOUR                         2
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_DAY                          3
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_WEEK                         4
#define RSDSA_VAL_TIME_PLOT_FILTER_PERIOD_LAST_MONTH                        5

// RsDSA_rngD31UPlotDataFilter
#define RSDSA_VAL_PLOT_DATA_FILTER_OVERALL                                  0
#define RSDSA_VAL_PLOT_DATA_FILTER_IUC                                      2
#define RSDSA_VAL_PLOT_DATA_FILTER_PILOT                                    3
#define RSDSA_VAL_PLOT_DATA_FILTER_COMPLEMENTARY_PILOT                      4

// RsDSA_rngD31DProfileFilter
#define RSDSA_VAL_PROFILE_FILTER_ALL                                        0
#define RSDSA_VAL_PROFILE_FILTER_A                                          1
#define RSDSA_VAL_PROFILE_FILTER_B                                          2
#define RSDSA_VAL_PROFILE_FILTER_C                                          3
#define RSDSA_VAL_PROFILE_FILTER_D                                          4
#define RSDSA_VAL_PROFILE_FILTER_E                                          5
#define RSDSA_VAL_PROFILE_FILTER_F                                          6
#define RSDSA_VAL_PROFILE_FILTER_G                                          7
#define RSDSA_VAL_PROFILE_FILTER_H                                          8
#define RSDSA_VAL_PROFILE_FILTER_I                                          9
#define RSDSA_VAL_PROFILE_FILTER_J                                          10
#define RSDSA_VAL_PROFILE_FILTER_K                                          11
#define RSDSA_VAL_PROFILE_FILTER_L                                          12
#define RSDSA_VAL_PROFILE_FILTER_M                                          13
#define RSDSA_VAL_PROFILE_FILTER_N                                          14
#define RSDSA_VAL_PROFILE_FILTER_O                                          15
#define RSDSA_VAL_PROFILE_FILTER_P                                          16

// RsDSA_rngDataFilterIUC
#define RSDSA_VAL_DATA_FILTER_IUC0                                          0
#define RSDSA_VAL_DATA_FILTER_IUC1                                          1
#define RSDSA_VAL_DATA_FILTER_IUC2                                          2
#define RSDSA_VAL_DATA_FILTER_IUC3                                          3
#define RSDSA_VAL_DATA_FILTER_IUC4                                          4
#define RSDSA_VAL_DATA_FILTER_IUC5                                          5
#define RSDSA_VAL_DATA_FILTER_IUC6                                          6
#define RSDSA_VAL_DATA_FILTER_IUC7                                          7
#define RSDSA_VAL_DATA_FILTER_IUC8                                          8
#define RSDSA_VAL_DATA_FILTER_IUC9                                          9
#define RSDSA_VAL_DATA_FILTER_IUC10                                         10
#define RSDSA_VAL_DATA_FILTER_IUC11                                         11
#define RSDSA_VAL_DATA_FILTER_IUC12                                         12
#define RSDSA_VAL_DATA_FILTER_IUC13                                         13
#define RSDSA_VAL_DATA_FILTER_IUC14                                         14
#define RSDSA_VAL_DATA_FILTER_IUC15                                         15
#define RSDSA_VAL_DATA_FILTER_ALL                                           16
#define RSDSA_VAL_DATA_FILTER_WIDEBAND_PROBE                                17

// RsDSA_rngCodewordLength
#define RSDSA_VAL_CODEWORD_LENGTH_ALL                                       0
#define RSDSA_VAL_CODEWORD_LENGTH_SHORT                                     1
#define RSDSA_VAL_CODEWORD_LENGTH_MEDIUM                                    2
#define RSDSA_VAL_CODEWORD_LENGTH_LONG                                      3

// RsDSA_rngEqualizerState
#define RSDSA_VAL_EQUALIZER_OFF                                             0
#define RSDSA_VAL_EQUALIZER_ON                                              1
#define RSDSA_VAL_EQUALIZER_FREEZE                                          2

// RsDSA_rngSlope
#define RSDSA_VAL_SLOPE_NEGATIVE                                            0
#define RSDSA_VAL_SLOPE_POSITIVE                                            1

// RsDSA_rngPolarity
#define RSDSA_VAL_POLARITY_LOW                                              0
#define RSDSA_VAL_POLARITY_HIGH                                             1

// RsDSA_rngUSBDrive
#define RSDSA_VAL_USB_DRIVE_1                                               1
#define RSDSA_VAL_USB_DRIVE_2                                               2
#define RSDSA_VAL_USB_DRIVE_3                                               3
#define RSDSA_VAL_USB_DRIVE_4                                               4

// RsDSA_rngPayloadCaptureFilterProfile
#define RSDSA_VAL_PROFILE_FILTER_PLC                                        17

// RsDSA_rngLockStatus
#define RSDSA_VAL_UNLOCKED                                                  0
#define RSDSA_VAL_LOCKED                                                    1

// RsDSA_rngChannelModulation
#define RSDSA_VAL_CHANNEL_MODULATION_QPSK                                   0
#define RSDSA_VAL_CHANNEL_MODULATION_QAM8                                   1
#define RSDSA_VAL_CHANNEL_MODULATION_QAM16                                  2
#define RSDSA_VAL_CHANNEL_MODULATION_QAM32                                  3
#define RSDSA_VAL_CHANNEL_MODULATION_QAM64                                  4
#define RSDSA_VAL_CHANNEL_MODULATION_QAM128                                 5
#define RSDSA_VAL_CHANNEL_MODULATION_QAM256                                 6
#define RSDSA_VAL_CHANNEL_MODULATION_QAM512                                 7
#define RSDSA_VAL_CHANNEL_MODULATION_QAM1024                                8
#define RSDSA_VAL_CHANNEL_MODULATION_QAM2048                                9
#define RSDSA_VAL_CHANNEL_MODULATION_QAM4096                                10

// RsDSA_rngChannelAcquisitionMode
#define RSDSA_VAL_CHANNEL_ACQUISITION_MODE_AUTO                             0
#define RSDSA_VAL_CHANNEL_ACQUISITION_MODE_MANUAL                           1

// RsDSA_rngOFDMFFTSize
#define RSDSA_VAL_CHANNEL_FFT_SIZE_4K                                       1
#define RSDSA_VAL_CHANNEL_FFT_SIZE_8K                                       2

// RsDSA_rngOFDMAFFTSize
#define RSDSA_VAL_CHANNEL_FFT_SIZE_2K                                       0

// RsDSA_rngCyclicPrefixLength
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S96                                 0
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S128                                1
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S160                                2
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S192                                3
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S224                                4
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S256                                5
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S288                                6
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S320                                7
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S384                                8
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S512                                9
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S640                                10
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S768                                11
#define RSDSA_VAL_CHANNEL_CYCLIC_PREFIX_S1024                               12

// RsDSA_rngFECAnnex
#define RSDSA_VAL_FEC_ANNEX_A                                               0
#define RSDSA_VAL_FEC_ANNEX_B                                               1
#define RSDSA_VAL_FEC_ANNEX_C                                               2

// RsDSA_rngRollOffPeriod
#define RSDSA_VAL_CHANNEL_ROLL_OFF_S0                                       0
#define RSDSA_VAL_CHANNEL_ROLL_OFF_S64                                      1
#define RSDSA_VAL_CHANNEL_ROLL_OFF_S128                                     2
#define RSDSA_VAL_CHANNEL_ROLL_OFF_S192                                     3
#define RSDSA_VAL_CHANNEL_ROLL_OFF_S256                                     4

// RsDSA_rngChannelPhaseTrackingLoopBandwidth
#define RSDSA_VAL_CHANNEL_PHASE_TRACKING_LOOP_BANDWIDTH_FINE                0
#define RSDSA_VAL_CHANNEL_PHASE_TRACKING_LOOP_BANDWIDTH_MEDIUM              1
#define RSDSA_VAL_CHANNEL_PHASE_TRACKING_LOOP_BANDWIDTH_COARSE              2

// RsDSA_rngD30Interleaver
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_1J                               0
#define RSDSA_VAL_CHANNEL_INTERLEAVER_64I_2J                                1
#define RSDSA_VAL_CHANNEL_INTERLEAVER_32I_4J                                2
#define RSDSA_VAL_CHANNEL_INTERLEAVER_16I_8J                                3
#define RSDSA_VAL_CHANNEL_INTERLEAVER_8I_16J                                4
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_2J                               5
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_3J                               6
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_4J                               7
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_5J                               8
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_6J                               9
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_7J                               10
#define RSDSA_VAL_CHANNEL_INTERLEAVER_128I_8J                               11

// RsDSA_rngPrefilterLength
#define RSDSA_VAL_CHANNEL_PREFILTER_LENGTH_SHORT                            0
#define RSDSA_VAL_CHANNEL_PREFILTER_LENGTH_MEDIUM                           1
#define RSDSA_VAL_CHANNEL_PREFILTER_LENGTH_LONG                             2

// RsDSA_rngMinislotConfigMode
#define RSDSA_VAL_CHANNEL_MINISLOT_CONFIGURATION_FILE                       0
#define RSDSA_VAL_CHANNEL_MINISLOT_CONFIGURATION_MANUAL                     1

// RsDSA_rngD30ASymbolRate
#define RSDSA_VAL_CHANNEL_SRATE_S1280                                       0
#define RSDSA_VAL_CHANNEL_SRATE_S2560                                       1
#define RSDSA_VAL_CHANNEL_SRATE_S5120                                       2

// RsDSA_rngLastCodewordLength
#define RSDSA_VAL_LAST_CODEWORD_LENGTH_FIXED                                0
#define RSDSA_VAL_LAST_CODEWORD_LENGTH_SHORT                                1

// RsDSA_rngPreambleType
#define RSDSA_VAL_CHANNEL_PREAMBLE_TYPE_QPSK0                               0
#define RSDSA_VAL_CHANNEL_PREAMBLE_TYPE_QPSK1                               1

// RsDSA_rngReedSolomonMode
#define RSDSA_VAL_REED_SOLOMON_MODE_DISABLED                                0
#define RSDSA_VAL_REED_SOLOMON_MODE_FIXED                                   1
#define RSDSA_VAL_REED_SOLOMON_MODE_DYNAMIC                                 2

// RsDSA_rngChannelSchedule
#define RSDSA_CHANNEL_SCHEDULE_PERIODIC                                     0
#define RSDSA_CHANNEL_SCHEDULE_SINGLE                                       1

// RsDSA_rngPowerUnits
#define RSDSA_VAL_POWER_UNITS_DBM                                           0
#define RSDSA_VAL_POWER_UNITS_DBV                                           1
#define RSDSA_VAL_POWER_UNITS_DBMV                                          2
#define RSDSA_VAL_POWER_UNITS_DBUV                                          3

// RsDSA_rngMERSign
#define RSDSA_VAL_MER_SIGN_NEGATIVE                                         0
#define RSDSA_VAL_MER_SIGN_POSITIVE                                         1

// RsDSA_rngSystemPreset
#define RSDSA_VAL_SYSTEM_PRESET_FACTORY                                     0
#define RSDSA_VAL_SYSTEM_PRESET_USER                                        1

// RsDSA_rngJQDRollOff
#define RSDSA_VAL_JQD_ROLL_OFF_12                                           12
#define RSDSA_VAL_JQD_ROLL_OFF_13                                           13
#define RSDSA_VAL_JQD_ROLL_OFF_14                                           14
#define RSDSA_VAL_JQD_ROLL_OFF_15                                           15
#define RSDSA_VAL_JQD_ROLL_OFF_16                                           16
#define RSDSA_VAL_JQD_ROLL_OFF_17                                           17
#define RSDSA_VAL_JQD_ROLL_OFF_18                                           18
#define RSDSA_VAL_JQD_ROLL_OFF_19                                           19
#define RSDSA_VAL_JQD_ROLL_OFF_20                                           20

// RsDSA_rngD31DPlotDataFilter
#define RSDSA_VAL_PLOT_DATA_FILTER_PROFILE                                  1
#define RSDSA_VAL_PLOT_DATA_FILTER_PLC                                      5
#define RSDSA_VAL_PLOT_DATA_FILTER_NCP                                      6
#define RSDSA_VAL_PLOT_DATA_FILTER_ZERO_BIT_LOADED                          7

// RsDSA_rngNetworkType
#define RSDSA_VAL_NETWORK_TYPE_STATIC                                       0
#define RSDSA_VAL_NETWORK_TYPE_DHCP                                         1

// RsDSA_rngSFPPlusDataRate
#define RSDSA_VAL_SFP_PLUS_DATA_RATE_1GBPS                                  0
#define RSDSA_VAL_SFP_PLUS_DATA_RATE_10GBPS                                 1

// RsDSA_rngSFDNetworkOption
#define RSDSA_VAL_SFD_NETWORK_OPTION_IP_ADDRESS                             0
#define RSDSA_VAL_SFD_NETWORK_OPTION_IP_HOST_NAME                           1

//repcap Marker
#define RSDSA_REPCAP_MARKER_M1                                              1
#define RSDSA_REPCAP_MARKER_M2                                              2
#define RSDSA_REPCAP_MARKER_M3                                              3
#define RSDSA_REPCAP_MARKER_M4                                              4
#define RSDSA_REPCAP_MARKER_M5                                              5

//repcap Delta
#define RSDSA_REPCAP_DELTA_DM1                                              1
#define RSDSA_REPCAP_DELTA_DM2                                              2
#define RSDSA_REPCAP_DELTA_DM3                                              3
#define RSDSA_REPCAP_DELTA_DM4                                              4
#define RSDSA_REPCAP_DELTA_DM5                                              5

//repcap Mask
#define RSDSA_REPCAP_MASK_MASK1                                             1
#define RSDSA_REPCAP_MASK_MASK2                                             2
#define RSDSA_REPCAP_MASK_MASK3                                             3
#define RSDSA_REPCAP_MASK_MASK4                                             4
#define RSDSA_REPCAP_MASK_MASK5                                             5

//repcap SavedState
#define RSDSA_REPCAP_SAVEDSTATE_SAVEDSTATE1                                 1
#define RSDSA_REPCAP_SAVEDSTATE_SAVEDSTATE2                                 2
#define RSDSA_REPCAP_SAVEDSTATE_SAVEDSTATE3                                 3
#define RSDSA_REPCAP_SAVEDSTATE_SAVEDSTATE4                                 4
#define RSDSA_REPCAP_SAVEDSTATE_SAVEDSTATE5                                 5

//repcap Trace
#define RSDSA_REPCAP_TRACE_ACTIVETRACE                                      1
#define RSDSA_REPCAP_TRACE_MINHOLD                                          2
#define RSDSA_REPCAP_TRACE_MAXHOLD                                          3
#define RSDSA_REPCAP_TRACE_CONSTELLATION                                    4
#define RSDSA_REPCAP_TRACE_IQDENSITY                                        5
#define RSDSA_REPCAP_TRACE_MERVSTIME                                        6
#define RSDSA_REPCAP_TRACE_MERPEAKVSTIME                                    7
#define RSDSA_REPCAP_TRACE_MERVSMINISLOT                                    8
#define RSDSA_REPCAP_TRACE_MERVSSUBCARRIER                                  9
#define RSDSA_REPCAP_TRACE_AMPLITUDEVSFREQUENCY                             10
#define RSDSA_REPCAP_TRACE_PHASEVSFREQUENCY                                 11
#define RSDSA_REPCAP_TRACE_GROUPDELAYVSFREQUENCY                            12
#define RSDSA_REPCAP_TRACE_ECHOPATTERN                                      13
#define RSDSA_REPCAP_TRACE_POWERVSTIME                                      14
#define RSDSA_REPCAP_TRACE_CCDF                                             15
#define RSDSA_REPCAP_TRACE_BERVSTIMEPRERS                                   16
#define RSDSA_REPCAP_TRACE_BERVSTIMEPOSTRS                                  17
#define RSDSA_REPCAP_TRACE_BERVSTIMEPREVITERBI                              18
#define RSDSA_REPCAP_TRACE_BERVSTIMEPOSTVITERBI                             19
#define RSDSA_REPCAP_TRACE_BERVSTIMEPLC                                     20
#define RSDSA_REPCAP_TRACE_BERVSTIMENCP                                     21
#define RSDSA_REPCAP_TRACE_BERVSTIMEPRELDPC                                 22
#define RSDSA_REPCAP_TRACE_BERVSTIMEPOSTLDPC                                23
#define RSDSA_REPCAP_TRACE_BERVSTIMEPOSTBCH                                 24

//repcap IUC_ATDMA
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT1                                      1
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT2                                      2
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT3                                      3
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT4                                      4
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT5                                      5
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT6                                      6
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT7                                      7
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT8                                      8
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT9                                      9
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT10                                     10
#define RSDSA_REPCAP_IUC_ATDMA_IUC_AT11                                     11

//repcap IUC_OFDMA
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF1                                      1
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF2                                      2
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF3                                      3
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF4                                      4
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF5                                      5
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF6                                      6
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF7                                      7
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF8                                      8
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF9                                      9
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF10                                     10
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF11                                     11
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF12                                     12
#define RSDSA_REPCAP_IUC_OFDMA_IUC_OF13                                     13

//repcap Profile
#define RSDSA_REPCAP_PROFILE_PROFILEA                                       1
#define RSDSA_REPCAP_PROFILE_PROFILEB                                       2
#define RSDSA_REPCAP_PROFILE_PROFILEC                                       3
#define RSDSA_REPCAP_PROFILE_PROFILED                                       4
#define RSDSA_REPCAP_PROFILE_PROFILEE                                       5
#define RSDSA_REPCAP_PROFILE_PROFILEF                                       6
#define RSDSA_REPCAP_PROFILE_PROFILEG                                       7
#define RSDSA_REPCAP_PROFILE_PROFILEH                                       8
#define RSDSA_REPCAP_PROFILE_PROFILEI                                       9
#define RSDSA_REPCAP_PROFILE_PROFILEJ                                       10
#define RSDSA_REPCAP_PROFILE_PROFILEK                                       11
#define RSDSA_REPCAP_PROFILE_PROFILEL                                       12
#define RSDSA_REPCAP_PROFILE_PROFILEM                                       13
#define RSDSA_REPCAP_PROFILE_PROFILEN                                       14
#define RSDSA_REPCAP_PROFILE_PROFILEO                                       15
#define RSDSA_REPCAP_PROFILE_PROFILEP                                       16

//repcap SidID
#define RSDSA_REPCAP_SIDID_SIDID1                                           1
#define RSDSA_REPCAP_SIDID_SIDID2                                           2
#define RSDSA_REPCAP_SIDID_SIDID3                                           3
#define RSDSA_REPCAP_SIDID_SIDID4                                           4
#define RSDSA_REPCAP_SIDID_SIDID5                                           5
#define RSDSA_REPCAP_SIDID_SIDID6                                           6
#define RSDSA_REPCAP_SIDID_SIDID7                                           7
#define RSDSA_REPCAP_SIDID_SIDID8                                           8
#define RSDSA_REPCAP_SIDID_SIDID9                                           9
#define RSDSA_REPCAP_SIDID_SIDID10                                          10

//repcap Band
#define RSDSA_REPCAP_BAND_BAND1                                             1
#define RSDSA_REPCAP_BAND_BAND2                                             2
#define RSDSA_REPCAP_BAND_BAND3                                             3
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSDSA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSDSA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSDSA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSDSA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

// Mask points valid range limits
#define RSDSA_VAL_MASK_POINTS_MIN 1
#define RSDSA_VAL_MASK_POINTS_MAX 20

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus rsdsa_init(ViRsrc     resourceName,
                    ViBoolean  IDQuery,
                    ViBoolean  resetDevice,
                    ViSession* instrSession);

ViStatus rsdsa_InitWithOptions(ViRsrc     resourceName,
                               ViBoolean  IDQuery,
                               ViBoolean  resetDevice,
                               ViString   optionString,
                               ViSession* instrSession);

ViStatus rsdsa_EnableEuroDOCSIS30Mode(ViSession instrSession);

ViStatus rsdsa_ED30QueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus);

ViStatus rsdsa_ED30QueryDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLockStatus);

ViStatus rsdsa_ED30QueryMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  MPEGLockStatus);

ViStatus rsdsa_ED30QuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower);

ViStatus rsdsa_ED30QueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_ED30QuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset);

ViStatus rsdsa_ED30QueryChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate);

ViStatus rsdsa_ED30QueryInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead);

ViStatus rsdsa_ED30ConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled);

ViStatus rsdsa_ED30ConfigureMERSign(ViSession instrSession,
                                    ViInt32   MERSign);

ViStatus rsdsa_ED30QueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* RMSCompensatedOverall);

ViStatus rsdsa_ED30QueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* RMSEqualized);

ViStatus rsdsa_ED30QueryMERPeak(ViSession instrSession,
                                ViReal64* MERPeak);

ViStatus rsdsa_ED30ConfigureErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   BERSource,
                                          ViInt32   testDepthProgressionSource);

ViStatus rsdsa_ED30ResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_ED30QueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime);

ViStatus rsdsa_ED30QueryBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed);

ViStatus rsdsa_ED30QueryBERPreReedSolomon(ViSession instrSession,
                                          ViReal64* BERPreRS);

ViStatus rsdsa_ED30QueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits);

ViStatus rsdsa_ED30QueryBERPostReedSolomon(ViSession instrSession,
                                           ViReal64* BERPostRS);

ViStatus rsdsa_ED30QueryBERPostReedSolomonTotalBits(ViSession instrSession,
                                                    ViReal64* postRSTotalBits);

ViStatus rsdsa_ED30QueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* postRSCorrectedBits);

ViStatus rsdsa_ED30QueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* postRSUncorrectedBits);

ViStatus rsdsa_ED30QueryCERCorrected(ViSession instrSession,
                                     ViReal64* CERCorrected);

ViStatus rsdsa_ED30QueryCERUncorrected(ViSession instrSession,
                                       ViReal64* CERUncorrected);

ViStatus rsdsa_ED30QueryCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords);

ViStatus rsdsa_ED30QueryCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords);

ViStatus rsdsa_ED30QueryCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords);

ViStatus rsdsa_ED30DisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_ED30DisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_ED30ConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth);

ViStatus rsdsa_ED30SetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom);

ViStatus rsdsa_ED30CaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_ED30DisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ED30ConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_ED30DisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ED30ConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_ED30DisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ED30ConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period);

ViStatus rsdsa_ED30DisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_ED30QueryCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_ED30DisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_ED30AutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_ED30DisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_ED30AutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_ED30DisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_ED30QueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead);

ViStatus rsdsa_ED30QuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_ED30QuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_ED30QuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  logCount);

ViStatus rsdsa_ED30SystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_ED30ConfigureGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViBoolean spectralInversionEnabled,
                                            ViInt32   equalizerState,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled,
                                            ViBoolean ASIOutputEnabled,
                                            ViInt32   phaseTrackingLoopBandwidth);

ViStatus rsdsa_ED30QueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead);

ViStatus rsdsa_ED30AutoLevel(ViSession instrSession);

ViStatus rsdsa_ED30ConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  RMSMEROverallThreshold,
                                            ViReal64  BERPreRSThreshold,
                                            ViReal64  BERPostRSThreshold);

ViStatus rsdsa_EnableDOCSIS30Mode(ViSession instrSession);

ViStatus rsdsa_QueryD30DDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLock);

ViStatus rsdsa_QueryD30DDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLock);

ViStatus rsdsa_QueryD30DMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  MPEGLock);

ViStatus rsdsa_QueryD30DSignalPower(ViSession instrSession,
                                    ViReal64* signalPower);

ViStatus rsdsa_QueryD30DFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_QueryD30DSymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset);

ViStatus rsdsa_QueryD30DChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate);

ViStatus rsdsa_QueryD30DInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureD30DMERCompensation(ViSession instrSession,
                                            ViBoolean enabled,
                                            ViInt32   sign);

ViStatus rsdsa_QueryD30DMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* compensatedOverall);

ViStatus rsdsa_QueryD30DMERRMSEqualized(ViSession instrSession,
                                        ViReal64* RMSEqualized);

ViStatus rsdsa_QueryD30DMERPeak(ViSession instrSession,
                                ViReal64* MERPeak);

ViStatus rsdsa_ConfigureD30DErrorAnalysis(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   BERSource,
                                          ViInt32   progressionSource);

ViStatus rsdsa_D30DResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_QueryD30DBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime);

ViStatus rsdsa_QueryD30DBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed);

ViStatus rsdsa_QueryD30DBERPreViterbi(ViSession instrSession,
                                      ViReal64* preViterbi);

ViStatus rsdsa_QueryD30DBERPreViterbiTotalBits(ViSession instrSession,
                                               ViReal64* totalBits);

ViStatus rsdsa_QueryD30DBERPostViterbi(ViSession instrSession,
                                       ViReal64* postViterbi);

ViStatus rsdsa_QueryD30DBERPostViterbiTotalBits(ViSession instrSession,
                                                ViReal64* totalBits);

ViStatus rsdsa_QueryD30DQueryBERPostReedSolomon(ViSession instrSession,
                                                ViReal64* postReedSolomon);

ViStatus rsdsa_QueryD30DBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* correctedBits);

ViStatus rsdsa_QueryD30DBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* uncorrectedBits);

ViStatus rsdsa_QueryD30DCERCorrected(ViSession instrSession,
                                     ViReal64* corrected);

ViStatus rsdsa_QueryD30DCERUncorrected(ViSession instrSession,
                                       ViReal64* uncorrected);

ViStatus rsdsa_QueryD30DCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords);

ViStatus rsdsa_QueryD30DCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords);

ViStatus rsdsa_QueryD30DCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords);

ViStatus rsdsa_D30DDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_D30DDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30DIQMeasurement(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   windowWidth);

ViStatus rsdsa_ConfigureD30DIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom);

ViStatus rsdsa_CaptureD30DIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_D30DDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D30DDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D30DDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30DMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_ConfigureD30DBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_ConfigureD30DPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period);

ViStatus rsdsa_D30DDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_QueryD30DCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_D30DDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30DAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30DDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30DAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30DDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_QueryD30DTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead);

ViStatus rsdsa_QueryD30DSystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_QueryD30DSystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_QueryD30DSystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count);

ViStatus rsdsa_D30DSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_D30DAutoLevel(ViSession instrSession);

ViStatus rsdsa_ConfigureD30D(ViSession instrSession,
                             ViReal64  frequency,
                             ViInt32   constellation,
                             ViBoolean spectralInversionEnabled,
                             ViInt32   equalizerState,
                             ViReal64  attenuation,
                             ViBoolean preamplifierEnabled,
                             ViBoolean ASIOutputEnabled,
                             ViInt32   phaseTrackingLoopBandwidth);

ViStatus rsdsa_QueryD30DEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureD30DAlarmThreshold(ViSession instrSession,
                                           ViReal64  RMSMEROverall,
                                           ViReal64  BERPreViterbi,
                                           ViReal64  BERPostViterbi,
                                           ViReal64  BERPostReedSolomon);

ViStatus rsdsa_EnableSCQAMJ83CMode(ViSession instrSession);

ViStatus rsdsa_D30JQueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus);

ViStatus rsdsa_D30JQueryDecoderLockStatus(ViSession instrSession,
                                          ViInt32*  decoderLockStatus);

ViStatus rsdsa_D30JQueryMPEGLockStatus(ViSession instrSession,
                                       ViInt32*  MPEGLockStatus);

ViStatus rsdsa_D30JQuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower);

ViStatus rsdsa_D30JQueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_D30JQuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset);

ViStatus rsdsa_D30JQueryChannelBitRate(ViSession instrSession,
                                       ViReal64* channelBitRate);

ViStatus rsdsa_D30JQueryInterleaver(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    interleaver[],
                                    ViInt32*  bytesRead);

ViStatus rsdsa_D30JConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled);

ViStatus rsdsa_D30JConfigureMERSign(ViSession instrSession,
                                    ViInt32   MERSign);

ViStatus rsdsa_D30JQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* RMSCompensatedOverall);

ViStatus rsdsa_D30JQueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* RMSEqualized);

ViStatus rsdsa_D30JQueryMERPeak(ViSession instrSession,
                                ViReal64* MERPeak);

ViStatus rsdsa_D30JConfigureErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   BERSource,
                                          ViInt32   testDepthProgressionSource);

ViStatus rsdsa_D30JResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_D30JQueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime);

ViStatus rsdsa_D30JQueryBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed);

ViStatus rsdsa_D30JQueryBERPreReedSolomon(ViSession instrSession,
                                          ViReal64* BERPreRS);

ViStatus rsdsa_D30JQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits);

ViStatus rsdsa_D30JQueryBERPostReedSolomon(ViSession instrSession,
                                           ViReal64* BERPostRS);

ViStatus rsdsa_D30JQueryBERPostReedSolomonTotalBits(ViSession instrSession,
                                                    ViReal64* postRSTotalBits);

ViStatus rsdsa_D30JQueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* postRSCorrectedBits);

ViStatus rsdsa_D30JQueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* postRSUncorrectedBits);

ViStatus rsdsa_D30JQueryCERCorrected(ViSession instrSession,
                                     ViReal64* CERCorrected);

ViStatus rsdsa_D30JQueryCERUncorrected(ViSession instrSession,
                                       ViReal64* CERUncorrected);

ViStatus rsdsa_D30JQueryCERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords);

ViStatus rsdsa_D30JQueryCERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords);

ViStatus rsdsa_D30JQueryCERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords);

ViStatus rsdsa_D30JDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_D30JDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_D30JConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth);

ViStatus rsdsa_D30JSetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom);

ViStatus rsdsa_D30JCaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_D30JDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D30JConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D30JDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D30JConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D30JDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D30JConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period);

ViStatus rsdsa_D30JDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_D30JQueryCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_D30JDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30JAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30JDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30JAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30JDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_D30JQueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead);

ViStatus rsdsa_D30JQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_D30JQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_D30JQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count);

ViStatus rsdsa_D30JSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_D30JConfigureGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViBoolean spectralInversionEnabled,
                                            ViInt32   equalizerState,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled,
                                            ViBoolean ASIOutputEnabled,
                                            ViInt32   phaseTrackingLoopBandwidth);

ViStatus rsdsa_D30JQueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead);

ViStatus rsdsa_D30JAutoLevel(ViSession instrSession);

ViStatus rsdsa_D30JConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  RMSMEROverallThreshold,
                                            ViReal64  BERPreRSThreshold,
                                            ViReal64  BERPostRSThreshold);

ViStatus rsdsa_EnableJ83QAMMode(ViSession instrSession);

ViStatus rsdsa_JQDQueryDemodulatorLockStatus(ViSession instrSession,
                                             ViInt32*  demodulatorLockStatus);

ViStatus rsdsa_JQDQueryDecoderLockStatus(ViSession instrSession,
                                         ViInt32*  decoderLockStatus);

ViStatus rsdsa_JQDQueryMPEGLockStatus(ViSession instrSession,
                                      ViInt32*  MPEGLockStatus);

ViStatus rsdsa_JQDQuerySignalPower(ViSession instrSession,
                                   ViReal64* signalPower);

ViStatus rsdsa_JQDQueryFrequencyOffset(ViSession instrSession,
                                       ViReal64* frequencyOffset);

ViStatus rsdsa_JQDQuerySymbolClockOffset(ViSession instrSession,
                                         ViReal64* symbolClockOffset);

ViStatus rsdsa_JQDQueryChannelBitRate(ViSession instrSession,
                                      ViReal64* channelBitRate);

ViStatus rsdsa_JQDQueryInterleaver(ViSession instrSession,
                                   ViInt32   bufferSize,
                                   ViChar    interleaver[],
                                   ViInt32*  bytesRead);

ViStatus rsdsa_JQDConfigureMERCompensationEnabled(ViSession instrSession,
                                                  ViBoolean enabled);

ViStatus rsdsa_JQDConfigureMERSign(ViSession instrSession,
                                   ViInt32   MERSign);

ViStatus rsdsa_JQDQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                ViBoolean* RMSCompensatedOverall);

ViStatus rsdsa_JQDQueryMERRMSEqualized(ViSession instrSession,
                                       ViReal64* RMSEqualized);

ViStatus rsdsa_JQDQueryMERPeak(ViSession instrSession,
                               ViReal64* MERPeak);

ViStatus rsdsa_JQDConfigureErrorAnalysis(ViSession instrSession,
                                         ViInt32   errorAnalysisMode,
                                         ViInt32   manualTestDepth,
                                         ViInt32   BERSource,
                                         ViInt32   testDepthProgressionSource);

ViStatus rsdsa_JQDResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_JQDQueryBERElapsedTime(ViSession instrSession,
                                      ViReal64* elapsedTime);

ViStatus rsdsa_JQDQueryBERBlocksProcessed(ViSession instrSession,
                                          ViReal64* blocksProcessed);

ViStatus rsdsa_JQDQueryBERPreReedSolomon(ViSession instrSession,
                                         ViReal64* BERPreRS);

ViStatus rsdsa_JQDQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                  ViReal64* preRSTotalBits);

ViStatus rsdsa_JQDQueryBERPostReedSolomon(ViSession instrSession,
                                          ViReal64* BERPostRS);

ViStatus rsdsa_JQDQueryBERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                       ViReal64* postRSCorrectedBits);

ViStatus rsdsa_JQDQueryBERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                         ViReal64* postRSUncorrectedBits);

ViStatus rsdsa_JQDQueryCERCorrected(ViSession instrSession,
                                    ViReal64* CERCorrected);

ViStatus rsdsa_JQDQueryCERUncorrected(ViSession instrSession,
                                      ViReal64* CERUncorrected);

ViStatus rsdsa_JQDQueryCERTotalCodewords(ViSession instrSession,
                                         ViReal64* totalCodewords);

ViStatus rsdsa_JQDQueryCERCorrectedCodewords(ViSession instrSession,
                                             ViReal64* correctedCodewords);

ViStatus rsdsa_JQDQueryCERUncorrectedCodewords(ViSession instrSession,
                                               ViReal64* uncorrectedCodewords);

ViStatus rsdsa_JQDDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_JQDDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_JQDConfigureIQMeasurementMode(ViSession instrSession,
                                             ViInt32   measurementMode,
                                             ViInt32   windowWidth);

ViStatus rsdsa_JQDSetIQDisplayZoom(ViSession instrSession,
                                   ViReal64  x,
                                   ViReal64  y,
                                   ViInt32   zoom);

ViStatus rsdsa_JQDCaptureIQDataToFile(ViSession instrSession,
                                      ViString  fileName,
                                      ViInt64   maxSamples,
                                      ViReal64  timeoutSeconds);

ViStatus rsdsa_JQDDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_JQDConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                     ViInt32   period);

ViStatus rsdsa_JQDDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_JQDConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                     ViInt32   period);

ViStatus rsdsa_JQDDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_JQDConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                       ViInt32   period);

ViStatus rsdsa_JQDDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_JQDQueryCCDFPeak(ViSession instrSession,
                                ViReal64* CCDFPeak);

ViStatus rsdsa_JQDDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_JQDAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_JQDDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_JQDAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_JQDDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_JQDQueryTraceData(ViSession instrSession,
                                 ViInt32   trace,
                                 ViInt32   bufferSize,
                                 ViReal64  traceData[],
                                 ViInt32*  bytesRead);

ViStatus rsdsa_JQDQuerySystemAcquisitionLogAll(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    log[],
                                               ViInt32*  bytesRead);

ViStatus rsdsa_JQDQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    entry[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_JQDQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                 ViInt32*  count);

ViStatus rsdsa_JQDSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_JQDConfigureGeneralSettings(ViSession instrSession,
                                           ViInt32   FEC,
                                           ViReal64  frequency,
                                           ViInt32   constellation,
                                           ViReal64  symbolRate,
                                           ViInt32   rollOff,
                                           ViBoolean spectralInversionEnabled,
                                           ViInt32   equalizerState,
                                           ViReal64  attenuation,
                                           ViBoolean preamplifierEnabled,
                                           ViBoolean ASIOutputEnabled,
                                           ViInt32   phaseTrackingLoopBandwidth);

ViStatus rsdsa_JQDQueryEqualizerCoefficients(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViReal64  equalizerCoefficients[],
                                             ViInt32*  bytesRead);

ViStatus rsdsa_JQDAutoLevel(ViSession instrSession);

ViStatus rsdsa_JQDConfigureAlarmThresholds(ViSession instrSession,
                                           ViReal64  RMSMEROverallThreshold,
                                           ViReal64  BERPreRSThreshold,
                                           ViReal64  BERPostRSThreshold);

ViStatus rsdsa_EnableDOCSIS31OFDMMode(ViSession instrSession);

ViStatus rsdsa_D31DConfigureProfileFilter(ViSession instrSession,
                                          ViInt32   profileFilter);

ViStatus rsdsa_D31DQueryPLCLockStatus(ViSession instrSession,
                                      ViInt32*  PLCLockStatus);

ViStatus rsdsa_D31DQueryDemodulatorLockStatus(ViSession instrSession,
                                              ViInt32*  demodulatorLockStatus);

ViStatus rsdsa_D31DQuerySignalPower(ViSession instrSession,
                                    ViReal64* signalPower);

ViStatus rsdsa_D31DQueryOccupiedBandwidth(ViSession instrSession,
                                          ViReal64* occupiedBandwidth);

ViStatus rsdsa_D31DQuerySignalPowerPer6MHz(ViSession instrSession,
                                           ViReal64* signalPowerPer6MHz);

ViStatus rsdsa_D31DQueryFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_D31DQuerySymbolClockOffset(ViSession instrSession,
                                          ViReal64* symbolClockOffset);

ViStatus rsdsa_D31DQueryNCPCRCErrors(ViSession instrSession,
                                     ViInt32*  NCPCRCErrors);

ViStatus rsdsa_D31DQueryPayloadDataRateProfile(ViSession instrSession,
                                               ViReal64* payloadDataRateProfile);

ViStatus rsdsa_D31DConfigureMERCompensationEnabled(ViSession instrSession,
                                                   ViBoolean enabled);

ViStatus rsdsa_D31DConfigureMERSign(ViSession instrSession,
                                    ViInt32   MERSign);

ViStatus rsdsa_D31DQueryMERRMSCompensatedOverall(ViSession  instrSession,
                                                 ViBoolean* RMSCompensatedOverall);

ViStatus rsdsa_D31DQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                ViReal64* RMSPartialyEqualized);

ViStatus rsdsa_D31DQueryMERRMSEqualized(ViSession instrSession,
                                        ViReal64* RMSEqualized);

ViStatus rsdsa_D31DQueryMERPeak(ViSession instrSession,
                                ViReal64* peak);

ViStatus rsdsa_D31DQueryMERProfileARMS(ViSession instrSession,
                                       ViReal64* profileARMS);

ViStatus rsdsa_D31DQueryMERProfileARMSCompensated(ViSession  instrSession,
                                                  ViBoolean* profileARMSCompensated);

ViStatus rsdsa_D31DQueryMERProfileAPeak(ViSession instrSession,
                                        ViReal64* profileAPeak);

ViStatus rsdsa_D31DQueryProfileAConstellations(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    profileAConstellations[],
                                               ViInt32*  bytesRead);

ViStatus rsdsa_D31DQueryMERProfileFilterRMS(ViSession instrSession,
                                            ViReal64* profileFilterRMS);

ViStatus rsdsa_D31DQueryMERProfileFilterRMSCompensated(ViSession  instrSession,
                                                       ViBoolean* profileFilterRMSCompensated);

ViStatus rsdsa_D31DQueryMERProfileFilterPeak(ViSession instrSession,
                                             ViReal64* profileFilterPeak);

ViStatus rsdsa_D31DQueryProfileFilterConstellations(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    profileFilterConstellations[],
                                                    ViInt32*  bytesRead);

ViStatus rsdsa_D31DQueryMERPilotRMS(ViSession instrSession,
                                    ViReal64* pilotRMS);

ViStatus rsdsa_D31DQueryMERPilotRMSCompensated(ViSession  instrSession,
                                               ViBoolean* pilotRMSCompensated);

ViStatus rsdsa_D31DQueryMERPilotPeak(ViSession instrSession,
                                     ViReal64* pilotPeak);

ViStatus rsdsa_D31DQueryPilotConstellations(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    pilotConstellations[],
                                            ViInt32*  bytesRead);

ViStatus rsdsa_D31DQueryMERPLCRMS(ViSession instrSession,
                                  ViReal64* PLCRMS);

ViStatus rsdsa_D31DQueryMERPLCRMSCompensated(ViSession  instrSession,
                                             ViBoolean* PLCRMSCompensated);

ViStatus rsdsa_D31DQueryMERPLCPeak(ViSession instrSession,
                                   ViReal64* PLCPeak);

ViStatus rsdsa_D31DQueryPLCConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    PLCConstellations[],
                                          ViInt32*  bytesRead);

ViStatus rsdsa_D31DQueryMERNCPRMS(ViSession instrSession,
                                  ViReal64* NCPRMS);

ViStatus rsdsa_D31DQueryMERNCPRMSCompensated(ViSession  instrSession,
                                             ViBoolean* NCPRMSCompensated);

ViStatus rsdsa_D31DQueryMERNCPPeak(ViSession instrSession,
                                   ViReal64* NCPPeak);

ViStatus rsdsa_D31DQueryNCPConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    NCPConstellations[],
                                          ViInt32*  bytesRead);

ViStatus rsdsa_D31DQueryMERZeroBitLoadedRMS(ViSession instrSession,
                                            ViReal64* ZBLRMS);

ViStatus rsdsa_D31DQueryMERZeroBitLoadedRMSCompensated(ViSession  instrSession,
                                                       ViBoolean* ZBLRMSCompensated);

ViStatus rsdsa_D31DQueryMERZeroBitLoadedPeak(ViSession instrSession,
                                             ViReal64* ZBLPeak);

ViStatus rsdsa_D31DResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_D31DQueryBERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime);

ViStatus rsdsa_D31DConfigureErrorAnalysisBERSource(ViSession instrSession,
                                                   ViInt32   source);

ViStatus rsdsa_D31DQueryBERPLC(ViSession instrSession,
                               ViReal64* BERPLC);

ViStatus rsdsa_D31DQueryBERPLCTotalBits(ViSession instrSession,
                                        ViReal64* PLCTotalBits);

ViStatus rsdsa_D31DQueryBERNCP(ViSession instrSession,
                               ViReal64* BERNCP);

ViStatus rsdsa_D31DQueryBERNCPTotalBits(ViSession instrSession,
                                        ViReal64* NCPTotalBits);

ViStatus rsdsa_D31DQueryCERPLC(ViSession instrSession,
                               ViReal64* CERPLC);

ViStatus rsdsa_D31DQueryCERPLCTotalCodewords(ViSession instrSession,
                                             ViReal64* PLCTotalCodewords);

ViStatus rsdsa_D31DQueryCERNCP(ViSession instrSession,
                               ViReal64* CERNCP);

ViStatus rsdsa_D31DQueryCERNCPTotalCodewords(ViSession instrSession,
                                             ViReal64* NCPTotalCodewords);

ViStatus rsdsa_D31DConfigureErrorAnalysisProfile(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViInt32   manualTestDepth,
                                                 ViInt32   testDepthSource);

ViStatus rsdsa_D31DQueryProfileBERBlocksProcessed(ViSession instrSession,
                                                  ViReal64* blocksProcessed);

ViStatus rsdsa_D31DQueryProfileBERPreLDPC(ViSession instrSession,
                                          ViReal64* BERPreLDPC);

ViStatus rsdsa_D31DQueryProfileBERPreLDPCTotalBits(ViSession instrSession,
                                                   ViReal64* preLDPCTotalBits);

ViStatus rsdsa_D31DQueryProfileBERPostLDPC(ViSession instrSession,
                                           ViReal64* BERPostLDPC);

ViStatus rsdsa_D31DQueryProfileBERPostLDPCTotalBits(ViSession instrSession,
                                                    ViReal64* postLDPCTotalBits);

ViStatus rsdsa_D31DQueryProfileBERPostBCH(ViSession instrSession,
                                          ViReal64* BERPostBCH);

ViStatus rsdsa_D31DQueryProfileBERPostBCHTotalBits(ViSession instrSession,
                                                   ViReal64* postBCHTotalBits);

ViStatus rsdsa_D31DQueryProfileCERPostLDPC(ViSession instrSession,
                                           ViReal64* CERPostLDPC);

ViStatus rsdsa_D31DQueryProfileCERPostLDPCTotalCodewords(ViSession instrSession,
                                                         ViReal64* postLDPCTotalCodewords);

ViStatus rsdsa_D31DQueryProfileCERPostLDPCCodewordErrors(ViSession instrSession,
                                                         ViReal64* postLDPCCodewordErrors);

ViStatus rsdsa_D31DQueryProfileCERPostBCH(ViSession instrSession,
                                          ViReal64* CERPostBCH);

ViStatus rsdsa_D31DQueryProfileCERPostBCHTotalCodewords(ViSession instrSession,
                                                        ViReal64* postBCHTotalCodewords);

ViStatus rsdsa_D31DQueryProfileCERPostBCHCodewordErrors(ViSession instrSession,
                                                        ViReal64* postBCHCodewordErrors);

ViStatus rsdsa_D31DQueryProfileCERAvgPayloadPerCodeword(ViSession instrSession,
                                                        ViReal64* avgPayloadPerCodeword);

ViStatus rsdsa_D31DQueryProfileCERAvgLDPCIterations(ViSession instrSession,
                                                    ViReal64* avgLDPCIterations);

ViStatus rsdsa_D31DDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_D31DDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_D31DConfigureIQMeasurementMode(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   windowWidth);

ViStatus rsdsa_D31DSetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom);

ViStatus rsdsa_D31DCaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_D31DDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D31DConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D31DDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D31DConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D31DDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_D31DConfigurePowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period);

ViStatus rsdsa_D31DDisplayMERVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31DDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_D31DQueryCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_D31DDisplaySpectrumAnalysisPlot(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisClearAllMasks(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDisplayAllMasksEnabled(
                            ViSession instrSession,
                            ViBoolean displayAllMasksEnabled);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureAmplitudeSettings(
                            ViSession instrSession,
                            ViReal64  referenceLevel,
                            ViReal64  scalePerDivision);

ViStatus rsdsa_D31DSpectrumAnalysisAutoScaleYAxis(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureBandwidthSettings(
                            ViSession instrSession,
                            ViInt32   manualAverageCount);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureMeasurementsSettings(
                            ViSession instrSession,
                            ViInt32   captureMode,
                            ViInt32   traceHold);

ViStatus rsdsa_D31DSpectrumAnalysisRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureMarkerState(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViBoolean state);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                            ViInt32   marker,
                                                            ViReal64  xPosition);

ViStatus rsdsa_D31DSpectrumAnalysisQueryMarkerYPosition(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViReal64* yPosition);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                                       ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                                        ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                         ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                         ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                          ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerSetCenterFrequency(ViSession instrSession,
                                                            ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   marker);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                             ViInt32   deltaMarker,
                                                             ViBoolean markerState);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition);

ViStatus rsdsa_D31DSpectrumAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                             ViInt32   deltaMarker,
                                                             ViReal64* yPosition);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                             ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPeakRight(
                            ViSession instrSession,
                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPointLeft(
                            ViSession instrSession,
                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerToNextPointRight(
                            ViSession instrSession,
                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerSetCenterFrequency(
                            ViSession instrSession,
                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisDeltaMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   deltaMarker);

ViStatus rsdsa_D31DSpectrumAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureDisplayMaskVisible(
                            ViSession instrSession,
                            ViInt32   mask,
                            ViBoolean visible);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                             ViInt32   mask,
                                                             ViInt32   points,
                                                             ViReal64  xValues[],
                                                             ViReal64  yValues[]);

ViStatus rsdsa_D31DSpectrumAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                             ViInt32   mask,
                                                             ViInt32   points,
                                                             ViReal64  xValues[],
                                                             ViReal64  yValues[]);

ViStatus rsdsa_D31DSpectrumAnalysisClearMask(ViSession instrSession,
                                             ViInt32   mask);

ViStatus rsdsa_D31DDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31DAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31DDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31DAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31DDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_D31DQueryTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead);

ViStatus rsdsa_D31DQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_D31DQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_D31DQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count);

ViStatus rsdsa_D31DSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_D31DCapturePayloadToFile(ViSession instrSession,
                                        ViString  fileName,
                                        ViInt32   filter,
                                        ViInt32   maxFileSize);

ViStatus rsdsa_D31DConfigurePlotDataFilter(ViSession instrSession,
                                           ViInt32   dataFilter);

ViStatus rsdsa_D31DConfigureSignalSettings(ViSession instrSession,
                                           ViReal64  frequency,
                                           ViReal64  subcarrier0Frequency,
                                           ViInt32   equalizerState,
                                           ViBoolean prefilterEnabled,
                                           ViReal64  attenuation,
                                           ViBoolean preamplifierEnabled);

ViStatus rsdsa_D31DQueryEqualizerCoefficients(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViReal64  equalizerCoefficients[],
                                              ViInt32*  bytesRead);

ViStatus rsdsa_D31DAutoLevel(ViSession instrSession);

ViStatus rsdsa_D31DConfigureAcquisitionSettings(ViSession instrSession,
                                                ViInt32   acquisitionMode,
                                                ViInt32   subcarrierSpacing,
                                                ViInt32   cyclicPrefixLength,
                                                ViInt32   timeInterleaverDepth,
                                                ViString  subcarrierConfigFile);

ViStatus rsdsa_D31DConfigureSFPOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled);

ViStatus rsdsa_D31DConfigureSFPPLCUDPPort(ViSession instrSession,
                                          ViBoolean enabled,
                                          ViInt32   UDPPort);

ViStatus rsdsa_D31DConfigureSFPProfileUDPPort(ViSession instrSession,
                                              ViInt32   profile,
                                              ViBoolean enabled,
                                              ViInt32   UDPPort);

ViStatus rsdsa_D31DConfigureAlarmThresholds(ViSession instrSession,
                                            ViReal64  RMSMEROverall,
                                            ViReal64  BERPreLDPC,
                                            ViReal64  BERPostLDPC,
                                            ViReal64  BERPostBCH,
                                            ViReal64  BERPLC,
                                            ViReal64  BERNCP);

ViStatus rsdsa_EnableDownstreamSpectrumAnalysis(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnCenterFrequency(ViSession instrSession,
                                                ViReal64  centerFrequency);

ViStatus rsdsa_ConfigureDSSpecAnFrequency(ViSession instrSession,
                                          ViReal64  startFrequency,
                                          ViReal64  stopFrequency);

ViStatus rsdsa_DSSpecAnClearAllDisplayStates(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnSpan(ViSession instrSession,
                                     ViReal64  sweepTime,
                                     ViReal64  frequencySpan);

ViStatus rsdsa_DSSpecAnFrequencySpanFull(ViSession instrSession);

ViStatus rsdsa_DSSpecAnFrequencySpanMinimum(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnZeroSpan(ViSession instrSession,
                                         ViBoolean enabled,
                                         ViReal64  sampleRate);

ViStatus rsdsa_ConfigureDSSpecAnAmplitude(ViSession instrSession,
                                          ViReal64  referenceLevel,
                                          ViReal64  scalePerDivision);

ViStatus rsdsa_DSSpecAnAutoScaleYAxis(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnRBW(ViSession instrSession,
                                    ViBoolean autoRBWEnabled,
                                    ViReal64  manualRBW);

ViStatus rsdsa_ConfigureDSSpecAnVBW(ViSession instrSession,
                                    ViBoolean autoVBWEnabled,
                                    ViReal64  manualVBW);

ViStatus rsdsa_ConfigureDSSpecAnAverageCount(ViSession instrSession,
                                             ViBoolean autoAverageCount,
                                             ViInt32   averageCount);

ViStatus rsdsa_ConfigureDSSpecAnMeasurement(ViSession instrSession,
                                            ViInt32   captureMode,
                                            ViInt32   hold,
                                            ViBoolean phaseNoise);

ViStatus rsdsa_ConfigureDSSpecAnDetector(ViSession instrSession,
                                         ViInt32   detector);

ViStatus rsdsa_DSSpecAnRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnACPMode(ViSession instrSession,
                                        ViBoolean enabled,
                                        ViInt32   gridSpacing,
                                        ViInt32   referenceChannel);

ViStatus rsdsa_QueryDSSpecAnACPPowerResult(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViReal64  powerResult[],
                                           ViInt32*  bytesRead);

ViStatus rsdsa_DSSpecAnSaveACPPowerResult(ViSession instrSession,
                                          ViString  fileName);

ViStatus rsdsa_ConfigureDSSpecAnPhaseNoiseOffset(ViSession instrSession,
                                                 ViReal64  startOffset,
                                                 ViReal64  stopOffset);

ViStatus rsdsa_QueryDSSpecAnPhaseNoiseIntegratedResult(ViSession instrSession,
                                                       ViReal64* integratedResult);

ViStatus rsdsa_QueryDSSpecAnPhaseNoiseSpotResult(ViSession instrSession,
                                                 ViReal64* spotResult);

ViStatus rsdsa_QueryDSSpecAnPhaseNoiseBandwidth(ViSession instrSession,
                                                ViReal64* bandwidth);

ViStatus rsdsa_DSSpecAnAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnMarkerEnabled(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled);

ViStatus rsdsa_ConfigureDSSpecAnMarkerXPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViReal64  xPosition);

ViStatus rsdsa_QueryDSSpecAnMarkerYPosition(ViSession instrSession,
                                            ViInt32   marker,
                                            ViReal64* yPosition);

ViStatus rsdsa_DSSpecAnMarkerToMaximumPeak(ViSession instrSession,
                                           ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerToNextPeakLeft(ViSession instrSession,
                                            ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerToNextPeakRight(ViSession instrSession,
                                             ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerToNextPointLeft(ViSession instrSession,
                                             ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerToNextPointRight(ViSession instrSession,
                                              ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerSetCenterFrequency(ViSession instrSession,
                                                ViInt32   marker);

ViStatus rsdsa_DSSpecAnZeroSpanMarkerToTriggerPoint(ViSession instrSession,
                                                    ViInt32   marker);

ViStatus rsdsa_DSSpecAnMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                    ViInt32   marker);

ViStatus rsdsa_DSSpecAnAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerEnabled(ViSession instrSession,
                                                   ViInt32   delta,
                                                   ViBoolean enabled);

ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerAbsoluteXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  absoluteXPosition);

ViStatus rsdsa_ConfigureDSSpecAnDeltaMarkerRelativeXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  relativeXPosition);

ViStatus rsdsa_QueryDSSpecAnDeltaMarkerYPosition(ViSession instrSession,
                                                 ViInt32   delta,
                                                 ViReal64* yPosition);

ViStatus rsdsa_DSSpecAnDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                 ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                  ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                  ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerToNextPointRight(ViSession instrSession,
                                                   ViInt32   delta);

ViStatus rsdsa_DSSpecAnZeroSpanDeltaMarkerToTriggerPoint(ViSession instrSession,
                                                         ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerSetCenterFrequency(ViSession instrSession,
                                                     ViInt32   delta);

ViStatus rsdsa_DSSpecAnDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                         ViInt32   delta);

ViStatus rsdsa_QueryDSSpecAnDeltaMarkerBandwidthPowerResult(ViSession instrSession,
                                                            ViInt32   delta,
                                                            ViReal64* bandwidthPowerResult);

ViStatus rsdsa_DSSpecAnDisplayStateStore(ViSession instrSession,
                                         ViInt32   savedState);

ViStatus rsdsa_DSSpecAnDisplayStateClear(ViSession instrSession,
                                         ViInt32   savedState);

ViStatus rsdsa_DSSpecAnDisplayStateRecall(ViSession instrSession,
                                          ViInt32   savedState);

ViStatus rsdsa_QueryDSSpecAnTraceData(ViSession instrSession,
                                      ViInt32   trace,
                                      ViInt32   bufferSize,
                                      ViReal64  traceData[],
                                      ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureDSSpecAnDisplayAllMasksEnabled(ViSession instrSession,
                                                       ViBoolean allMasksEnabled);

ViStatus rsdsa_DSSpecAnClearAllMasks(ViSession instrSession);

ViStatus rsdsa_DSSpecAnClearMask(ViSession instrSession,
                                 ViInt32   mask);

ViStatus rsdsa_ConfigureDSSpecAnMaskVisible(ViSession instrSession,
                                            ViInt32   mask,
                                            ViBoolean maskVisible);

ViStatus rsdsa_DSSpecAnConfigureLowerDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[]);

ViStatus rsdsa_DSSpecAnConfigureUpperDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[]);

ViStatus rsdsa_EnableDOCSIS30ATDMAMode(ViSession instrSession);

ViStatus rsdsa_ConfigureD30AMapGenIUC(ViSession instrSession,
                                      ViInt32   IUC);

ViStatus rsdsa_QueryD30ABurstDemodulatorLockStatus(ViSession instrSession,
                                                   ViInt32*  burstDemodulatorLock);

ViStatus rsdsa_QueryD30AFECLockStatus(ViSession instrSession,
                                      ViInt32*  FECLock);

ViStatus rsdsa_QueryD30ABurstReceivePower(ViSession instrSession,
                                          ViReal64* burstReceivePower);

ViStatus rsdsa_QueryD30ASignalPowerPer1600kHz(ViSession instrSession,
                                              ViReal64* signalPowerPer1600kHz);

ViStatus rsdsa_QueryD30AFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_QueryD30ABurstTimingOffset(ViSession instrSession,
                                          ViReal64* burstTimingOffset);

ViStatus rsdsa_QueryD30APayloadDataRateIUC(ViSession instrSession,
                                           ViReal64* payloadDataRateIUC);

ViStatus rsdsa_QueryD30AMinislotsReceived(ViSession instrSession,
                                          ViInt64*  minislotsReceived);

ViStatus rsdsa_QueryD30ABurstCoverage(ViSession instrSession,
                                      ViReal64* burstCoverage);

ViStatus rsdsa_ConfigureD30AMERSign(ViSession instrSession,
                                    ViInt32   MERSign);

ViStatus rsdsa_QueryD30AMERRMS(ViSession instrSession,
                               ViReal64* MERRMS);

ViStatus rsdsa_QueryD30AMERPeak(ViSession instrSession,
                                ViReal64* MERPeak);

ViStatus rsdsa_QueryD30AMERIUCRMS(ViSession instrSession,
                                  ViReal64* MERIUCRMS);

ViStatus rsdsa_QueryD30AMERIUCPeak(ViSession instrSession,
                                   ViReal64* MERIUCPeak);

ViStatus rsdsa_QueryD30AIQImbalanceAmplitudedB(ViSession instrSession,
                                               ViReal64* amplitudeDB);

ViStatus rsdsa_QueryD30AIQImbalanceAmplitudePercent(ViSession instrSession,
                                                    ViReal64* amplitudePercent);

ViStatus rsdsa_QueryD30AIQImbalanceQuadratureError(ViSession instrSession,
                                                   ViReal64* quadratureError);

ViStatus rsdsa_ConfigureD30AErrorAnalysis(ViSession instrSession,
                                          ViInt32   errorAnalysisMode,
                                          ViInt32   manualTestDepth,
                                          ViInt32   testDepthSource,
                                          ViInt32   progressionSource);

ViStatus rsdsa_ResetD30AErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_QueryD30ABERElapsedTime(ViSession instrSession,
                                       ViReal64* elapsedTime);

ViStatus rsdsa_QueryD30ABERBlocksProcessed(ViSession instrSession,
                                           ViReal64* blocksProcessed);

ViStatus rsdsa_QueryD30ABERBurstsExpected(ViSession instrSession,
                                          ViReal64* burstsExpected);

ViStatus rsdsa_QueryD30ABERBurstsReceived(ViSession instrSession,
                                          ViReal64* burstsReceived);

ViStatus rsdsa_QueryD30ABERPreReedSolomon(ViSession instrSession,
                                          ViReal64* BERPreRS);

ViStatus rsdsa_QueryD30ABERPreReedSolomonTotalBits(ViSession instrSession,
                                                   ViReal64* preRSTotalBits);

ViStatus rsdsa_QueryD30ABERPostReedSolomon(ViSession instrSession,
                                           ViReal64* BERPostRS);

ViStatus rsdsa_QueryD30ABERPostReedSolomonCorrectedBits(ViSession instrSession,
                                                        ViReal64* correctedBits);

ViStatus rsdsa_QueryD30ABERPostReedSolomonUncorrectedBits(ViSession instrSession,
                                                          ViReal64* uncorrectedBits);

ViStatus rsdsa_QueryD30ACERCorrected(ViSession instrSession,
                                     ViReal64* corrected);

ViStatus rsdsa_QueryD30ACERUncorrected(ViSession instrSession,
                                       ViReal64* uncorrected);

ViStatus rsdsa_QueryD30ACERTotalCodewords(ViSession instrSession,
                                          ViReal64* totalCodewords);

ViStatus rsdsa_QueryD30ACERCorrectedCodewords(ViSession instrSession,
                                              ViReal64* correctedCodewords);

ViStatus rsdsa_QueryD30ACERUncorrectedCodewords(ViSession instrSession,
                                                ViReal64* uncorrectedCodewords);

ViStatus rsdsa_QueryD30ACERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                       ViReal64* payloadPerCodewordPerIUC);

ViStatus rsdsa_QueryD30ACERCodewordCoverageIUC(ViSession instrSession,
                                               ViReal64* codewordCoverageIUC);

ViStatus rsdsa_D30ADisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_D30ADisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30AIQ(ViSession instrSession,
                               ViInt32   measurementMode,
                               ViInt32   windowWidth);

ViStatus rsdsa_ConfigureD30AIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom);

ViStatus rsdsa_D30ACaptureIQDataToFile(ViSession instrSession,
                                       ViString  fileName,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_D30ADisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30AMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D30ADisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30ABERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   period);

ViStatus rsdsa_D30ADisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD30APowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   period);

ViStatus rsdsa_D30ADisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_QueryD30ACCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_D30ADisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30AAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30ADisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30AAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D30ADisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_QueryD30ASystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    log[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_QueryD30ASystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    entry[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_QueryD30ASystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  count);

ViStatus rsdsa_D30ASystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_D30AAutoLevel(ViSession instrSession);

ViStatus rsdsa_ConfigureD30AGeneralSettings(ViSession instrSession,
                                            ViReal64  frequency,
                                            ViInt32   constellation,
                                            ViInt32   channelSymbolRate,
                                            ViInt32   lastCodewordLength,
                                            ViInt32   guardTimeSize,
                                            ViInt32   minislotSize);

ViStatus rsdsa_ConfigureD30AFECParameter(ViSession instrSession,
                                         ViInt32   FECParameterT,
                                         ViInt32   FECParameterK);

ViStatus rsdsa_ConfigureD30APreamble(ViSession instrSession,
                                     ViString  pattern,
                                     ViInt32   length,
                                     ViInt32   offset,
                                     ViInt32   type);

ViStatus rsdsa_ConfigureD30AScrambler(ViSession instrSession,
                                      ViBoolean enabled,
                                      ViString  seed);

ViStatus rsdsa_ConfigureD30AReedSolomon(ViSession instrSession,
                                        ViInt32   mode,
                                        ViInt32   interleaverDepth,
                                        ViInt32   interleaverBlockSize);

ViStatus rsdsa_ConfigureD30AMinislotSize(ViSession instrSession,
                                         ViInt32   minislotSize);

ViStatus rsdsa_ConfigureD30ASignalPath(ViSession instrSession,
                                       ViInt32   equalizerState,
                                       ViReal64  attenuation,
                                       ViBoolean preamplifierEnabled);

ViStatus rsdsa_ConfigureD30AScheduling(ViSession instrSession,
                                       ViInt32   grantSize,
                                       ViInt32   startingMinislot,
                                       ViInt32   scheduling,
                                       ViInt32   repeatInterval,
                                       ViInt32   numberOfBursts,
                                       ViBoolean rangingMode,
                                       ViInt32   untimedBurstLength);

ViStatus rsdsa_ConfigureD30AInputTriggerOffset(ViSession instrSession,
                                               ViReal64  inputTriggerOffset);

ViStatus rsdsa_ConfigureD30AAlarmThreshold(ViSession instrSession,
                                           ViReal64  RMSMEROverall,
                                           ViReal64  BERPreRS,
                                           ViReal64  BERPostRS);

ViStatus rsdsa_ConfigureD30AMERBursts(ViSession instrSession,
                                      ViInt32   MERBursts);

ViStatus rsdsa_ConfigureD30ASFPOutputEnabled(ViSession instrSession,
                                             ViBoolean SFPOutputEnabled);

ViStatus rsdsa_ConfigureD30ASFPIUCUDPPort(ViSession instrSession,
                                          ViInt32   IUC,
                                          ViBoolean enabled,
                                          ViInt32   UDPPort);

ViStatus rsdsa_EnableDOCSIS31OFDMAMode(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UMapGenIUC(ViSession instrSession,
                                      ViInt32   mapGenIUC);

ViStatus rsdsa_QueryD31UBurstDemodulatorLockStatus(ViSession instrSession,
                                                   ViInt32*  burstDemodulatorLock);

ViStatus rsdsa_QueryD31UFECLockStatus(ViSession instrSession,
                                      ViInt32*  FECLock);

ViStatus rsdsa_QueryD31UReadBurstReceivePower(ViSession instrSession,
                                              ViReal64* readBurstReceivePower);

ViStatus rsdsa_QueryD31USignalPowerPer1600kHz(ViSession instrSession,
                                              ViReal64* signalPowerPer1600kHz);

ViStatus rsdsa_QueryD31UFrequencyOffset(ViSession instrSession,
                                        ViReal64* frequencyOffset);

ViStatus rsdsa_QueryD31UBurstTimingOffset(ViSession instrSession,
                                          ViReal64* burstTimingOffset);

ViStatus rsdsa_QueryD31UPayloadDataRateIUC(ViSession instrSession,
                                           ViReal64* payloadDataRateIUC);

ViStatus rsdsa_QueryD31UMinislotsReceived(ViSession instrSession,
                                          ViInt64*  minislotsReceived);

ViStatus rsdsa_QueryD31UBurstCoverage(ViSession instrSession,
                                      ViReal64* burstCoverage);

ViStatus rsdsa_ConfigureD31UMERSign(ViSession instrSession,
                                    ViInt32   MERSign);

ViStatus rsdsa_QueryD31UMERRMS(ViSession instrSession,
                               ViReal64* MERRMS);

ViStatus rsdsa_QueryD31UMERRMSPartialyEqualized(ViSession instrSession,
                                                ViReal64* MERRMSPartialyEqualized);

ViStatus rsdsa_QueryD31UMERPeak(ViSession instrSession,
                                ViReal64* MERPeak);

ViStatus rsdsa_QueryD31UMERPilotRMS(ViSession instrSession,
                                    ViReal64* MERPilotRMS);

ViStatus rsdsa_QueryD31UMERPilotPeak(ViSession instrSession,
                                     ViReal64* MERPilotPeak);

ViStatus rsdsa_QueryD31UPilotConstellations(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    pilotConstellations[],
                                            ViInt32*  bytesRead);

ViStatus rsdsa_QueryD31UMERComplementaryPilotRMS(ViSession instrSession,
                                                 ViReal64* MERComplementaryPilotRMS);

ViStatus rsdsa_QueryD31UMERComplementaryPilotPeak(ViSession instrSession,
                                                  ViReal64* MERComplementaryPilotPeak);

ViStatus rsdsa_QueryD31UComplementaryPilotConstellations(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    pilotConstellations[],
                                                         ViInt32*  bytesRead);

ViStatus rsdsa_QueryD31UMERMapGenIUCRMS(ViSession instrSession,
                                        ViReal64* MERMapGenIUCRMS);

ViStatus rsdsa_QueryD31UMERMapGenIUCPeak(ViSession instrSession,
                                         ViReal64* MERMapGenIUCPeak);

ViStatus rsdsa_QueryD31UIUCConstellations(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    IUCConstellations[],
                                          ViInt32*  bytesRead);

ViStatus rsdsa_ResetD31UErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UErrorAnalysis(ViSession instrSession,
                                          ViInt32   mode,
                                          ViInt32   testDepth,
                                          ViInt32   testDepthSource,
                                          ViInt32   progressionSource);

ViStatus rsdsa_QueryD31UBERElapsedTime(ViSession instrSession,
                                       ViReal64* BERElapsedTime);

ViStatus rsdsa_QueryD31UBERBlocksProcessed(ViSession instrSession,
                                           ViReal64* BERBlocksProcessed);

ViStatus rsdsa_QueryD31UBERBurstsExpected(ViSession instrSession,
                                          ViReal64* BERBurstsExpected);

ViStatus rsdsa_QueryD31UBERBurstsReceived(ViSession instrSession,
                                          ViReal64* BERBurstsReceived);

ViStatus rsdsa_QueryD31UBERIUCPreLDPC(ViSession instrSession,
                                      ViReal64* BERIUCPreLDPC);

ViStatus rsdsa_QueryD31UBERIUCPreLDPCTotalBits(ViSession instrSession,
                                               ViReal64* BERIUCPreLDPCTotalBits);

ViStatus rsdsa_QueryD31UBERIUCPostLDPC(ViSession instrSession,
                                       ViReal64* BERIUCPostLDPC);

ViStatus rsdsa_QueryD31UBERIUCPostLDPCTotalBits(ViSession instrSession,
                                                ViReal64* BERIUCPostLDPCTotalBits);

ViStatus rsdsa_ConfigureD31UCodewordLengthFilter(ViSession instrSession,
                                                 ViInt32   codewordLengthFilter);

ViStatus rsdsa_QueryD31UCERPostLDPC(ViSession instrSession,
                                    ViReal64* postLDPC);

ViStatus rsdsa_QueryD31UCERPostLDPCCodewordErrors(ViSession instrSession,
                                                  ViReal64* postLDPCCodewordErrors);

ViStatus rsdsa_QueryD31UCERAverageLDPCIterations(ViSession instrSession,
                                                 ViReal64* averageLDPCIterations);

ViStatus rsdsa_QueryD31UCERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                       ViReal64* payloadPerCodewordPerIUC);

ViStatus rsdsa_QueryD31UCERCodewordCoverageIUC(ViSession instrSession,
                                               ViReal64* CERCodewordCoverageIUC);

ViStatus rsdsa_D31UDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UIQ(ViSession instrSession,
                               ViInt32   mode,
                               ViInt32   windowWidth);

ViStatus rsdsa_D31USetIQDisplayZoom(ViSession instrSession,
                                    ViReal64  x,
                                    ViReal64  y,
                                    ViInt32   zoom);

ViStatus rsdsa_D31UCaptureIQDataToFile(ViSession instrSession,
                                       ViString  filename,
                                       ViInt64   maxSamples,
                                       ViReal64  timeoutSeconds);

ViStatus rsdsa_D31UDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   filterPeriod);

ViStatus rsdsa_D31UDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                      ViInt32   filterPeriod);

ViStatus rsdsa_D31UDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureD31UPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                        ViInt32   filterPeriod);

ViStatus rsdsa_D31UDisplayMERVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplayMERVsMinislotPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_QueryD31UCCDFPeak(ViSession instrSession,
                                 ViReal64* CCDFPeak);

ViStatus rsdsa_D31UDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31UAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31UAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_D31UDisplaySpectrumAnalysisPlot(ViSession instrSession);

ViStatus rsdsa_D31USpectrumAnalysisRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_D31USpectrumAnalysisAmplitudeAutoScale(ViSession instrSession);

ViStatus rsdsa_ConfigureD31USpectrumAnalysis(ViSession instrSession,
                                             ViInt32   traceMode,
                                             ViInt32   captureMode);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisAmplitude(ViSession instrSession,
                                                      ViReal64  scalePerDivision,
                                                      ViInt32   referenceLevel);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisAverages(ViSession instrSession,
                                                     ViInt32   averageCount);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisDisplayAllMasksEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus rsdsa_D31USpectrumAnalysisAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_D31USpectrumAnalysisAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_D31USpectrumAnalysisClearAllMasks(ViSession instrSession);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisMarker(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViBoolean enabled,
                                                   ViReal64  analysisMarkerXPosition);

ViStatus rsdsa_QueryD31USpectrumAnalysisMarkerYPosition(ViSession instrSession,
                                                        ViInt32   marker,
                                                        ViReal64* yPosition);

ViStatus rsdsa_D31USpectrumAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                                       ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                                        ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                         ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                         ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                          ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerSetCenterFrequency(ViSession instrSession,
                                                            ViInt32   marker);

ViStatus rsdsa_D31USpectrumAnalysisMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   marker);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViBoolean enabled);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64  xPosition);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64  xPosition);

ViStatus rsdsa_QueryD31USpectrumAnalysisDeltaMarkerYPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64* yPosition);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                            ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                             ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPeakRight(
                            ViSession instrSession,
                            ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPointLeft(
                            ViSession instrSession,
                            ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerToNextPointRight(
                            ViSession instrSession,
                            ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerSetCenterFrequency(
                            ViSession instrSession,
                            ViInt32   delta);

ViStatus rsdsa_D31USpectrumAnalysisDeltaMarkerSetPowerReferenceLevel(
                            ViSession instrSession,
                            ViInt32   delta);

ViStatus rsdsa_QueryD31USpectrumAnalysisDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViReal64* bandwidthPowerResult);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisMaskVisible(ViSession instrSession,
                                                        ViInt32   mask,
                                                        ViBoolean maskVisible);

ViStatus rsdsa_D31USpectrumAnalysisClearMask(ViSession instrSession,
                                             ViInt32   mask);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisLowerMask(ViSession instrSession,
                                                      ViInt32   mask,
                                                      ViInt32   points,
                                                      ViReal64  xValues[],
                                                      ViReal64  yValues[]);

ViStatus rsdsa_ConfigureD31USpectrumAnalysisUpperMask(ViSession instrSession,
                                                      ViInt32   mask,
                                                      ViInt32   points,
                                                      ViReal64  xValues[],
                                                      ViReal64  yValues[]);

ViStatus rsdsa_QueryD31UTraceData(ViSession instrSession,
                                  ViInt32   trace,
                                  ViInt32   bufferSize,
                                  ViReal64  traceData[],
                                  ViInt32*  bytesRead);

ViStatus rsdsa_QueryD31USystemAcquisitionLogAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    acquisitionLogAll[],
                                                ViInt32*  bytesRead);

ViStatus rsdsa_QueryD31USystemAcquisitionLogNext(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    acquisitionLogNext[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_QueryD31USystemAcquisitionLogCount(ViSession instrSession,
                                                  ViInt32*  acquisitionLogCount);

ViStatus rsdsa_D31USystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_D31UAutoLevel(ViSession instrSession);

ViStatus rsdsa_ConfigureD31U(ViSession instrSession,
                             ViInt32   plotDataFilter,
                             ViReal64  frequency,
                             ViInt32   rollOff,
                             ViInt32   cyclicPrefixLength,
                             ViInt32   symbolsPerFrame,
                             ViInt32   subcarrierSpacing);

ViStatus rsdsa_ConfigureD31URandomizer(ViSession instrSession,
                                       ViBoolean enabled,
                                       ViString  seed);

ViStatus rsdsa_ConfigureD31UExclusionBand(ViSession instrSession,
                                          ViInt32   band,
                                          ViInt32   start,
                                          ViInt32   width);

ViStatus rsdsa_ConfigureD31UWidebandProbe(ViSession instrSession,
                                          ViInt32   startSubcarrier,
                                          ViInt32   subcarrierSkip,
                                          ViBoolean stagger,
                                          ViInt32   symbolInFrame);

ViStatus rsdsa_ConfigureD31UBWRequest(ViSession instrSession,
                                      ViInt32   numberOfMinislots,
                                      ViInt32   subslot);

ViStatus rsdsa_ConfigureD31USignalPath(ViSession instrSession,
                                       ViReal64  attenuation,
                                       ViBoolean preamplifier,
                                       ViBoolean prefilter,
                                       ViInt32   prefilterLength);

ViStatus rsdsa_ConfigureD31UScheduling(ViSession instrSession,
                                       ViInt32   scheduling,
                                       ViInt32   repeatInterval);

ViStatus rsdsa_ConfigureD31UInputTriggerOffset(ViSession instrSession,
                                               ViReal64  inputTriggerOffset);

ViStatus rsdsa_ConfigureD31UAlarmThreshold(ViSession instrSession,
                                           ViReal64  RMSMEROverall,
                                           ViReal64  BERPreLDPC,
                                           ViReal64  BERPostLDPC);

ViStatus rsdsa_ConfigureD31UMERBursts(ViSession instrSession,
                                      ViInt32   MERBursts);

ViStatus rsdsa_ConfigureD31UMinislotConfigurationMode(ViSession instrSession,
                                                      ViInt32   mode);

ViStatus rsdsa_ConfigureD31UMinislotConfigurationFile(ViSession instrSession,
                                                      ViString  filename);

ViStatus rsdsa_ConfigureD31UMinislot(ViSession instrSession,
                                     ViInt32   constellation,
                                     ViInt32   pilotPattern,
                                     ViInt64   startingMinislot,
                                     ViInt32   numberOfMinislots,
                                     ViInt32   firstActiveSubcarrier,
                                     ViInt32   lastActiveSubcarrier);

ViStatus rsdsa_ConfigureD31UInitialRanging(ViSession instrSession,
                                           ViInt32   minislotInFrame,
                                           ViInt32   numberOfMinislots,
                                           ViInt32   numberOfSubcarriers);

ViStatus rsdsa_ConfigureD31UInitialRangingPreamble(ViSession instrSession,
                                                   ViString  pattern,
                                                   ViInt32   length,
                                                   ViInt32   offset);

ViStatus rsdsa_ConfigureD31UFineRanging(ViSession instrSession,
                                        ViInt32   startingMinislotInFrame,
                                        ViInt32   numberOfMinislots,
                                        ViInt32   numberOfSubcarriers);

ViStatus rsdsa_ConfigureD31UFineRangingPreamble(ViSession instrSession,
                                                ViString  pattern,
                                                ViInt32   offset);

ViStatus rsdsa_ConfigureD31USFPOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled);

ViStatus rsdsa_ConfigureD31USFPIUCUDPPort(ViSession instrSession,
                                          ViInt32   IUC,
                                          ViBoolean enabled,
                                          ViInt32   UDPPort);

ViStatus rsdsa_EnableUpstreamSpectrumAnalysis(ViSession instrSession);

ViStatus rsdsa_USSpecAnClearAllDisplayStates(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnCenterFrequency(ViSession instrSession,
                                                ViReal64  centerFrequency);

ViStatus rsdsa_ConfigureUSSpecAnFrequency(ViSession instrSession,
                                          ViReal64  startFrequency,
                                          ViReal64  stopFrequency);

ViStatus rsdsa_ConfigureUSSpecAnZeroSpan(ViSession instrSession,
                                         ViBoolean enabled,
                                         ViReal64  sweepTime,
                                         ViReal64  sampleRate);

ViStatus rsdsa_ConfigureUSSpecAnFrequencySpan(ViSession instrSession,
                                              ViReal64  frequencySpan);

ViStatus rsdsa_USSpecAnSetFrequencySpanFull(ViSession instrSession);

ViStatus rsdsa_USSpecAnSetFrequencySpanMinimum(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnPhaseNoiseOffset(ViSession instrSession,
                                                 ViReal64  startOffset,
                                                 ViReal64  stopOffset);

ViStatus rsdsa_QueryUSSpecAnPhaseNoiseSpotResult(ViSession instrSession,
                                                 ViReal64* spotResult);

ViStatus rsdsa_QueryUSSpecAnPhaseNoiseIntegratedResult(ViSession instrSession,
                                                       ViReal64* integratedResult);

ViStatus rsdsa_QueryUSSpecAnPhaseNoiseBandwidth(ViSession instrSession,
                                                ViReal64* bandwidth);

ViStatus rsdsa_USSpecAnDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnCCDFModeEnabled(ViSession instrSession,
                                                ViBoolean enabled);

ViStatus rsdsa_QueryUSSpecAnCCDFPeak(ViSession instrSession,
                                     ViReal64* CCDFPeak);

ViStatus rsdsa_USSpecAnAutoScaleYAxis(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnAmplitude(ViSession instrSession,
                                          ViReal64  referenceLevel,
                                          ViInt32   scalePerDivision);

ViStatus rsdsa_ConfigureUSSpecAnRBW(ViSession instrSession,
                                    ViBoolean autoRBW,
                                    ViReal64  manualRBW);

ViStatus rsdsa_ConfigureUSSpecAnVBW(ViSession instrSession,
                                    ViBoolean autoVBW,
                                    ViReal64  manualVBW);

ViStatus rsdsa_ConfigureUSSpecAnAverageCount(ViSession instrSession,
                                             ViBoolean autoAverageCount,
                                             ViInt32   manualAverageCount);

ViStatus rsdsa_USSpecAnRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnMeasurement(ViSession instrSession,
                                            ViInt32   captureMode,
                                            ViInt32   traceHold,
                                            ViBoolean ACPMode,
                                            ViBoolean phaseNoiseMode);

ViStatus rsdsa_ConfigureUSSpecAnDetector(ViSession instrSession,
                                         ViInt32   detector);

ViStatus rsdsa_USSpecAnAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnMarkerEnabled(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled);

ViStatus rsdsa_ConfigureUSSpecAnMarkerXPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViReal64  xPosition);

ViStatus rsdsa_QueryUSSpecAnMarkerYPosition(ViSession instrSession,
                                            ViInt32   marker,
                                            ViReal64* yPosition);

ViStatus rsdsa_USSpecAnMarkerToMaximumPeak(ViSession instrSession,
                                           ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerToNextPeakLeft(ViSession instrSession,
                                            ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerToNextPeakRight(ViSession instrSession,
                                             ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerToNextPointLeft(ViSession instrSession,
                                             ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerToNextPointRight(ViSession instrSession,
                                              ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerSetCenterFrequency(ViSession instrSession,
                                                ViInt32   marker);

ViStatus rsdsa_USSpecAnZeroSpanMarkerToTriggerPoint(ViSession instrSession,
                                                    ViInt32   marker);

ViStatus rsdsa_USSpecAnMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                    ViInt32   marker);

ViStatus rsdsa_USSpecAnAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerEnabled(ViSession instrSession,
                                                   ViInt32   delta,
                                                   ViBoolean enabled);

ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   delta,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerAbsoluteXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  absoluteXPosition);

ViStatus rsdsa_ConfigureUSSpecAnDeltaMarkerRelativeXPosition(ViSession instrSession,
                                                             ViInt32   delta,
                                                             ViReal64  relativeXPosition);

ViStatus rsdsa_QueryUSSpecAnDeltaMarkerYPosition(ViSession instrSession,
                                                 ViInt32   delta,
                                                 ViReal64* yPosition);

ViStatus rsdsa_USSpecAnDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                 ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                  ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                  ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerToNextPointRight(ViSession instrSession,
                                                   ViInt32   delta);

ViStatus rsdsa_USSpecAnZeroSpanDeltaMarkerToTriggerPoint(ViSession instrSession,
                                                         ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerSetCenterFrequency(ViSession instrSession,
                                                     ViInt32   delta);

ViStatus rsdsa_USSpecAnDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                         ViInt32   delta);

ViStatus rsdsa_QueryUSSpecAnDeltaMarkerBandwidthPowerResult(ViSession instrSession,
                                                            ViInt32   delta,
                                                            ViReal64* powerResult);

ViStatus rsdsa_USSpecAnDisplayStateStore(ViSession instrSession,
                                         ViInt32   savedState);

ViStatus rsdsa_USSpecAnDisplayStateClear(ViSession instrSession,
                                         ViInt32   savedState);

ViStatus rsdsa_USSpecAnDisplayStateRecall(ViSession instrSession,
                                          ViInt32   savedState);

ViStatus rsdsa_QueryUSSpecAnTraceData(ViSession instrSession,
                                      ViInt32   trace,
                                      ViInt32   bufferSize,
                                      ViReal64  traceData[],
                                      ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureUSSpecAnDisplayAllMasksEnabled(ViSession instrSession,
                                                       ViBoolean enabled);

ViStatus rsdsa_USSpecAnClearMask(ViSession instrSession,
                                 ViInt32   mask);

ViStatus rsdsa_USSpecAnClearAllMasks(ViSession instrSession);

ViStatus rsdsa_ConfigureUSSpecAnMaskVisible(ViSession instrSession,
                                            ViInt32   mask,
                                            ViBoolean maskVisible);

ViStatus rsdsa_USSpecAnConfigureLowerDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[]);

ViStatus rsdsa_USSpecAnConfigureUpperDisplayMask(ViSession instrSession,
                                                 ViInt32   mask,
                                                 ViInt32   points,
                                                 ViReal64  xValues[],
                                                 ViReal64  yValues[]);

ViStatus rsdsa_EnableDynamicUpstreamAnalysis(ViSession instrSession);

ViStatus rsdsa_DUAnalysisRestartDistanceEstimation(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureFilterIUC(ViSession instrSession,
                                            ViInt32   filterIUC);

ViStatus rsdsa_DUAnalysisConfigureFilterSID(ViSession instrSession,
                                            ViString  filterSID);

ViStatus rsdsa_DUAnalysisQueryBurstDemodulatorLockStatus(ViSession instrSession,
                                                         ViInt32*  burstDemodulatorLockStatus);

ViStatus rsdsa_DUAnalysisQueryFECLockStatus(ViSession instrSession,
                                            ViInt32*  FECLockStatus);

ViStatus rsdsa_DUAnalysisQueryMPEGLockStatus(ViSession instrSession,
                                             ViInt32*  MPEGLockStatus);

ViStatus rsdsa_DUAnalysisQueryPLCLockStatus(ViSession instrSession,
                                            ViInt32*  PLCLockStatus);

ViStatus rsdsa_DUAnalysisQueryBurstReceivePower(ViSession instrSession,
                                                ViReal64* burstReceivePower);

ViStatus rsdsa_DUAnalysisQueryFrequencyOffset(ViSession instrSession,
                                              ViReal64* frequencyOffset);

ViStatus rsdsa_DUAnalysisQueryBurstTimingOffset(ViSession instrSession,
                                                ViReal64* burstTimingOffset);

ViStatus rsdsa_DUAnalysisQueryPayloadDataRateIUC(ViSession instrSession,
                                                 ViReal64* payloadDataRateIUC);

ViStatus rsdsa_DUAnalysisQueryMinislotsReceived(ViSession instrSession,
                                                ViInt64*  minislotsReceived);

ViStatus rsdsa_DUAnalysisQueryBurstCoverage(ViSession instrSession,
                                            ViReal64* burstCoverage);

ViStatus rsdsa_DUAnalysisQueryDetectedUpstreamMode(ViSession instrSession,
                                                   ViInt32*  upstreamMode);

ViStatus rsdsa_DUAnalysisConfigureMERSign(ViSession instrSession,
                                          ViInt32   sign);

ViStatus rsdsa_DUAnalysisQueryMERRMS(ViSession instrSession,
                                     ViReal64* MERRMS);

ViStatus rsdsa_DUAnalysisQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                      ViReal64* MERRMSPartialyEqualized);

ViStatus rsdsa_DUAnalysisQueryMERPeak(ViSession instrSession,
                                      ViReal64* MERPeak);

ViStatus rsdsa_DUAnalysisQueryMERPilotRMS(ViSession instrSession,
                                          ViReal64* MERPilotRMS);

ViStatus rsdsa_DUAnalysisQueryMERPilotPeak(ViSession instrSession,
                                           ViReal64* MERPilotPeak);

ViStatus rsdsa_DUAnalysisQueryPilotConstellations(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    constellations[],
                                                  ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisQueryMERComplementaryPilotRMS(ViSession instrSession,
                                                       ViReal64* MERComplementaryPilotRMS);

ViStatus rsdsa_DUAnalysisQueryMERComplementaryPilotPeak(ViSession instrSession,
                                                        ViReal64* MERComplementaryPilotPeak);

ViStatus rsdsa_DUAnalysisQueryComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    constellations[],
                            ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisQueryMERFilteredRMS(ViSession instrSession,
                                             ViReal64* MERFilteredRMS);

ViStatus rsdsa_DUAnalysisQueryMERFilteredPeak(ViSession instrSession,
                                              ViReal64* MERFilteredPeak);

ViStatus rsdsa_DUAnalysisQueryIQImbalanceAmplitudedB(ViSession instrSession,
                                                     ViReal64* amplitudeDB);

ViStatus rsdsa_DUAnalysisQueryIQImbalanceAmplitudePercent(ViSession instrSession,
                                                          ViReal64* amplitudePercent);

ViStatus rsdsa_DUAnalysisQueryIQImbalanceQuadratureError(ViSession instrSession,
                                                         ViReal64* quadratureError);

ViStatus rsdsa_DUAnalysisQueryDownstreamChannelID(ViSession instrSession,
                                                  ViInt32*  downstreamChannelID);

ViStatus rsdsa_DUAnalysisQueryUpstreamChannelID(ViSession instrSession,
                                                ViInt32*  upstreamChannelID);

ViStatus rsdsa_DUAnalysisConfigureErrorAnalysis(ViSession instrSession,
                                                ViInt32   errorAnalysisMode,
                                                ViInt32   manualTestDepth,
                                                ViInt32   testDepthSource,
                                                ViInt32   testDepthProgressionSource);

ViStatus rsdsa_DUAnalysisResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_DUAnalysisQueryBERElapsedTime(ViSession instrSession,
                                             ViReal64* elapsedTime);

ViStatus rsdsa_DUAnalysisQueryBERBlocksProcessed(ViSession instrSession,
                                                 ViReal64* blocksProcessed);

ViStatus rsdsa_DUAnalysisQueryBERBurstsExpected(ViSession instrSession,
                                                ViReal64* burstsExpected);

ViStatus rsdsa_DUAnalysisQueryBERBurstsReceived(ViSession instrSession,
                                                ViReal64* burstsReceived);

ViStatus rsdsa_DUAnalysisQueryBERPreReedSolomon(ViSession instrSession,
                                                ViReal64* BERPreRS);

ViStatus rsdsa_DUAnalysisQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                         ViReal64* preRSTotalBits);

ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomon(ViSession instrSession,
                                                 ViReal64* BERPostRS);

ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomonCorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSCorrectedBits);

ViStatus rsdsa_DUAnalysisQueryBERPostReedSolomonUncorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSUncorrectedBits);

ViStatus rsdsa_DUAnalysisQueryBERIUCPreLDPC(ViSession instrSession,
                                            ViReal64* BERIUCPreLDPC);

ViStatus rsdsa_DUAnalysisQueryBERIUCPreLDPCTotalBits(ViSession instrSession,
                                                     ViReal64* IUCPreLDPCTotalBits);

ViStatus rsdsa_DUAnalysisQueryBERIUCPostLDPC(ViSession instrSession,
                                             ViReal64* BERIUCPostLDPC);

ViStatus rsdsa_DUAnalysisQueryBERIUCPostLDPCTotalBits(ViSession instrSession,
                                                      ViReal64* IUCPostLDPCTotalBits);

ViStatus rsdsa_DUAnalysisConfigureCodewordLengthFilter(ViSession instrSession,
                                                       ViInt32   codewordLengthFilter);

ViStatus rsdsa_DUAnalysisQueryCERCorrected(ViSession instrSession,
                                           ViReal64* CERCorrected);

ViStatus rsdsa_DUAnalysisQueryCERUncorrected(ViSession instrSession,
                                             ViReal64* CERUncorrected);

ViStatus rsdsa_DUAnalysisQueryCERTotalCodewords(ViSession instrSession,
                                                ViReal64* totalCodewords);

ViStatus rsdsa_DUAnalysisQueryCERCorrectedCodewords(ViSession instrSession,
                                                    ViReal64* correctedCodewords);

ViStatus rsdsa_DUAnalysisQueryCERUncorrectedCodewords(ViSession instrSession,
                                                      ViReal64* uncorrectedCodewords);

ViStatus rsdsa_DUAnalysisQueryCERPostLDPC(ViSession instrSession,
                                          ViReal64* CERPostLDPC);

ViStatus rsdsa_DUAnalysisQueryCERPostLDPCCodewordErrors(ViSession instrSession,
                                                        ViReal64* postLDPCCodewordErrors);

ViStatus rsdsa_DUAnalysisQueryCERAverageLDPCIterations(ViSession instrSession,
                                                       ViReal64* averageLDPCIterations);

ViStatus rsdsa_DUAnalysisQueryCERAvgPayloadPerCodewordPerIUC(ViSession instrSession,
                                                             ViReal64* avgPayload);

ViStatus rsdsa_DUAnalysisQueryCERCodewordCoverageIUC(ViSession instrSession,
                                                     ViReal64* codewordCoverageIUC);

ViStatus rsdsa_DUAnalysisDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureIQMeasurementMode(ViSession instrSession,
                                                    ViInt32   measurementMode,
                                                    ViInt32   windowWidth);

ViStatus rsdsa_DUAnalysisSetIQDisplayZoom(ViSession instrSession,
                                          ViReal64  x,
                                          ViReal64  y,
                                          ViInt32   zoom);

ViStatus rsdsa_DUAnalysisCaptureIQDataToFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViInt64   maxSamples,
                                             ViReal64  timeoutSeconds);

ViStatus rsdsa_DUAnalysisDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                            ViInt32   period);

ViStatus rsdsa_DUAnalysisDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                            ViInt32   period);

ViStatus rsdsa_DUAnalysisDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigurePowerVsTimePlotFilterPeriod(
                            ViSession instrSession,
                            ViInt32   period);

ViStatus rsdsa_DUAnalysisDisplayMERVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplayMERVsMinislotPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_DUAnalysisQueryCCDFPeak(ViSession instrSession,
                                       ViReal64* CCDFPeak);

ViStatus rsdsa_DUAnalysisDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisDisplaySpectrumAnalysisPlot(ViSession instrSession);

ViStatus rsdsa_DUAnalysisAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_DUAnalysisAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_DUAnalysisClearAllMasks(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureDisplayAllMasksEnabled(ViSession instrSession,
                                                         ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureAmplitudeSettings(ViSession instrSession,
                                                    ViReal64  referenceLevel,
                                                    ViInt32   scalePerDivision);

ViStatus rsdsa_DUAnalysisAutoScaleYAxis(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureBandwidthSettings(ViSession instrSession,
                                                    ViBoolean autoAverageCount,
                                                    ViInt32   manualAverageCount);

ViStatus rsdsa_DUAnalysisConfigureMeasurementsSettings(ViSession instrSession,
                                                       ViInt32   captureMode,
                                                       ViInt32   traceHold);

ViStatus rsdsa_DUAnalysisRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureMarkerState(ViSession instrSession,
                                              ViInt32   marker,
                                              ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                  ViInt32   marker,
                                                  ViReal64  xPosition);

ViStatus rsdsa_DUAnalysisQueryMarkerYPosition(ViSession instrSession,
                                              ViInt32   marker,
                                              ViReal64* yPosition);

ViStatus rsdsa_DUAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                             ViInt32   marker);

ViStatus rsdsa_DUAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                              ViInt32   marker);

ViStatus rsdsa_DUAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                               ViInt32   marker);

ViStatus rsdsa_DUAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                               ViInt32   marker);

ViStatus rsdsa_DUAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                ViInt32   marker);

ViStatus rsdsa_DUAnalysisMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                      ViInt32   marker);

ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                   ViInt32   deltaMarker,
                                                   ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition);

ViStatus rsdsa_DUAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition);

ViStatus rsdsa_DUAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                   ViInt32   deltaMarker,
                                                   ViReal64* yPosition);

ViStatus rsdsa_DUAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                  ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                   ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                    ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                    ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisDeltaMarkerToNextPointRight(ViSession instrSession,
                                                     ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                           ViInt32   deltaMarker);

ViStatus rsdsa_DUAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult);

ViStatus rsdsa_DUAnalysisConfigureMaskVisible(ViSession instrSession,
                                              ViInt32   mask,
                                              ViBoolean visible);

ViStatus rsdsa_DUAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                   ViInt32   mask,
                                                   ViInt32   points,
                                                   ViReal64  xValues[],
                                                   ViReal64  yValues[]);

ViStatus rsdsa_DUAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                   ViInt32   mask,
                                                   ViInt32   points,
                                                   ViReal64  xValues[],
                                                   ViReal64  yValues[]);

ViStatus rsdsa_DUAnalysisClearMask(ViSession instrSession,
                                   ViInt32   mask);

ViStatus rsdsa_DUAnalysisQueryTraceData(ViSession instrSession,
                                        ViInt32   trace,
                                        ViInt32   bufferSize,
                                        ViReal64  traceData[],
                                        ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    log[],
                                                      ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    entry[],
                                                       ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                        ViInt32*  count);

ViStatus rsdsa_DUAnalysisSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureDownstreamStandard(ViSession instrSession,
                                                     ViInt32   standard);

ViStatus rsdsa_DUAnalysisQueryDownstreamEqualizerCoefficients(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  equalizerCoefficients[],
                            ViInt32*  bytesRead);

ViStatus rsdsa_DUAnalysisConfigureDownstreamConstellation(ViSession instrSession,
                                                          ViInt32   constellation);

ViStatus rsdsa_DUAnalysisDownstreamAutoLevel(ViSession instrSession);

ViStatus rsdsa_DUAnalysisConfigureDownstreamAttenuation(ViSession instrSession,
                                                        ViReal64  attenuation);

ViStatus rsdsa_DUAnalysisConfigureDownstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureUpstreamChannelID(ViSession instrSession,
                                                    ViInt32   channelID);

ViStatus rsdsa_DUAnalysisConfigureUpstreamAttenuation(ViSession instrSession,
                                                      ViReal64  attenuation);

ViStatus rsdsa_DUAnalysisConfigureUpstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureSFPOutputEnabled(ViSession instrSession,
                                                   ViBoolean enabled);

ViStatus rsdsa_DUAnalysisConfigureSFPIUCUDPPort(ViSession instrSession,
                                                ViInt32   IUC,
                                                ViBoolean UDPPortEnabled,
                                                ViInt32   UDPPort);

ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdRMSMEROverall(
                            ViSession instrSession,
                            ViReal64  RMSMEROverall);

ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdsBERReedSolomon(
                            ViSession instrSession,
                            ViReal64  BERPreRS,
                            ViReal64  BERPostRS);

ViStatus rsdsa_DUAnalysisConfigureAlarmThresholdsBERLDPC(ViSession instrSession,
                                                         ViReal64  BERPreLDPC,
                                                         ViReal64  BERPostLDPC);

ViStatus rsdsa_EnableDynamicSFDAnalysis(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisQueryBurstDemodulatorLockStatus(ViSession instrSession,
                                                          ViInt32*  burstDemodulatorLockStatus);

ViStatus rsdsa_SFDAnalysisQueryFECLockStatus(ViSession instrSession,
                                             ViInt32*  FECLockStatus);

ViStatus rsdsa_SFDAnalysisQueryBurstReceivePower(ViSession instrSession,
                                                 ViReal64* burstReceivePower);

ViStatus rsdsa_SFDAnalysisQueryFrequencyOffset(ViSession instrSession,
                                               ViReal64* frequencyOffset);

ViStatus rsdsa_SFDAnalysisQueryBurstTimingOffset(ViSession instrSession,
                                                 ViReal64* burstTimingOffset);

ViStatus rsdsa_SFDAnalysisQueryMinislotsReceived(ViSession instrSession,
                                                 ViInt64*  minislotsReceived);

ViStatus rsdsa_SFDAnalysisQueryBurstCoverage(ViSession instrSession,
                                             ViReal64* burstCoverage);

ViStatus rsdsa_SFDAnalysisQuerySFDPilotAcquired(ViSession instrSession,
                                                ViInt32*  SFDPilotAcquired);

ViStatus rsdsa_SFDAnalysisQueryDetectedUpstreamMode(ViSession instrSession,
                                                    ViInt32*  detectedUpstreamMode);

ViStatus rsdsa_SFDAnalysisConfigureMERSign(ViSession instrSession,
                                           ViInt32   MERSign);

ViStatus rsdsa_SFDAnalysisQueryMERRMS(ViSession instrSession,
                                      ViReal64* MERRMS);

ViStatus rsdsa_SFDAnalysisQueryMERRMSPartialyEqualized(ViSession instrSession,
                                                       ViReal64* MERRMSPartialyEqualized);

ViStatus rsdsa_SFDAnalysisQueryMERPeak(ViSession instrSession,
                                       ViReal64* MERPeak);

ViStatus rsdsa_SFDAnalysisQueryMERPilotRMS(ViSession instrSession,
                                           ViReal64* MERPilotRMS);

ViStatus rsdsa_SFDAnalysisQueryMERPilotPeak(ViSession instrSession,
                                            ViReal64* MERPilotPeak);

ViStatus rsdsa_SFDAnalysisQueryPilotConstellations(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    constellations[],
                                                   ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisQueryMERComplementaryPilotRMS(ViSession instrSession,
                                                        ViReal64* MERComplementaryPilotRMS);

ViStatus rsdsa_SFDAnalysisQueryMERComplementaryPilotPeak(ViSession instrSession,
                                                         ViReal64* MERComplementaryPilotPeak);

ViStatus rsdsa_SFDAnalysisQueryComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    constellations[],
                            ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisQueryIQImbalanceAmplitudedB(ViSession instrSession,
                                                      ViReal64* amplitudeDB);

ViStatus rsdsa_SFDAnalysisQueryIQImbalanceAmplitudePercent(ViSession instrSession,
                                                           ViReal64* amplitudePercent);

ViStatus rsdsa_SFDAnalysisQueryIQImbalanceQuadratureError(ViSession instrSession,
                                                          ViReal64* quadratureError);

ViStatus rsdsa_SFDAnalysisConfigureErrorAnalysis(ViSession instrSession,
                                                 ViInt32   errorAnalysisMode,
                                                 ViInt32   manualTestDepth,
                                                 ViInt32   testDepthSource,
                                                 ViInt32   testDepthProgressionSource);

ViStatus rsdsa_SFDAnalysisResetErrorAnalysis(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisQueryBERElapsedTime(ViSession instrSession,
                                              ViReal64* elapsedTime);

ViStatus rsdsa_SFDAnalysisQueryBERBlocksProcessed(ViSession instrSession,
                                                  ViReal64* blocksProcessed);

ViStatus rsdsa_SFDAnalysisQueryBERBurstsExpected(ViSession instrSession,
                                                 ViReal64* burstsExpected);

ViStatus rsdsa_SFDAnalysisQueryBERBurstsReceived(ViSession instrSession,
                                                 ViReal64* burstsReceived);

ViStatus rsdsa_SFDAnalysisQueryBERPreReedSolomon(ViSession instrSession,
                                                 ViReal64* preRS);

ViStatus rsdsa_SFDAnalysisQueryBERPreReedSolomonTotalBits(ViSession instrSession,
                                                          ViReal64* preRSTotalBits);

ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomon(ViSession instrSession,
                                                  ViReal64* postRS);

ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomonCorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSCorrectedBits);

ViStatus rsdsa_SFDAnalysisQueryBERPostReedSolomonUncorrectedBits(
                            ViSession instrSession,
                            ViReal64* postRSUncorrectedBits);

ViStatus rsdsa_SFDAnalysisConfigureCodewordLengthFilter(ViSession instrSession,
                                                        ViInt32   codewordLengthFilter);

ViStatus rsdsa_SFDAnalysisQueryCERCorrected(ViSession instrSession,
                                            ViReal64* corrected);

ViStatus rsdsa_SFDAnalysisQueryCERUncorrected(ViSession instrSession,
                                              ViReal64* uncorrected);

ViStatus rsdsa_SFDAnalysisQueryCERTotalCodewords(ViSession instrSession,
                                                 ViReal64* totalCodewords);

ViStatus rsdsa_SFDAnalysisQueryCERCorrectedCodewords(ViSession instrSession,
                                                     ViReal64* correctedCodewords);

ViStatus rsdsa_SFDAnalysisQueryCERUncorrectedCodewords(ViSession instrSession,
                                                       ViReal64* uncorrectedCodewords);

ViStatus rsdsa_SFDAnalysisQueryCERPostLDPC(ViSession instrSession,
                                           ViReal64* postLDPC);

ViStatus rsdsa_SFDAnalysisQueryCERPostLDPCCodewordErrors(ViSession instrSession,
                                                         ViReal64* postLDPCCodewordErrors);

ViStatus rsdsa_SFDAnalysisQueryCERAverageLDPCIterations(ViSession instrSession,
                                                        ViReal64* averageLDPCIterations);

ViStatus rsdsa_SFDAnalysisDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureIQMeasurementMode(ViSession instrSession,
                                                     ViInt32   measurementMode,
                                                     ViInt32   windowWidth);

ViStatus rsdsa_SFDAnalysisSetIQDisplayZoom(ViSession instrSession,
                                           ViReal64  x,
                                           ViReal64  y,
                                           ViInt32   zoom);

ViStatus rsdsa_SFDAnalysisCaptureIQDataToFile(ViSession instrSession,
                                              ViString  fileName,
                                              ViInt64   maxSamples,
                                              ViReal64  timeoutSeconds);

ViStatus rsdsa_SFDAnalysisDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   period);

ViStatus rsdsa_SFDAnalysisDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   period);

ViStatus rsdsa_SFDAnalysisDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigurePowerVsTimePlotFilterPeriod(
                            ViSession instrSession,
                            ViInt32   period);

ViStatus rsdsa_SFDAnalysisDisplayMERVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplayMERVsMinislotPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisQueryCCDFPeak(ViSession instrSession,
                                        ViReal64* CCDFPeak);

ViStatus rsdsa_SFDAnalysisDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisDisplaySpectrumAnalysisPlot(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisAllMarkersOff(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisAllDeltaMarkersOff(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisClearAllMasks(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureDisplayAllMasksEnabled(ViSession instrSession,
                                                          ViBoolean enabled);

ViStatus rsdsa_SFDAnalysisConfigureAmplitudeSettings(ViSession instrSession,
                                                     ViInt32   referenceLevel,
                                                     ViReal64  scalePerDivision);

ViStatus rsdsa_SFDAnalysisAutoScaleYAxis(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureBandwidthSettings(ViSession instrSession,
                                                     ViBoolean autoAverageCount,
                                                     ViInt32   manualAverageCount);

ViStatus rsdsa_SFDAnalysisConfigureMeasurementsSettings(ViSession instrSession,
                                                        ViInt32   captureMode,
                                                        ViInt32   traceHold);

ViStatus rsdsa_SFDAnalysisRunSingleSweep(ViSession instrSession);

ViStatus rsdsa_SFDAnalysisConfigureMarkerState(ViSession instrSession,
                                               ViInt32   marker,
                                               ViBoolean state);

ViStatus rsdsa_SFDAnalysisConfigureMarkerXPosition(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViReal64  xPosition);

ViStatus rsdsa_SFDAnalysisQueryMarkerYPosition(ViSession instrSession,
                                               ViInt32   marker,
                                               ViReal64* yPosition);

ViStatus rsdsa_SFDAnalysisMarkerToMaximumPeak(ViSession instrSession,
                                              ViInt32   marker);

ViStatus rsdsa_SFDAnalysisMarkerToNextPeakLeft(ViSession instrSession,
                                               ViInt32   marker);

ViStatus rsdsa_SFDAnalysisMarkerToNextPeakRight(ViSession instrSession,
                                                ViInt32   marker);

ViStatus rsdsa_SFDAnalysisMarkerToNextPointLeft(ViSession instrSession,
                                                ViInt32   marker);

ViStatus rsdsa_SFDAnalysisMarkerToNextPointRight(ViSession instrSession,
                                                 ViInt32   marker);

ViStatus rsdsa_SFDAnalysisMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                       ViInt32   marker);

ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerState(ViSession instrSession,
                                                    ViInt32   deltaMarker,
                                                    ViBoolean state);

ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerLinkedReferenceMarker(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   linkedReferenceMarker);

ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerAbsoluteXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  absoluteXPosition);

ViStatus rsdsa_SFDAnalysisConfigureDeltaMarkerRelativeXPosition(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  relativeXPosition);

ViStatus rsdsa_SFDAnalysisQueryDeltaMarkerYPosition(ViSession instrSession,
                                                    ViInt32   deltaMarker,
                                                    ViReal64* yPosition);

ViStatus rsdsa_SFDAnalysisDeltaMarkerToMaximumPeak(ViSession instrSession,
                                                   ViInt32   delta);

ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPeakLeft(ViSession instrSession,
                                                    ViInt32   delta);

ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPeakRight(ViSession instrSession,
                                                     ViInt32   delta);

ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPointLeft(ViSession instrSession,
                                                     ViInt32   delta);

ViStatus rsdsa_SFDAnalysisDeltaMarkerToNextPointRight(ViSession instrSession,
                                                      ViInt32   delta);

ViStatus rsdsa_SFDAnalysisDeltaMarkerSetPowerReferenceLevel(ViSession instrSession,
                                                            ViInt32   delta);

ViStatus rsdsa_SFDAnalysisQueryDeltaMarkerBandwidthPowerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* bandwidthPowerResult);

ViStatus rsdsa_SFDAnalysisConfigureMaskVisible(ViSession instrSession,
                                               ViInt32   mask,
                                               ViBoolean visible);

ViStatus rsdsa_SFDAnalysisClearMask(ViSession instrSession,
                                    ViInt32   mask);

ViStatus rsdsa_SFDAnalysisConfigureLowerDisplayMask(ViSession instrSession,
                                                    ViInt32   mask,
                                                    ViInt32   points,
                                                    ViReal64  xValues[],
                                                    ViReal64  yValues[]);

ViStatus rsdsa_SFDAnalysisConfigureUpperDisplayMask(ViSession instrSession,
                                                    ViInt32   mask,
                                                    ViInt32   points,
                                                    ViReal64  xValues[],
                                                    ViReal64  yValues[]);

ViStatus rsdsa_SFDAnalysisQueryTraceData(ViSession instrSession,
                                         ViInt32   trace,
                                         ViInt32   bufferSize,
                                         ViReal64  traceData[],
                                         ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogAll(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    log[],
                                                       ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogNext(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    entry[],
                                                        ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisQuerySystemAcquisitionLogCount(ViSession instrSession,
                                                         ViInt32*  count);

ViStatus rsdsa_SFDAnalysisSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_ConfigureSFDAnalysisConfigureSFDPilotFrequency(
                            ViSession instrSession,
                            ViReal64  SFDPilotFrequency);

ViStatus rsdsa_SFDAnalysisConfigurePlotDataFilter(ViSession instrSession,
                                                  ViInt32   plotDataFilter);

ViStatus rsdsa_SFDAnalysisQueryDownstreamEqualizerCoefficients(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  equalizerCoefficients[],
                            ViInt32*  bytesRead);

ViStatus rsdsa_SFDAnalysisConfigureDownstreamAttenuation(ViSession instrSession,
                                                         ViReal64  attenuation);

ViStatus rsdsa_SFDAnalysisConfigureDownstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus rsdsa_SFDAnalysisConfigureUpstreamAttenuation(ViSession instrSession,
                                                       ViReal64  attenuation);

ViStatus rsdsa_SFDAnalysisConfigureUpstreamPreamplifierEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus rsdsa_SFDAnalysisConfigureSFPOutputEnabled(ViSession instrSession,
                                                    ViBoolean enabled);

ViStatus rsdsa_SFDAnalysisConfigureSFPIUCUDPPort(ViSession instrSession,
                                                 ViInt32   IUC,
                                                 ViBoolean enabled,
                                                 ViInt32   UDPPort);

ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdRMSMEROverall(
                            ViSession instrSession,
                            ViReal64  RMSMEROverall);

ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdsBERReedSolomon(
                            ViSession instrSession,
                            ViReal64  BERPreRS,
                            ViReal64  BERPostRS);

ViStatus rsdsa_SFDAnalysisConfigureAlarmThresholdsBERLDPC(ViSession instrSession,
                                                          ViReal64  BERPreLDPC,
                                                          ViReal64  BERPostLDPC);

ViStatus rsdsa_EnableCMRangingAnalysis(ViSession instrSession);

ViStatus rsdsa_QueryCMRangingBurstDemodulatorLockStatus(ViSession instrSession,
                                                        ViInt32*  burstDemodulatorLockStatus);

ViStatus rsdsa_QueryCMRangingFECLockStatus(ViSession instrSession,
                                           ViInt32*  FECLockStatus);

ViStatus rsdsa_QueryCMRangingBurstReceivePower(ViSession instrSession,
                                               ViReal64* burstReceivePower);

ViStatus rsdsa_QueryCMRangingFrequencyOffset(ViSession instrSession,
                                             ViReal64* frequencyOffset);

ViStatus rsdsa_QueryCMRangingBurstTimingOffset(ViSession instrSession,
                                               ViReal64* burstTimingOffset);

ViStatus rsdsa_QueryCMRangingMinislotsReceived(ViSession instrSession,
                                               ViInt64*  minislotsReceived);

ViStatus rsdsa_QueryCMRangingBurstCoverage(ViSession instrSession,
                                           ViReal64* burstCoverage);

ViStatus rsdsa_ConfigureCMRangingMERSign(ViSession instrSession,
                                         ViInt32   MERSign);

ViStatus rsdsa_QueryCMRangingMERRMS(ViSession instrSession,
                                    ViReal64* MERRMS);

ViStatus rsdsa_QueryCMRangingMERRMSPartialyEqualized(ViSession instrSession,
                                                     ViReal64* MERRMSPartialyEqualized);

ViStatus rsdsa_QueryCMRangingMERPeak(ViSession instrSession,
                                     ViReal64* MERPeak);

ViStatus rsdsa_QueryCMRangingMERPilotRMS(ViSession instrSession,
                                         ViReal64* MERPilotRMS);

ViStatus rsdsa_QueryCMRangingMERPilotPeak(ViSession instrSession,
                                          ViReal64* MERPilotPeak);

ViStatus rsdsa_QueryCMRangingPilotConstellations(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    pilotConstellations[],
                                                 ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingMERComplementaryPilotRMS(ViSession instrSession,
                                                      ViReal64* MERComplementaryPilotRMS);

ViStatus rsdsa_QueryCMRangingMERComplementaryPilotPeak(ViSession instrSession,
                                                       ViReal64* MERComplementaryPilotPeak);

ViStatus rsdsa_QueryCMRangingComplementaryPilotConstellations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    pilotConstellations[],
                            ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingMERIUCRMS(ViSession instrSession,
                                       ViReal64* MERIUCRMS);

ViStatus rsdsa_QueryCMRangingMERIUCPeak(ViSession instrSession,
                                        ViReal64* MERIUCPeak);

ViStatus rsdsa_QueryCMRangingIUCConstellations(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    IUCConstellations[],
                                               ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingIQImbalanceAmplitudedB(ViSession instrSession,
                                                    ViReal64* IQImbalanceAmplitudeDB);

ViStatus rsdsa_QueryCMRangingIQImbalanceAmplitudePercent(ViSession instrSession,
                                                         ViReal64* IQImbalanceAmplitudePercent);

ViStatus rsdsa_QueryCMRangingIQImbalanceQuadratureError(ViSession instrSession,
                                                        ViReal64* IQImbalanceQuadratureError);

ViStatus rsdsa_CMRangingDisplayConstellationPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplayIQDensityPlot(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingIQMeasurement(ViSession instrSession,
                                               ViInt32   mode,
                                               ViInt32   windowWidth);

ViStatus rsdsa_CMRangingSetIQDisplayZoom(ViSession instrSession,
                                         ViReal64  x,
                                         ViReal64  y,
                                         ViInt32   z);

ViStatus rsdsa_CMRangingCaptureIQDataToFile(ViSession instrSession,
                                            ViInt64   maxSamples,
                                            ViReal64  timeoutSeconds,
                                            ViString  filename);

ViStatus rsdsa_CMRangingDisplayMERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingMERVsTimePlotFilterPeriod(ViSession instrSession,
                                                           ViInt32   MERVsTimePlotFilterPeriod);

ViStatus rsdsa_CMRangingDisplayBERVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingBERVsTimePlotFilterPeriod(ViSession instrSession,
                                                           ViInt32   BERVsTimePlotFilterPeriod);

ViStatus rsdsa_CMRangingDisplayPowerVsTimePlot(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingPowerVsTimePlotFilterPeriod(ViSession instrSession,
                                                             ViInt32   powVsTimePlotFilterPeriod);

ViStatus rsdsa_CMRangingDisplayMERVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplayMERVsMinislotPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplayCCDFHistogram(ViSession instrSession);

ViStatus rsdsa_QueryCMRangingCCDFPeak(ViSession instrSession,
                                      ViReal64* CCDFPeak);

ViStatus rsdsa_CMRangingDisplayAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingAutoScaleAmplitudePhaseVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplayAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingAutoScaleAmplitudeGroupDelayVsSubcarrierPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplayEchoPatternPlot(ViSession instrSession);

ViStatus rsdsa_CMRangingDisplaySpectrumAnalysisPlot(ViSession instrSession);

ViStatus rsdsa_QueryCMRangingTraceData(ViSession instrSession,
                                       ViInt32   trace,
                                       ViInt32   bufferSize,
                                       ViReal64  traceData[],
                                       ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogAll(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    acquisitionLogAll[],
                                                     ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogNext(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    acquisitionLogNext[],
                                                      ViInt32*  bytesRead);

ViStatus rsdsa_QueryCMRangingSystemAcquisitionLogCount(ViSession instrSession,
                                                       ViInt32*  acquisitionLogCount);

ViStatus rsdsa_CMRangingSystemAcquisitionLogClearAll(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingPlotDataFilter(ViSession instrSession,
                                                ViInt32   plotDataFilter);

ViStatus rsdsa_ConfigureCMRangingFEC(ViSession instrSession,
                                     ViInt32   FEC);

ViStatus rsdsa_ConfigureCMRangingDownstreamMode(ViSession instrSession,
                                                ViInt32   downstreamMode);

ViStatus rsdsa_ConfigureCMRangingSFDDownstreamTransmitPower(ViSession instrSession,
                                                            ViReal64  SFDDownstreamTransmitPower);

ViStatus rsdsa_ConfigureCMRangingDownstreamASIOutputEnabled(ViSession instrSession,
                                                            ViBoolean ASIOutputEnabled);

ViStatus rsdsa_ConfigureCMRangingED30(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation);

ViStatus rsdsa_ConfigureCMRangingD30D(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation,
                                      ViInt32   interleaver);

ViStatus rsdsa_ConfigureCMRangingD30J(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation);

ViStatus rsdsa_ConfigureCMRangingD31D(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   cyclicPrefixLength,
                                      ViInt32   timeInterleaverDepth);

ViStatus rsdsa_ConfigureCMRangingD31DSubcarrier(ViSession instrSession,
                                                ViReal64  subcarrier0Frequency,
                                                ViInt32   subcarrierSpacing);

ViStatus rsdsa_ConfigureCMRangingD31DSubcarrierConfigFile(ViSession instrSession,
                                                          ViString  subcarrierConfigFile);

ViStatus rsdsa_ConfigureCMRangingDownstream(ViSession instrSession,
                                            ViReal64  attenuation,
                                            ViBoolean preamplifierEnabled);

ViStatus rsdsa_QueryCMRangingDownstreamEqualizerCoefficients(ViSession instrSession,
                                                             ViInt32   bufferSize,
                                                             ViReal64  equalizerCoefficients[],
                                                             ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureCMRangingUpstreamMode(ViSession instrSession,
                                              ViInt32   upstreamMode);

ViStatus rsdsa_ConfigureCMRangingUpstreamPlotDataFilterIUC(ViSession instrSession,
                                                           ViInt32   plotDataFilterIUC);

ViStatus rsdsa_ConfigureCMRangingD30A(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   constellation,
                                      ViInt32   channelSymbolRate,
                                      ViInt32   lastCodewordLength,
                                      ViInt32   guardTimeSize,
                                      ViInt32   minislotSize);

ViStatus rsdsa_ConfigureCMRangingD30AFECParameter(ViSession instrSession,
                                                  ViInt32   FECParameterT,
                                                  ViInt32   FECParameterK);

ViStatus rsdsa_ConfigureCMRangingD30APreamble(ViSession instrSession,
                                              ViString  pattern,
                                              ViInt32   preambleLength,
                                              ViInt32   offset,
                                              ViInt32   type);

ViStatus rsdsa_ConfigureCMRangingD30AScrambler(ViSession instrSession,
                                               ViBoolean enabled,
                                               ViString  seed);

ViStatus rsdsa_ConfigureCMRangingD30AReedSolomon(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViInt32   interleaverDepth,
                                                 ViInt32   interleaverBlockSize);

ViStatus rsdsa_ConfigureCMRangingD31U(ViSession instrSession,
                                      ViReal64  frequency,
                                      ViInt32   rollOff,
                                      ViInt32   cyclicPrefixLength,
                                      ViInt32   symbolsPerFrame,
                                      ViInt32   subcarrierSpacing);

ViStatus rsdsa_ConfigureCMRangingD31URandomizer(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViString  seed);

ViStatus rsdsa_ConfigureCMRangingD31UExclusionBand(ViSession instrSession,
                                                   ViInt32   band,
                                                   ViInt32   start,
                                                   ViInt32   width);

ViStatus rsdsa_ConfigureCMRangingD31UWidebandProbe(ViSession instrSession,
                                                   ViInt32   startSubcarrier,
                                                   ViInt32   subcarrierSkip,
                                                   ViBoolean staggerEnabled,
                                                   ViInt32   symbolInFrame);

ViStatus rsdsa_ConfigureCMRangingD31UBWRequest(ViSession instrSession,
                                               ViInt32   numberOfMinislots,
                                               ViInt32   subslot);

ViStatus rsdsa_ConfigureCMRangingD31UMinislotMode(ViSession instrSession,
                                                  ViInt32   configurationMode);

ViStatus rsdsa_ConfigureCMRangingD31UMinislotConfigurationFile(
                            ViSession instrSession,
                            ViString  configurationFile);

ViStatus rsdsa_ConfigureCMRangingD31UMinislotManual(ViSession instrSession,
                                                    ViInt32   constellation,
                                                    ViInt32   pilotPattern,
                                                    ViInt64   startingMinislot,
                                                    ViInt32   numberOfMinislots);

ViStatus rsdsa_ConfigureCMRangingD31UActiveSubcarrier(ViSession instrSession,
                                                      ViInt32   firstActiveSubcarrier,
                                                      ViInt32   lastActiveSubcarrier);

ViStatus rsdsa_ConfigureCMRangingD31UInitialRanging(ViSession instrSession,
                                                    ViInt32   minislotInFrame,
                                                    ViInt32   numberOfMinislots,
                                                    ViInt32   numberOfSubcarriers);

ViStatus rsdsa_ConfigureCMRangingD31UInitialRangingPreamble(ViSession instrSession,
                                                            ViString  pattern,
                                                            ViInt32   length,
                                                            ViInt32   offset);

ViStatus rsdsa_ConfigureCMRangingD31UFineRanging(ViSession instrSession,
                                                 ViInt32   startingMinislotInFrame,
                                                 ViInt32   numberOfMinislots,
                                                 ViInt32   numberOfSubcarriers);

ViStatus rsdsa_ConfigureCMRangingD31UFineRangingPreamble(ViSession instrSession,
                                                         ViString  pattern,
                                                         ViInt32   offset);

ViStatus rsdsa_ConfigureCMRangingSFPOutputEnabled(ViSession instrSession,
                                                  ViBoolean enabled);

ViStatus rsdsa_ConfigureCMRangingSFPIUCUDPPort(ViSession instrSession,
                                               ViInt32   iuc_ofdma,
                                               ViBoolean enabled,
                                               ViInt32   udpPort);

ViStatus rsdsa_CMRangingUpstreamAutoLevel(ViSession instrSession);

ViStatus rsdsa_ConfigureCMRangingUpstream(ViSession instrSession,
                                          ViReal64  attenuation,
                                          ViBoolean preamplifierEnabled);

ViStatus rsdsa_RFInputDownstreamAutomaticSettings(ViSession instrSession);

ViStatus rsdsa_ConfigureRFInputDownstreamSettings(ViSession instrSession,
                                                  ViReal64  attenuation,
                                                  ViReal64  offsetLevel);

ViStatus rsdsa_ConfigureRFInputDownstreamPreamplifierEnabled(ViSession instrSession,
                                                             ViBoolean enabled);

ViStatus rsdsa_RFInputUpstreamAutomaticSettings(ViSession instrSession);

ViStatus rsdsa_ConfigureRFInputUpstreamSettings(ViSession instrSession,
                                                ViReal64  attenuation,
                                                ViReal64  offsetLevel);

ViStatus rsdsa_ConfigureRFInputUpstreamPreamplifierEnabled(ViSession instrSession,
                                                           ViBoolean enabled);

ViStatus rsdsa_RFInputOptimizeMER(ViSession instrSession);

ViStatus rsdsa_ConfigureTriggerInputEnabled(ViSession instrSession,
                                            ViBoolean enabled);

ViStatus rsdsa_ConfigureTriggerInputOffset(ViSession instrSession,
                                           ViReal64  offset);

ViStatus rsdsa_ConfigureTriggerInputSlope(ViSession instrSession,
                                          ViInt32   slope);

ViStatus rsdsa_ConfigureTriggerInputPolarity(ViSession instrSession,
                                             ViInt32   polarity);

ViStatus rsdsa_RestartTriggerInput(ViSession instrSession);

ViStatus rsdsa_GenerateTriggerInputPulse(ViSession instrSession);

ViStatus rsdsa_ConfigureTriggerOutputEnabled(ViSession instrSession,
                                             ViBoolean enabled);

ViStatus rsdsa_ConfigureTriggerOutputSettings(ViSession instrSession,
                                              ViReal64  offset,
                                              ViInt32   polarity,
                                              ViReal64  pulseLength,
                                              ViReal64  cyclePeriod);

ViStatus rsdsa_ConfigureInstrumentMode(ViSession instrSession,
                                       ViInt32   instrumentMode);

ViStatus rsdsa_QueryInstrumentModeStatus(ViSession instrSession,
                                         ViInt32   bufferSize,
                                         ViChar    instrumentActiveModes[],
                                         ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureSystemFirmwareUpdate(ViSession instrSession,
                                             ViString  firmwareUpdate);

ViStatus rsdsa_SetSystemDateTime(ViSession instrSession,
                                 ViInt32   year,
                                 ViInt32   month,
                                 ViInt32   day,
                                 ViInt32   hour,
                                 ViInt32   minute,
                                 ViInt32   second);

ViStatus rsdsa_QuerySystemDateTime(ViSession instrSession,
                                   ViInt32*  year,
                                   ViInt32*  month,
                                   ViInt32*  day,
                                   ViInt32*  hour,
                                   ViInt32*  minute,
                                   ViInt32*  second);

ViStatus rsdsa_SystemReboot(ViSession instrSession);

ViStatus rsdsa_SystemShutdown(ViSession instrSession);

ViStatus rsdsa_ConfigureSystemPreset(ViSession instrSession,
                                     ViInt32   systemPreset);

ViStatus rsdsa_SystemPresetStore(ViSession instrSession);

ViStatus rsdsa_ConfigureSystemRemoteDesktop(ViSession instrSession,
                                            ViBoolean remoteDesktop);

ViStatus rsdsa_ConfigurePowerUnits(ViSession instrSession,
                                   ViInt32   powerUnits);

ViStatus rsdsa_QueryMemoryFileCatalogLength(ViSession instrSession,
                                            ViInt32*  catalogLength);

ViStatus rsdsa_ListCurrentDirectoryFiles(ViSession instrSession,
                                         ViInt32*  usedBytes,
                                         ViInt32*  freeBytes,
                                         ViInt32   bufferSize,
                                         ViChar    fileCatalog[],
                                         ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureMemoryCurrentDirectory(ViSession instrSession,
                                               ViString  currentDirectory);

ViStatus rsdsa_CopyFileOrDirectory(ViSession instrSession,
                                   ViString  source,
                                   ViString  destination);

ViStatus rsdsa_MoveFileOrDirectory(ViSession instrSession,
                                   ViString  source,
                                   ViString  destination);

ViStatus rsdsa_DeleteFile(ViSession instrSession,
                          ViString  filename);

ViStatus rsdsa_QueryMemoryDirectoryCatalogLength(ViSession instrSession,
                                                 ViInt32*  directoryCatalogLength);

ViStatus rsdsa_ListCurrentDirectorySubdirectories(ViSession instrSession,
                                                  ViInt32*  usedBytes,
                                                  ViInt32*  freeBytes,
                                                  ViInt32   bufferSize,
                                                  ViChar    fileCatalog[],
                                                  ViInt32*  bytesRead);

ViStatus rsdsa_CreateDirectory(ViSession instrSession,
                               ViString  directory);

ViStatus rsdsa_DeleteDirectory(ViSession instrSession,
                               ViString  directory);

ViStatus rsdsa_SaveConfigurationFile(ViSession instrSession,
                                     ViString  filename);

ViStatus rsdsa_LoadConfigurationFile(ViSession instrSession,
                                     ViString  filename);

ViStatus rsdsa_LoadFactoryConfiguration(ViSession instrSession);

ViStatus rsdsa_SaveScreenCapture(ViSession instrSession,
                                 ViString  filename);

ViStatus rsdsa_SaveIQConstellationData(ViSession instrSession,
                                       ViString  filename);

ViStatus rsdsa_StoreTraceData(ViSession instrSession,
                              ViInt32   trace,
                              ViString  filename);

ViStatus rsdsa_EjectUSB(ViSession instrSession,
                        ViInt32   USBDevice);

ViStatus rsdsa_StatusPreset(ViSession instrSession);

ViStatus rsdsa_QueryEventStatusRegister(ViSession instrSession,
                                        ViInt32*  eventStatusRegister);

ViStatus rsdsa_ConfigureOperationEnableRegister(ViSession instrSession,
                                                ViInt32   operationEnableRegister);

ViStatus rsdsa_QueryOperationalStatusRegister(ViSession instrSession,
                                              ViInt32*  operationalStatusRegister);

ViStatus rsdsa_QueryOperationEventRegister(ViSession instrSession,
                                           ViInt32*  operationEventRegister);

ViStatus rsdsa_ConfigureQuestionableEnableRegister(ViSession instrSession,
                                                   ViInt32   questionableEnableRegister);

ViStatus rsdsa_QueryQuestionableStatusRegister(ViSession instrSession,
                                               ViInt32*  questionableStatusRegister);

ViStatus rsdsa_QueryQuestionableEventRegister(ViSession instrSession,
                                              ViInt32*  questionableEventRegister);

ViStatus rsdsa_SystemNetworkReset(ViSession instrSession);

ViStatus rsdsa_ConfigureSystemNetwork(ViSession instrSession,
                                      ViString  IPAddress,
                                      ViString  subnet,
                                      ViString  gateway);

ViStatus rsdsa_ConfigureSystemNetworkType(ViSession instrSession,
                                          ViInt32   networkType);

ViStatus rsdsa_QuerySystemDSAMACAddress(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    MACAddress[],
                                        ViInt32*  bytesRead);

ViStatus rsdsa_ConfigureSystemSFPPlusIPAddress(ViSession instrSession,
                                               ViString  IPAddress);

ViStatus rsdsa_ConfigureSystemSFPPlusSubnetMask(ViSession instrSession,
                                                ViString  subnetMask);

ViStatus rsdsa_ConfigureSystemSFPPlusGateway(ViSession instrSession,
                                             ViString  gateway);

ViStatus rsdsa_ConfigureSystemSFPPlusDestinationIPAddress(ViSession instrSession,
                                                          ViString  destinationIPAddress);

ViStatus rsdsa_ConfigureSystemSFPPlusDataRate(ViSession instrSession,
                                              ViInt32   dataRate);

ViStatus rsdsa_ConfigureSystemSFDNetworkOption(ViSession instrSession,
                                               ViInt32   networkOption);

ViStatus rsdsa_ConfigureSystemSFDHostName(ViSession instrSession,
                                          ViString  hostName);

ViStatus rsdsa_ConfigureSystemSFDIPAddress(ViSession instrSession,
                                           ViString  IPAddress);

ViStatus rsdsa_WriteFromFileToInstrument(ViSession instrSession,
                                         ViString  source,
                                         ViString  destination);

ViStatus rsdsa_ReadToFileFromInstrument(ViSession instrSession,
                                        ViString  source,
                                        ViString  destination);

ViStatus rsdsa_SendSoftwareTrigger(ViSession instrSession);

ViStatus rsdsa_Reset(ViSession instrSession);

ViStatus rsdsa_SelfTest(ViSession instrSession,
                        ViInt16*  selfTestResult,
                        ViChar    selfTestMessage[]);

ViStatus rsdsa_RevisionQuery(ViSession instrSession,
                             ViChar    instrumentDriverRevision[],
                             ViChar    firmwareRevision[]);

ViStatus rsdsa_error_query(ViSession instrSession,
                           ViInt32*  errorCode,
                           ViChar    errorMessage[]);

ViStatus rsdsa_error_message(ViSession instrSession,
                             ViStatus  errorCode,
                             ViChar    errorMessage[]);

ViStatus rsdsa_ConfigureErrorChecking(ViSession instrSession,
                                      ViBoolean optionChecking,
                                      ViBoolean rangeChecking,
                                      ViBoolean statusChecking);

ViStatus rsdsa_ClearStatus(ViSession instrSession);

ViStatus rsdsa_IDQueryResponse(ViSession instrSession,
                               ViInt32   bufferSize,
                               ViChar    IDQueryResponse[],
                               ViInt32*  returnedValues);

ViStatus rsdsa_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus rsdsa_QueryOPC(ViSession instrSession,
                        ViInt32*  OPC);

ViStatus rsdsa_SetOPCTimeout(ViSession instrSession,
                             ViUInt32  OPCTimeout);

ViStatus rsdsa_GetOPCTimeout(ViSession instrSession,
                             ViInt32*  timeout);

ViStatus rsdsa_SetVISATimeout(ViSession instrSession,
                              ViUInt32  VISATimeout);

ViStatus rsdsa_GetVISATimeout(ViSession instrSession,
                              ViUInt32* VISATimeout);

ViStatus rsdsa_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                           ViBoolean autoSystErrQuery);

ViStatus rsdsa_ConfigureMultiThreadLocking(ViSession instrSession,
                                           ViBoolean multiThreadLocking);

ViStatus rsdsa_ConfigureAttributeDataTypeMismatchReporting(ViSession instrSession,
                                                           ViBoolean state);

ViStatus rsdsa_GetError(ViSession instrSession,
                        ViStatus* code,
                        ViInt32   bufferSize,
                        ViChar    description[]);

ViStatus rsdsa_ClearError(ViSession instrSession);

ViStatus rsdsa_WriteInstrData(ViSession instrSession,
                              ViString  writeBuffer);

ViStatus rsdsa_QueryViBoolean(ViSession  instrSession,
                              ViString   command,
                              ViBoolean* value);

ViStatus rsdsa_QueryViInt32(ViSession instrSession,
                            ViString  command,
                            ViInt32*  value);

ViStatus rsdsa_QueryViReal64(ViSession instrSession,
                             ViString  command,
                             ViReal64* value);

ViStatus rsdsa_QueryViString(ViSession instrSession,
                             ViString  command,
                             ViInt32   bufferSize,
                             ViChar    value[]);

ViStatus rsdsa_SetAttributeViInt32(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt32   attributeValue);

ViStatus rsdsa_SetAttributeViInt64(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt64   attributeValue);

ViStatus rsdsa_SetAttributeViReal64(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViReal64  attributeValue);

ViStatus rsdsa_SetAttributeViString(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViString  attributeValue);

ViStatus rsdsa_SetAttributeViBoolean(ViSession instrSession,
                                     ViString  repeatedCapabilityName,
                                     ViAttr    attributeID,
                                     ViBoolean attributeValue);

ViStatus rsdsa_GetAttributeViInt32(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt32*  attributeValue);

ViStatus rsdsa_GetAttributeViInt64(ViSession instrSession,
                                   ViString  repeatedCapabilityName,
                                   ViAttr    attributeID,
                                   ViInt64*  attributeValue);

ViStatus rsdsa_GetAttributeViReal64(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViReal64* attributeValue);

ViStatus rsdsa_GetAttributeViString(ViSession instrSession,
                                    ViString  repeatedCapabilityName,
                                    ViAttr    attributeID,
                                    ViInt32   bufferSize,
                                    ViChar    attributeValue[]);

ViStatus rsdsa_GetAttributeViBoolean(ViSession  instrSession,
                                     ViString   repeatedCapabilityName,
                                     ViAttr     attributeID,
                                     ViBoolean* attributeValue);

ViStatus rsdsa_GetAttributeRepCapName(ViSession instrSession,
                                      ViUInt32  attributeID,
                                      ViString  repCapNameID,
                                      ViInt32   index,
                                      ViInt32   bufferSize,
                                      ViChar    repCapName[]);

ViStatus rsdsa_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    repeatedCapabilityIds[]);

ViStatus rsdsa_GetAttributeRepeatedCapabilityIdNames(ViSession instrSession,
                                                     ViUInt32  attributeID,
                                                     ViString  repeatedCapabilityId,
                                                     ViInt32   bufferSize,
                                                     ViChar    repeatedCapabilityIdNames[]);

ViStatus rsdsa_close(ViSession instrSession);

ViStatus rsdsa_CheckStatus(ViSession instrSession);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSDSA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSDSA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)
#define RSDSA_WARN_DATA_NOT_AVAILABLE      (RS_CLASS_WARN_BASE + 0x0003L)

#define RSDSA_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSDSA_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSDSA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)
#define RSDSA_ERROR_RELIABILITY_INDICATOR  (RS_SPECIFIC_ERROR_BASE + 0x0004L)

#define RSDSA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSDSA_WARNMSG_OVER_RANGE           "Measurement overrange"
#define RSDSA_WARNMSG_DATA_NOT_AVAILABLE   "Measurement data not available"

#define RSDSA_ERRMSG_INCORRECT_STATE       "Instrument in incorrect state"
#define RSDSA_ERRMSG_INCORRECT_RESPONSE    "Response incorrect; report problem to R&S support!"
#define RSDSA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"
#define RSDSA_ERRMSG_RELIABILITY_INDICATOR "Reliability Indicator error"

#define RSDSA_ERROR_CODES_AND_MSGS \
    {RSDSA_ERROR_INCORRECT_STATE,   RSDSA_ERRMSG_INCORRECT_STATE},\
    {RSDSA_ERROR_INCORRECT_RESPONSE,     RSDSA_ERRMSG_INCORRECT_RESPONSE},\
    {RSDSA_WARN_DATA_NOT_AVAILABLE,    RSDSA_WARNMSG_DATA_NOT_AVAILABLE},\
    {RSDSA_ERROR_MAX_TIME_EXCEEDED,    RSDSA_ERRMSG_MAX_TIME_EXCEEDED},\
    {RSDSA_ERROR_RELIABILITY_INDICATOR,    RSDSA_ERRMSG_RELIABILITY_INDICATOR}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSDSA_HEADER */
